/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */



#include <QApplication>
#include "AnimPacket.h"
#include <iostream>
#include <sstream>

namespace netanim {

#define NS_FATAL_ERROR(x) std::cout<<x<<std::endl;
#define NS_LOG_INFO(x)
#define NS_LOG_UNCOND(x)
uint32_t AnimPacket::greqId = 0;

AnimPacket::AnimPacket ()
{

}

AnimPacket::~AnimPacket ()
{
}

uint32_t AnimPacket::getnewreqId ()
{
  return greqId++;
}

void AnimPacket::SetRequestType (uint8_t reqType)
{
  if ((reqType != 0) && (reqType != 1))
    {
      NS_FATAL_ERROR ("Invalid Request Type");
    }
  m_reqType = (reqType)? true : false;
  NS_LOG_INFO ("Request Type:" << (m_reqType? "true":"false"));
}

void AnimPacket::SetRequestId (void * reqId)
{
  m_reqId = *(reinterpret_cast <uint32_t*> (reqId));
  NS_LOG_INFO ("Request Id:" << m_reqId);
}

void AnimPacket::SetCommandId (void * commandId)
{
  m_commandId = *(reinterpret_cast <uint32_t*> (commandId));
  NS_LOG_INFO ("Command Id:" << m_commandId);
}

void AnimPacket::SetPayloadLength (void * payloadLength)
{
  m_payloadLength = *(reinterpret_cast <uint32_t*> (payloadLength));
  NS_LOG_INFO ("PayloadLength:" << m_payloadLength);
}


void AnimPacket::SetPayload (void * data)
{
  uint32_t * p =  reinterpret_cast <uint32_t *> (data);
  for (uint32_t i = 0; i < m_payloadLength; i++)
    {

      m_payload.push_back (static_cast <uint32_t> (*(p + i)));
    }
  NS_LOG_INFO ("Data:" << GetString (m_payload));
}


uint32_t AnimPacket::GetTotalLength ()
{
  uint32_t totalLength = 0;

  totalLength += 4; // Magic Start
  totalLength += 1;  // Request Type
  totalLength += 4; // Request Id
  totalLength += 4; // Command Id
  totalLength += 4; // Payload Length field
  totalLength += (m_payloadLength * 4);
  totalLength += 4; // Magic End
  return totalLength;
}

std::string AnimPacket::GetHexString (uint32_t value)
{
  std::stringstream ss;
  ss << "0x" << std::hex << (uint32_t) value;
  return ss.str ();
}


std::string AnimPacket::GetHexString (std::vector <uint32_t> values)
{
  std::stringstream ss;
  ss << "0x" << std::hex;
  for (uint32_t i = 0; i <values.size (); i++)
    {
      ss << (int) values[i];
    }
  return ss.str ();
}

std::string AnimPacket::GetString (std::vector <uint32_t> values)
{
  std::stringstream ss;
  ss << "0x";
  for (uint32_t i = 0; i <values.size (); i++)
    {
      uint8_t * p = reinterpret_cast <uint8_t *> (&values[i]);
      ss << *p << *(p+1) << *(p+2) << *(p+3) ;
    }
  return ss.str ();
}


uint8_t AnimPacket::GetRequestTypeAsByte ()
{
  return (m_reqType ? 1 : 0);
}

uint32_t * AnimPacket::GetRequestId ()
{
  return &m_reqId;
}

uint32_t * AnimPacket::GetCommandId ()
{
  return &m_commandId;
}

uint32_t * AnimPacket::GetPayloadLength ()
{
  return &m_payloadLength;
}

std::vector <uint32_t> * AnimPacket::GetPayload ()
{
  return &m_payload;
}

void AnimPacket::AppendPayload (uint32_t value)
{
  m_payload.push_back (value);
  m_payloadLength = m_payload.size ();
}

void AnimPacket::Print ()
{
  NS_LOG_UNCOND ("Packet details: Type:" << (m_reqType? "Request":"Response")
                 << " Request Id:" << m_reqId
                 << " Command Id:" << m_commandId
                 << " Data Length:" << m_payloadLength
                 << " Data:" << GetString (m_payload));
}
} //namespace netanim


