/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */


#ifndef _ANIMPACKET_H_
#define _ANIMPACKET_H_

#include <vector>
#include <stdint.h>

namespace netanim {

#define VALIDATELENGTH ValidateLength(__FUNCTION__, ret, fieldLength)

class AnimPacket {

public:
  AnimPacket ();
  ~AnimPacket ();
  static uint32_t greqId; // Unique Request Id
  static uint32_t getnewreqId ();
  void SetRequestType (uint8_t reqType);
  void SetRequestId (void * reqId);
  void SetCommandId (void * commandId);
  void SetPayloadLength (void * dataLength);
  void SetPayload (void * data);
  void AppendPayload (uint32_t value);
  uint8_t GetRequestTypeAsByte ();
  uint32_t * GetRequestId ();
  uint32_t * GetCommandId ();
  uint32_t * GetPayloadLength ();
  uint32_t GetTotalLength ();
  std::vector <uint32_t> * GetPayload ();
  std::string GetHexString (uint32_t value);
  std::string GetHexString (std::vector <uint32_t> values);
  std::string GetString (std::vector <uint32_t> values);
  void Print ();
private:
  bool m_reqType;                  // True if Request , False if Response
  uint32_t m_reqId;                // Id to match Request and Responses
  uint32_t m_commandId;            // Command Id
  uint32_t m_payloadLength;        // Payload Length in 32 bit chunks
  std::vector <uint32_t> m_payload;// Payload

}; // class AnimPacket

} // namespace netanim
#endif
