/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Define the mobility class for Animator

#ifndef __MOBILITY__H__
#define __MOBILITY__H__


#include "Common/CommonDefs.h"
#include "Points.h"
#include <map>
#include <set>

using namespace std;
namespace netanim {

// Mobility is maintained as a set, sorted by start time stored
// in the Node object. The startTime/endTimes cannot overlap with
// other mobility items in the list
class Mobility {
public:
  Mobility(Point3 sp, Time_t st, Point3 ep, Time_t et);
public:
  Point3 startPosition;
  Time_t startTime;
  Point3 endPosition;
  Time_t endTime;
};

// Define the less than operator for the mobility bject
class mobility_less
{
  public:
  mobility_less(){}
  bool operator()(const netanim::Mobility& l, const netanim::Mobility& r) const
  {
    return l.startTime < r.startTime;
  }
};

typedef std::set<netanim::Mobility, netanim::mobility_less> MobilitySet_t;

} // namespace netanim

#endif

