/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Define the model for the Network Animation Viewer

#ifndef __ANIM_MODEL_H__
#define __ANIM_MODEL_H__

#include <vector>
#include <map>
#include <pthread.h>

#include <QRectF>

#include "Node.h"
#include "WiredLink.h"
#include "Packet.h"
#include "MainInfo/MainInfo.h"

namespace netanim {
class NetView;
class NetWiredLink;
#define nil 0

class NetModel 
{ // The interface to add things to the model
public:
  NetModel(MainInfo *mi);
  // History buffer deprecate, no longer need the copy constructor
  //NetModel(const NetModel&); // We need a copy constructor for the hist buffer
  void SetView(NetView*);
  // Time Management
  //  void      SetTime(Time_t); // Moved to view
  // Node, Packet, and Link management
  void      AddNode(const Node&);
  Node      GetNode(LpId_t, NodeId_t);
  QRectF    GetBoundingRect();    // Get the bounding rect with no border
  QRect     GetViewRect();         // Get the view rect with a border
  void      AddPacket(const Packet&);
  void      AddLink(const WiredLink&);
  void      AddMobility(LpId_t, NodeId_t, const Mobility&);
  // Accessors
  // Each accessor locks the mutex, so caller must call Release when done
  NodeMap_t& GetNodes();
  void       ReleaseNodes();
  PktMap_t&  GetPackets();
  void       ReleasePackets();
  LinkVec_t& GetLinks();
  void       ReleaseLinks();
public:
  NetView*  view;
  // Topology members
private:
  NodeMap_t nodes;
  PktMap_t  packets;
  LinkVec_t links;
  MainInfo * mainInfo;
public:
  QRectF    minMaxCoord;          // The bounding box for the nodes
  QRectF    defaultMinMax;        // As specified on the <topology> node
  // Mutexes for access to the various network object containers
  pthread_mutex_t nodeMutex;
  pthread_mutex_t linkMutex;
  pthread_mutex_t packetMutex;
};

} // namespace netanim
#endif
