/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Network Animator Node Object

#ifndef __NET_NODE__
#define __NET_NODE__

#include <algorithm>
#include <vector>
#include <string>
#include <map>

#include <QPointF>
#include <QRectF>
#include <QImage>
#include "Common/CommonDefs.h"
#include "Mobility.h"

using namespace std;
namespace netanim {


class WiredLink;

// Define the key for the node map
typedef std::pair<LpId_t, NodeId_t> KeyPair_t;

// Define the less than operator for the key pair
class key_less
{
  public:
  key_less(){}
  bool operator()(const KeyPair_t& l, const KeyPair_t& r) const 
  {
    return l.first < r.first ||
      (l.first == r.first && l.second < r.second);
  }
};

// Define the node map
class Node;
typedef std::map<KeyPair_t, Node, key_less> NodeMap_t;

class Node {
public:
  typedef enum 
  { // How to represent this node
    CIRCLE, SQUARE, CUSTOM
  } NodeShape_t;
  Node();
  Node(LpId_t, NodeId_t, Meters_t, Meters_t, Meters_t, int = 3);
  Node(LpId_t, NodeId_t, const Point3&, int = 3);
  Node(LpId_t, NodeId_t, const Point2&, int = 3);
  void SetShape(NodeShape_t);
  void Show();
  void Hide();
  WiredLink FindLink(NodeId_t);   // Find link with specified peer
  void AddLink(const WiredLink&);
  void AddMobility(const netanim::Mobility&);
  QPointF GetLoc2d(Time_t) const;
public:
  std::vector<WiredLink> links; // All wired links associated w/ this node
  LpId_t   lpId; // Logical Process Identifier
  NodeId_t id;   // Node id
  // Position
private:
  Point3        position;
public:
  netanim::MobilitySet_t mobility;
  NodeShape_t   shape;
  bool          visible;
  bool          exists;
  std::string   imageName;
  double        imageScale;
  QImage*       image;
  int           radius;  // In pixels
  std::string   color;   // If color specified (not used if image)
public:
};

} // namespace netanim

#endif
