/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Network Animator Packet class

#ifndef __NET_PACKET_H__
#define __NET_PACKET_H__

#include <map>
#include "Node.h"

using namespace std;

namespace netanim {

class Packet;

// Define the sorted map of packets
typedef std::multimap<Time_t, Packet> PktMap_t;

// For broadcast-based networkd (ie. wireless) each packet can
// have multiple rx events at different times.
class PacketRx 
{
public:
  PacketRx(Time_t, Time_t, LpId_t, NodeId_t, const std::string&);
public:
  Time_t   fbRx;      // first bit rx time
  Time_t   lbRx;      // last bit rx time
  netanim::LpId_t   rxLp;      // Receiving lp
  NodeId_t rxId;      // Receiving Node
  std::string color;  // Color for receiving node
};

class Packet {
public:
  Packet();
  Packet(LpId_t, NodeId_t,
         bool,
         Time_t, Time_t);
  void AddRx(Time_t, Time_t, LpId_t, NodeId_t, const std::string&);
  Time_t   fbTx;    // first bit tx time
  Time_t   lbTx;    // last bit tx time
  bool     wired;   // true if wired
  netanim::LpId_t   txLp;    // Transmitting lp
  NodeId_t txId;    // Transmitting node
  bool     valid;   // True if packet exists
  std::vector<PacketRx> rxVec;  // Vector of receipt events
  std::string           color;  // If color specified
  bool forwardshown;
public:
};

} // namespace netanim

#endif
