/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3
#ifndef APPLICATIONSTAB_H
#define APPLICATIONSTAB_H

#include "MainInfo/MainInfo.h"
#include <AnimTab/animtab.h>
#include <QtGui>
#include "ns3/core-module.h"
#include "ns3/applications-module.h"
#include "ns3/sgi-hashmap.h"
#include <map>
#include "Graphing/TwodGrapher.h"

using namespace std;

namespace netanim {
class ApplicationsTab;


class Ipv4AddressHashNetAnim : public std::unary_function<ns3::Ipv4Address, size_t> {
public:
  size_t operator() (ns3::Ipv4Address const &x) const
  {
    return x.Get();
  }
  size_t operator() (ns3::Ipv4Address const &x, ns3::Ipv4Address const &y) const
  {
    return x < y;
  }
};


class InetSocketAddressHashNetAnim {
public:

  size_t operator() (ns3::InetSocketAddress const &x, ns3::InetSocketAddress const &y) const
  {
      return ((x.GetIpv4().Get() < y.GetIpv4().Get()) ||
              (x.GetPort() < y.GetPort()));

  }
};



class ApplicationCombo3 : public QWidget
{
    Q_OBJECT
public:
    void OpenFile(QPointer <QFile> f);
    void AppendToFile(QPointer <QFile> f,QString str);
    QWidget * getCentralWidget();
    ApplicationCombo3 (QString appName, QString traceName, QString auxString);
    TwoDGrapherView * getTwodGrapher();
    void incStat();
    void setStat(double t);
    QPointer <QLabel> pCounterLabel;
    void timeToPlot();
private:
    uint32_t theCounter;
    uint32_t plotCounter;
    QWidget * centralWidget;
    TwoDGrapherView * pgrapher;
    bool plotOn;
    QPointer <QPushButton> plotButton;

    QString m_appName;
    QString m_traceName;
    QString m_auxString;
    QPointer <QHBoxLayout> hbox0 ;
    QPointer <QFile> file;

private slots:
     void plotSlot();

};

class ApplicationCombo2 : public QWidget
{

    Q_OBJECT
    public:
    ApplicationCombo2 (ns3::Ptr<ns3::Node> node, ns3::Ptr<ns3::Application> pApp);
    QWidget * getCentralWidget();


    private:
    QPointer <QVBoxLayout> vbox0;
    QPointer <QLabel> appName;
    std::string sAppName;
    std::vector <QPointer <QLabel> > vCounters;
    ns3::Ptr<ns3::Node> m_node;
    ns3::Ptr<ns3::Application> m_app;
    QWidget * centralWidget;


    // Node statistics


    // Packet Sink stats
    uint32_t PacketSink_Rx_stat;

    // Bulk Send stats
    uint32_t BulkSend_Tx_stat;

    // OnOff stats

    uint32_t OnOff_Tx_stat;

    // UDP echo client stats

    uint32_t UdpEchoClient_stat;

    // v4Ping Stats;

    ns3::Time v4Ping_Rtt_stat;

public:
    void ConnectTraces();
    void UpdateApplication();


    void ConnectPacketSinkTraces();
    void UpdatePacketSinkWidget();
    void PacketSink_Rx_cb ( ns3::Ptr <const ns3::Packet> p, const ns3::Address & addr);
    typedef std::map<ns3::Ipv4Address,uint32_t,Ipv4AddressHashNetAnim> mIpv4Address_t;
    mIpv4Address_t mIpv4AddressCounters;


    typedef std::map<ns3::InetSocketAddress,uint32_t,InetSocketAddressHashNetAnim> mInetSocketAddress_t;
    mInetSocketAddress_t mInetSocketAddress;

    typedef std::map<ns3::Ipv4Address, ApplicationCombo3 * , Ipv4AddressHashNetAnim> mIpv4Addres_Counters_t;
    mIpv4Addres_Counters_t mIpv4Addres_Plots;


    typedef std::map<ns3::InetSocketAddress, ApplicationCombo3 * , InetSocketAddressHashNetAnim> mInetSocketAddressCounters_t;
    mInetSocketAddressCounters_t mInetSocketAddress_Plots;
    mInetSocketAddressCounters_t mInetSocketAddressTput_Plots;


    typedef std::map<QString, ApplicationCombo3 *> mTraceName_t;
    mTraceName_t mTraceNameCounters;


    void ConnectBulkSendTraces();
    void UpdateBulkSendWidget();
    void BulkSend_Tx_cb ( ns3::Ptr <const ns3::Packet> p);

    void ConnectOnOffTraces();
    void UpdateOnOffWidget();
    void OnOff_Tx_cb ( ns3::Ptr <const ns3::Packet> p);

    void ConnectUdpEchoClientTraces();
    void UpdateUdpEchoClientWidget();
    void UdpEchoClient_Tx_cb (ns3::Ptr <const ns3::Packet> p);

    void Connectv4PingTraces();
    void Updatev4PingWidget();
    void v4Ping_Rtt_cb (ns3::Time t);

    void ResetStats();
    void Reset_PacketSinkStats();
    void Reset_BulkSendStats();
    void Reset_OnOffStats();
    void Reset_UDPEchoClientStats();
    void Reset_v4PingStats();


};


class ApplicationsCombo : public QWidget
{
    Q_OBJECT
    public:

        ApplicationsCombo(ApplicationsTab * at, uint32_t nodeId,MainInfo *);
        ~ApplicationsCombo();
        QPointer <QPushButton> getButton();
        QWidget * getCentralWidget();
        bool isactive();
        void SetLogtoFile(bool logit);
        void UpdateApplications();
    private:

        QPointer <QGroupBox> gb;
        std::vector < QPointer <ApplicationCombo2> > vApplicationCombos;

        bool logtoFile;

        QPointer <QPushButton> nodeButton;
        QPointer <QVBoxLayout> vbox0;
        QPointer <QLabel> nodeLabel;
        QPointer <QLabel> NApplicationsLabel;

        ns3::Ptr <ns3::Node> n;
        uint32_t NApplications;


        ApplicationsTab * m_at;
        uint32_t m_nodeId;
        bool active;

        QWidget * centralWidget;
        MainInfo * mainInfo;


    public:


    public slots:
        void onbuttonToggled ();

};



class ApplicationsTab : public AnimTab
{
    Q_OBJECT
public:
    ApplicationsTab( QObject *parent = 0);
    QWidget * Create (QApplication * app, MainInfo *mi);

    QString GetTabName();
    QWidget * getCentralWidget();
    void onFocus();
    void onLostFocus();
    void NotifyToggled(QPointer <ApplicationsCombo>, uint32_t,bool active,QWidget *);
    void UpdateApplications();
    typedef std::map <uint32_t,QPointer <ApplicationsCombo> > mAType;
    void AdjustGrid();



public slots:


private:
    MainInfo * mainInfo;
    uint32_t numNodes;
    QPointer <QVBoxLayout> vbox0;

    // Hbox 0
    QPointer <QHBoxLayout> hbox0;
    QPointer <QPushButton> logtoFileButton;
    QPointer <QLineEdit> pollLineEdit;
    QPointer <QPushButton> applyButton;
    QPointer <QGroupBox> polling_gb;
    QPointer <QRadioButton> wallClock_rb;
    QPointer <QRadioButton> simulationTime_rb;

    //Polling
    bool polling;
    bool simtimePolling;
    QString pollString;
    void DoSimTimePolling();
    void DoWallClockPolling();
    QPointer <QTimer> wallClocktimer;
    ns3::Timer simtimePollingTimer;

    // Gridbox 2

    QPointer <QGridLayout> gridbox2;
    void AddApplicationCombos();
    ApplicationsCombo * AddApplicationCombo(uint32_t nodeId);
    std::map <uint32_t,QPointer <ApplicationsCombo> > mApplicationsCombo;


    // Nodes Hbox
    QPointer <QHBoxLayout> nodesHBox;
    QPointer <QScrollArea> nodesSA;



    QWidget * centralWidget;





public slots:

    void apply();
    void logtofileslot();
    void wallclockrb_slot(bool state);
    void simtimerb_slot(bool state);



};

} // namespace netanim
#endif // APPLICATIONSTAB_H
#endif  //WITH_NS3
