/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */
#include "CmdDemux.h"
#include <map>

namespace netanim {


#define NS_FATAL_ERROR(x)
#define NS_LOG_LOGIC(x)

std::map <ANIM_CMD,pf> CmdDemux::CmdHandlerRegistry;
bool CmdDemux::RegisterCmdHandler (ANIM_CMD cmd, pf handler)
{
    if (CmdDemux::CmdHandlerRegistry.find (cmd) != CmdDemux::CmdHandlerRegistry.end())
      {
          NS_FATAL_ERROR (cmd << " Already Registered");
          return false;
      }
    CmdDemux::CmdHandlerRegistry[cmd] = handler;
    return true;
}

void CmdDemux::Demux (ANIM_CMD cmd, std::vector <uint32_t> data)
{
    NS_LOG_LOGIC ("Demux:"<<cmd);
    if (CmdDemux::CmdHandlerRegistry.find (cmd) != CmdDemux::CmdHandlerRegistry.end())
      {
          NS_FATAL_ERROR (cmd << " Unknown Command");
      }
    CmdDemux::CmdHandlerRegistry[cmd](data);

}
} //namespace netanim
