#ifdef WITH_NS3
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include "packetfilter.h"
#include "QtNetwork/QHostAddress"

PacketFilter::PacketFilter(bool showMac, bool showIp, bool showPort, QWidget *parent) :
    QWidget(parent),m_showMac(showMac),m_showIp(showIp),m_showPort (showPort)
{


    src_gb = new QGroupBox ("Source");

    src_ip_le = new QLineEdit;
    src_ip_le->setInputMask("000.000.000.000;");
    src_ip_le->setMaximumWidth(150);
    QPointer <QLabel> src_ip_label = new QLabel("IP");
    src_ip_label->setBuddy(src_ip_le);
    src_ip_cb = new QCheckBox ("ANY");
    connect(src_ip_cb,SIGNAL (stateChanged(int)),this,SLOT(src_ip_cb_slot(int)));
    src_ip_cb->setChecked(true);
    src_ip_hbox = new QHBoxLayout;
    src_ip_hbox->addWidget(src_ip_label);
    src_ip_hbox->addWidget(src_ip_le);
    src_ip_hbox->addWidget(src_ip_cb);

    src_port_hbox = new QHBoxLayout;
    src_port_le = new QLineEdit;
    src_port_le->setInputMask("00000;");
    src_port_le->setMaximumWidth(50);


    QPointer <QLabel> src_port_label = new QLabel ("Port");
    src_port_cb = new QCheckBox ("ANY");
    connect(src_port_cb,SIGNAL (stateChanged(int)),this,SLOT(src_port_cb_slot(int)));
    src_port_cb->setChecked(true);
    src_port_label->setBuddy(src_port_le);
    src_port_hbox->addWidget(src_port_label);
    src_port_hbox->addWidget(src_port_le);
    src_port_hbox->addWidget(src_port_cb);


    src_mac_hbox = new QHBoxLayout;
    src_mac_le = new QLineEdit;
    src_mac_le->setInputMask("HH:HH:HH:HH:HH:HH;");
    src_mac_le->setMaximumWidth(150);

    QPointer <QLabel> src_mac_label = new QLabel ("Mac");

    src_mac_cb = new QCheckBox ("ANY");
    connect(src_mac_cb,SIGNAL (stateChanged(int)),this,SLOT(src_mac_cb_slot(int)));
    src_mac_cb->setChecked(true);

    src_mac_label->setBuddy(src_mac_le);
    src_mac_hbox->addWidget(src_mac_label);
    src_mac_hbox->addWidget(src_mac_le);
    src_mac_hbox->addWidget(src_mac_cb);






    src_vbox = new QVBoxLayout;

    if (m_showIp)
    src_vbox->addLayout(src_ip_hbox);
    if (m_showPort)
    src_vbox->addLayout(src_port_hbox);
    if (m_showMac)
    src_vbox->addLayout(src_mac_hbox);
    src_gb->setLayout(src_vbox);






    dst_gb = new QGroupBox ("Destination");


    dst_ip_le = new QLineEdit;
    QPointer <QLabel> dst_ip_label = new QLabel("IP");
    dst_ip_le->setInputMask("000.000.000.000;");
    dst_ip_le->setMaximumWidth(150);

    dst_ip_label->setBuddy(dst_ip_le);

    dst_ip_cb = new QCheckBox ("ANY");
    connect(dst_ip_cb,SIGNAL (stateChanged(int)),this,SLOT(dst_ip_cb_slot(int)));
    dst_ip_cb->setChecked(true);

    dst_ip_hbox = new QHBoxLayout;
    dst_ip_hbox->addWidget(dst_ip_label);
    dst_ip_hbox->addWidget(dst_ip_le);
    dst_ip_hbox->addWidget(dst_ip_cb);

    dst_port_hbox = new QHBoxLayout;
    dst_port_le = new QLineEdit;
    dst_port_le->setInputMask("00000;");
    dst_port_le->setMaximumWidth(50);

    QPointer <QLabel> dst_port_label = new QLabel ("Port");
    dst_port_cb = new QCheckBox ("ANY");
    connect(dst_port_cb,SIGNAL (stateChanged(int)),this,SLOT(dst_port_cb_slot(int)));
    dst_port_cb->setChecked(true);

    dst_port_label->setBuddy(dst_port_le);
    dst_port_hbox->addWidget(dst_port_label);
    dst_port_hbox->addWidget(dst_port_le);
    dst_port_hbox->addWidget(dst_port_cb);


    dst_mac_hbox = new QHBoxLayout;
    dst_mac_le = new QLineEdit;
    dst_mac_le->setInputMask("HH:HH:HH:HH:HH:HH;");
    dst_mac_le->setMaximumWidth(150);

    QPointer <QLabel> dst_mac_label = new QLabel ("Mac");
    dst_mac_cb = new QCheckBox ("ANY");
    connect(dst_mac_cb,SIGNAL (stateChanged(int)),this,SLOT(dst_mac_cb_slot(int)));
    dst_mac_cb->setChecked(true);

    dst_mac_label->setBuddy(dst_mac_le);
    dst_mac_hbox->addWidget(dst_mac_label);
    dst_mac_hbox->addWidget(dst_mac_le);
    dst_mac_hbox->addWidget(dst_mac_cb);



    dst_vbox = new QVBoxLayout;
    if (m_showIp)
    dst_vbox->addLayout(dst_ip_hbox);
    if (m_showPort)
    dst_vbox->addLayout(dst_port_hbox);
    if (m_showMac)
    dst_vbox->addLayout(dst_mac_hbox);
    dst_gb->setLayout(dst_vbox);


    QPointer <QHBoxLayout> applyEditHbox = new QHBoxLayout;
    applyButton = new QPushButton("Apply");
    connect(applyButton,SIGNAL(clicked()),this,SLOT(applyButton_slot()));
    editButton = new QPushButton ("Edit");
    connect(editButton,SIGNAL(clicked()),this,SLOT(editButton_slot()));
    applyEditHbox->addWidget(applyButton);
    applyEditHbox->addWidget(editButton);
    QPointer <QGroupBox> apply_gb = new QGroupBox;
    apply_gb->setLayout(applyEditHbox);

    hbox0 = new QHBoxLayout;
    hbox0->addWidget(src_gb);
    hbox0->addWidget(dst_gb);
    hbox0->addWidget(apply_gb);
    centralWidget = new QWidget;
    centralWidget->setLayout(hbox0);
    applyButton_slot();
}


QWidget * PacketFilter::getCentralWidget()
{
    return centralWidget;

}


bool PacketFilter::IsPortValid(QString port)
{
    if (port == "ANY")
        return true;
    int intPort = port.toInt();
    if (intPort < 0 || intPort > 65535)
    {
        QMessageBox msg;
        msg.setText("Invalid Port:"+port);
        msg.exec();
        return false;
    }
    else
        return true;
}

bool PacketFilter::IsIpValid(QString ip)
{
    if (ip == "ANY")
        return true;
    QHostAddress addr ;
    if (addr.setAddress(ip))
        return true;
    else
    {
        QMessageBox msg;
        msg.setText("Invalid IP:"+ip);
        msg.exec();
        return false;
    }
}

void PacketFilter::applyButton_slot()
{


    if (src_ip_cb->isChecked())
        src_ip_string = "ANY";
    else
        src_ip_string = src_ip_le->text();

    if (src_port_cb->isChecked())
        src_port_string = "ANY";
    else
        src_port_string = src_port_le->text();


    if (src_mac_cb->isChecked())
        src_mac_string = "ANY";
    else
        src_mac_string = src_mac_le->text();



    if (dst_ip_cb->isChecked())
        dst_ip_string = "ANY";
    else
        dst_ip_string = dst_ip_le->text();

    if (dst_port_cb->isChecked())
        dst_port_string = "ANY";
    else
        dst_port_string = dst_port_le->text();


    if (dst_mac_cb->isChecked())
        dst_mac_string = "ANY";
    else
        dst_mac_string = dst_mac_le->text();





    if (!IsPortValid(src_port_string))
    return;
    if (!IsIpValid(src_ip_string))
    return;
    if (!IsIpValid(dst_ip_string))
    return;
    if (!IsPortValid(dst_port_string))
    return;

    src_ip_le->setEnabled(false);
    src_ip_cb->setEnabled(false);
    src_port_le->setEnabled(false);
    src_port_cb->setEnabled(false);
    src_mac_le->setEnabled(false);
    src_mac_cb->setEnabled(false);

    dst_ip_le->setEnabled(false);
    dst_ip_cb->setEnabled(false);
    dst_port_le->setEnabled(false);
    dst_port_cb->setEnabled(false);
    dst_mac_le->setEnabled(false);
    dst_mac_cb->setEnabled(false);

    applyButton->setEnabled(false);
    editButton->setEnabled(true);



}

void PacketFilter::editButton_slot()
{

    if (!src_ip_cb->isChecked())
    src_ip_le->setEnabled(true);
    src_ip_cb->setEnabled(true);
    if (!src_port_cb->isChecked())
    src_port_le->setEnabled(true);
    src_port_cb->setEnabled(true);
    if (!src_mac_cb->isChecked())
    src_mac_le->setEnabled(true);
    src_mac_cb->setEnabled(true);

    if (!dst_ip_cb->isChecked())
    dst_ip_le->setEnabled(true);
    dst_ip_cb->setEnabled(true);
    if (!dst_port_cb->isChecked())
    dst_port_le->setEnabled(true);
    dst_port_cb->setEnabled(true);
    if (!dst_mac_cb->isChecked())
    dst_mac_le->setEnabled(true);
    dst_mac_cb->setEnabled(true);

    applyButton->setEnabled(true);
    editButton->setEnabled(false);

}


void PacketFilter::src_ip_cb_slot(int state)
{
    if (src_ip_cb->isChecked())
    {
       src_ip_le->setEnabled(false);
       src_ip_string = "ANY";
    }
    else
    {
        src_ip_le->setEnabled(true);
        src_ip_string = src_ip_le->text();
    }

}
void PacketFilter::src_port_cb_slot(int state)
{
    if (src_port_cb->isChecked())
    {
       src_port_le->setEnabled(false);
       src_port_string = "ANY";

    }
    else
    {
        src_port_le->setEnabled(true);
        src_port_string = src_port_le->text();


    }


}
void PacketFilter::src_mac_cb_slot(int state)
{
    if (src_mac_cb->isChecked())
    {
       src_mac_le->setEnabled(false);
       src_mac_string = "ANY";

    }
    else
    {
        src_mac_le->setEnabled(true);
        src_mac_string = src_mac_le->text();


    }

}

void PacketFilter::dst_ip_cb_slot(int state)
{

    if (dst_ip_cb->isChecked())
    {
       dst_ip_le->setEnabled(false);
       dst_ip_string = "ANY";

    }
    else
    {
        dst_ip_le->setEnabled(true);
        dst_ip_string = dst_ip_le->text();


    }


}
void PacketFilter::dst_port_cb_slot(int state)
{
    if (dst_port_cb->isChecked())
    {
       dst_port_le->setEnabled(false);
       dst_port_string = "ANY";

    }
    else
    {
        dst_port_le->setEnabled(true);
        dst_port_string = dst_port_le->text();


    }

}
void PacketFilter::dst_mac_cb_slot(int state)
{
    if (dst_mac_cb->isChecked())
    {
       dst_mac_le->setEnabled(false);
       dst_mac_string = "ANY";

    }
    else
    {
        dst_mac_le->setEnabled(true);
        dst_mac_string = dst_mac_le->text();


    }


}


QString PacketFilter::get_src_ip()
{
    return src_ip_string;

}

QString PacketFilter::get_src_port()
{
    return src_port_string;
}

QString PacketFilter::get_src_mac()
{
    return src_mac_string;

}



QString PacketFilter::get_dst_ip()
{
    return dst_ip_string;

}

QString PacketFilter::get_dst_port()
{
    return dst_port_string;
}

QString PacketFilter::get_dst_mac()
{
    return dst_mac_string;

}

bool PacketFilter::pass(ns3::Ptr<const ns3::Packet> p)
{
    ns3::Ptr <ns3::Packet> packet = p->Copy();
    ns3::Ipv4Header iphdr;
    packet->PeekHeader(iphdr);
    std::stringstream ss_srcip;
    ss_srcip << iphdr.GetSource();
    //std::cout << "Source : " << iphdr.GetSource() << " To match:" << src_ip_string.toAscii().data()<<std::endl;
    if ((src_ip_string != "ANY") &&  ( QString(ss_srcip.str().c_str()) != src_ip_string))
        return false;

    std::stringstream ss_dstip;
    ss_dstip << iphdr.GetDestination();
    if ((dst_ip_string != "ANY") &&  ( QString(ss_dstip.str().c_str()) != dst_ip_string))
        return false;

    //std::cout << "Destination : " << iphdr.GetDestination() << " To match:" << dst_ip_string.toAscii().data() <<std::endl;
    return true;

}

#endif
