#ifdef WITH_NS3

#ifndef PACKETFILTER_H
#define PACKETFILTER_H

/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include <QtGui>
#include <QWidget>
#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/internet-module.h"
#include "stdint.h"

class PacketFilter : public QWidget
{
    Q_OBJECT
public:
    explicit PacketFilter(bool showMac, bool showIp, bool showPort, QWidget *parent = 0);
    QWidget * getCentralWidget ();
    QPointer <QVBoxLayout> src_vbox;
    QPointer <QGroupBox> src_gb;
    QPointer <QLineEdit> src_ip_le;
    QPointer <QCheckBox> src_ip_cb;
    QPointer <QLineEdit> src_port_le;
    QPointer <QCheckBox> src_port_cb;
    QPointer <QLineEdit> src_mac_le;
    QPointer <QCheckBox> src_mac_cb;
    QPointer <QHBoxLayout> src_ip_hbox;
    QPointer <QHBoxLayout> src_port_hbox;
    QPointer <QHBoxLayout> src_mac_hbox;

    QString src_ip_string;
    QString src_port_string;
    QString src_mac_string;


    QPointer <QVBoxLayout> dst_vbox;
    QPointer <QGroupBox> dst_gb;
    QPointer <QLineEdit> dst_ip_le;
    QPointer <QCheckBox> dst_ip_cb;
    QPointer <QLineEdit> dst_port_le;
    QPointer <QCheckBox> dst_port_cb;
    QPointer <QLineEdit> dst_mac_le;
    QPointer <QCheckBox> dst_mac_cb;
    QPointer <QHBoxLayout> dst_ip_hbox;
    QPointer <QHBoxLayout> dst_mac_hbox;
    QPointer <QHBoxLayout> dst_port_hbox;


    QString dst_ip_string;
    QString dst_port_string;
    QString dst_mac_string;

    QPointer <QHBoxLayout> hbox0;

    QPointer <QPushButton> applyButton;
    QPointer <QPushButton> editButton;


    bool IsIpValid(QString ip);
    bool IsPortValid(QString port);


    QString get_src_ip();
    QString get_src_port();
    QString get_src_mac();

    QString get_dst_ip();
    QString get_dst_port();
    QString get_dst_mac();


    bool pass (ns3::Ptr <const ns3::Packet> p);
    //bool pass (ns3::Ptr <const ns3::Ipv4Header &> hdr);
    bool m_showIp;
    bool m_showMac;
    bool m_showPort;


private:
    QWidget * centralWidget;

signals:

public slots:
    void applyButton_slot();
    void editButton_slot();

    void src_ip_cb_slot(int state);
    void src_port_cb_slot(int state);
    void src_mac_cb_slot(int state);

    void dst_ip_cb_slot(int state);
    void dst_port_cb_slot(int state);
    void dst_mac_cb_slot(int state);

};

#endif // PACKETFILTER_H

#endif // WITH_NS3
