#ifdef WITH_NS3

#ifndef TWODGRAPHER_H
#define TWODGRAPHER_H
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include <QtGui>
#include <QGraphicsView>
#include <map>
#include <stdint.h>

namespace netanim {

class TwoDGrapherView : public QGraphicsView
{
    Q_OBJECT
public:
    explicit TwoDGrapherView(QString title,QString xaxislabel, QString yaxislabel,QString snapshotfilename,QWidget *parent = 0);
    typedef std::map<double,double> DataPointMap_t;
    std::map <double,double> mDataPoints;
    void AppendDataPoint (double x, double y);
    QPointer <QTimer> datagenTimer;
    void setXAxisLabel(QString l);
    void setYAxisLabel(QString l);


    void resizeEvent(QResizeEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void paintEvent(QPaintEvent *event);
    void drawBackground(QPainter *painter, const QRectF &rect);
    void setAxisLabelPosition();

    void drawAxisLabels(QPainter *painter, const QRectF &rectbg);
    void drawAxis(QPainter *painter, const QRectF &rectbg);
    void AddAxes();
    QLine GetYAxisLine();
    QLine GetXAxisLine();

    void InitXAxisPoints();
    void setXAxisPoints();
    std::vector <QGraphicsEllipseItem * > vXAxisPoints;
    std::vector <QGraphicsSimpleTextItem *> vXAxisPointsText;
    std::vector <QGraphicsLineItem *> vXGridLines;


    void InitYAxisPoints();
    void setYAxisPoints();
    std::vector <QGraphicsEllipseItem * > vYAxisPoints;
    std::vector <QGraphicsSimpleTextItem *> vYAxisPointsText;
    std::vector <QGraphicsLineItem *> vYGridLines;


    void getXDataMinMax(double *x, double *y);
    void getYDataMinMax(double *x, double *y);
    void CalcXAxisCoord();
    void CalcYAxisCoord();

    QPointF xAxisStart;
    QPointF xAxisEnd;

    QPointF yAxisStart;
    QPointF yAxisEnd;


    void increaseDataPoints();
    void decreaseDataPoints();

    std::vector <QGraphicsLineItem *> vPlotPoints;
    void InitPlotPoints();
    void SetPlotPoints();


    bool gridLinesOn;
    void setGridLines(bool s);
    void InitOptions();


    QPointer <QPushButton> showOptionsButton;
    QGroupBox * optionsWidget;
    QPointer <QCheckBox> grid_cb;
    QPointer <QLabel> dataPointsLabel;
    QPointer <QLabel> charTitleLabel;
    QGraphicsSimpleTextItem * chartTitlegWidget;

    void setChartTitle(QString title);
    QPointer <QPushButton> plusDP;
    QPointer <QPushButton> minusDP;
    QString m_snapShotBaseName;

    bool showOptions;

    QGraphicsSimpleTextItem * XAxisLabel;
    QGraphicsSimpleTextItem * YAxisLabel;


signals:


public slots:
    void snapshot_slot();
    void showall_slot();
    void onTimeout();
    void grid_cb_slot(int state);

    void increaseDataPoints_slot();
    void decreaseDataPoints_slot();

    void options_gb_slot();

private:
    uint32_t nDataPoints;

    int xAxisMargin;
    int yAxisMargin;
    QGraphicsLineItem * pXAxisLine;
    QGraphicsLineItem * pYAxisLine;

    int xAxisDataSections;
    int yAxisDataSections;

    QPointer <QGraphicsScene> scene;
    QString xAxisLabelString;
    QString yAxisLabelString;

private:
    void InitAxisLabels();



};

} // namespace netanim

#endif // TWODGRAPHER_H
#endif //WITH_NS3
