#ifdef WITH_NS3
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include "Logging.h"
#include "MainInfo/MainInfo.h"
#include <QtGui>
#include "ns3/log.h"
#include "Animator/NetView.h"
#include <iostream>
#include <iostream>
#include <streambuf>
#include <string>
using namespace std;

namespace netanim {

static const char * const remove_log_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                ",
" +++++++++++++  ",
" +++++++++++++  ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                "};

static const char * const add_log_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
" ++++++++++++++ ",
" ++++++++++++++ ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"                "};

QDebugStream::QDebugStream(std::ostream &stream, Logging * logging) : m_stream(stream), m_logging(logging)
 {
             m_old_buf = stream.rdbuf();
             stream.rdbuf(this);
 }

QDebugStream::~QDebugStream()
 {
             // output anything that is left
             if (!m_string.empty())
                     m_logging->HandleData(m_string.c_str());
             m_stream.rdbuf(m_old_buf);
 }

void QDebugStream::HandleOverflow (const char * s)
 {
         m_logging->HandleData(s);
 }
 std::streamsize QDebugStream::xsputn(const char *p, std::streamsize n)
 {
         m_string.append(p, p + n);

         int pos = 0;
         while (pos != std::string::npos)
         {
                 pos = m_string.find('\n');
                 if (pos != std::string::npos)
                 {
                         std::string tmp(m_string.begin(),m_string.begin() + pos);
                         m_logging->HandleData(tmp.c_str());
                         m_string.erase(m_string.begin(),m_string.begin() + pos + 1);
                 }
         }

         return n;
 }
 Logging::Logging (QObject *parent):AnimTab(parent),pcout(0),pcerr(0),pclog(0)
{


}
Logging::~Logging()
{
    if (pcout)
        delete pcout;
    if (pcerr)
        delete pcerr;
    if (pclog)
        delete pclog;
    pcout=pcerr=pclog=0;
}

void Logging::closeChannels()
{
    if (pcout)
        delete pcout;
    if (pcerr)
        delete pcerr;
    if (pclog)
        delete pclog;
    pcout=pcerr=pclog=0;

}

 void Logging::HandleData(const char *s)
 {
     if (!pcout || !pcerr || !pclog)
         return;
     for (uint8_t i = 0 ; i < vCompTextCombo.size(); i++)
     {
        QString component = vCompTextCombo[i]->GetCurrentComboSelection();
        if(component == "")
            continue;
        QString dataString = s;
        //QString debugString = "Comp:"+ component + " data:" + dataString;
        //std::cout<<"Component String:"<<component.toAscii().data() << " Data:"<<dataString.toAscii().data()<<std::endl;
        if(dataString.contains(component))
        {
            //vCompTextCombo[i]->SetText(debugString);
            vCompTextCombo[i]->SetText(dataString);
        }
     }

 }

void Logging::onFocus()
{
    hbox0->insertWidget(0,mainInfo->GetCommonVCR());
    mainInfo->GetCommonVCR()->show();
   // std::cout<<"Logging Got Focus"<<std::endl;
    pcout = new QDebugStream(std::cout,this);
    pcerr = new QDebugStream(std::cerr,this);
    pclog = new QDebugStream(std::clog,this);
}

void Logging::onLostFocus()
{
   // std::cout<<"Logging Lost Focus"<<std::endl;
    if (pcout)
        delete pcout;
    if (pcerr)
        delete pcerr;
    if (pclog)
        delete pclog;
    pcout=pcerr=pclog=0;
}

QWidget * Logging::Create(QApplication * app,MainInfo *mi)
{
  (void)app;
  mainInfo = mi;
  centralWidget = new QWidget;
  vbox = new QVBoxLayout;

  hbox0 = new QHBoxLayout;
  hbox0->addWidget(mi->GetCommonVCR());

  QPointer <QCheckBox> log_error = new QCheckBox;
  log_error->setText("ERROR");
  hbox0->addWidget(log_error);
  connect(log_error, SIGNAL(stateChanged(int)), this, SLOT(log_error_slot(int)));
  log_error->setChecked(false);


  QPointer <QCheckBox> log_warn = new QCheckBox;
  log_warn->setText("WARN");
  hbox0->addWidget(log_warn);
  connect(log_warn, SIGNAL(stateChanged(int)), this, SLOT(log_warn_slot(int)));
  log_warn->setChecked(false);


  QPointer <QCheckBox> log_debug = new QCheckBox;
  log_debug->setText("DEBUG");
  hbox0->addWidget(log_debug);
  connect(log_debug, SIGNAL(stateChanged(int)), this, SLOT(log_debug_slot(int)));
  log_debug->setChecked(false);


  QPointer <QCheckBox> log_info = new QCheckBox;
  log_info->setText("INFO");
  hbox0->addWidget(log_info);
  connect(log_info, SIGNAL(stateChanged(int)), this, SLOT(log_info_slot(int)));
  log_info->setChecked(false);


  QPointer <QCheckBox> log_logic = new QCheckBox;
  log_logic->setText("LOGIC");
  hbox0->addWidget(log_logic);
  connect(log_logic, SIGNAL(stateChanged(int)), this, SLOT(log_logic_slot(int)));
  log_logic->setChecked(false);

  QPointer <QCheckBox> log_function = new QCheckBox;
  log_function->setText("FUNCTION");
  hbox0->addWidget(log_function);
  connect(log_function, SIGNAL(stateChanged(int)), this, SLOT(log_function_slot(int)));
  log_logic->setChecked(false);


  QPointer <QCheckBox> log_all = new QCheckBox;
  log_all->setText("ALL");
  hbox0->addWidget(log_all);
  connect(log_all, SIGNAL(stateChanged(int)), this, SLOT(log_all_slot(int)));
  log_all->setChecked(true);



  QPointer <QCheckBox> prefix_time = new QCheckBox;
  prefix_time->setText("Prefix_time");
  hbox0->addWidget(prefix_time);
  connect(prefix_time, SIGNAL(stateChanged(int)), this, SLOT(log_prefix_time_slot(int)));
  prefix_time->setChecked(true);


  QPointer <QCheckBox> prefix_node = new QCheckBox;
  prefix_node->setText("Prefix_node");
  hbox0->addWidget(prefix_node);
  connect(prefix_node, SIGNAL(stateChanged(int)), this, SLOT(log_prefix_node_slot(int)));
  prefix_node->setChecked(true);

  QPointer <QCheckBox> prefix_func = new QCheckBox;
  prefix_func->setText("Prefix_func");
  hbox0->addWidget(prefix_func);
  connect(prefix_func, SIGNAL(stateChanged(int)), this, SLOT(log_prefix_func_slot(int)));
  prefix_func->setChecked(true);


  QPushButton* add_log = new QPushButton( QPixmap(add_log_xpm), QString());
  connect(add_log, SIGNAL(clicked()), this, SLOT(add_log_box()));

  QPushButton* remove_log = new QPushButton( QPixmap(remove_log_xpm), QString());
  connect(remove_log, SIGNAL(clicked()), this, SLOT(remove_log_box()));

  hbox0->addWidget(add_log);
  hbox0->addWidget(remove_log);

  QPointer <QWidget> hw0 = new QWidget;
  hw0->setLayout(hbox0);

  vbox->addWidget(hw0);

  hbox_log_box = new QHBoxLayout;


  hbox_log_box->addWidget(AddCombo());
  hbox_log_box->addWidget(AddCombo());

  QPointer <QWidget> hw = new QWidget;
  hw->setLayout(hbox_log_box);

  vbox->addWidget(hw);

  centralWidget->setLayout(vbox);
  return centralWidget;
}


void Logging::Notify()
{
    QStringList stlist = GetComponentList ();
    ns3::LogComponentDisableAll((ns3::LogLevel)(ns3::LOG_LEVEL_ALL|ns3::LOG_PREFIX_FUNC|ns3::LOG_PREFIX_NODE|ns3::LOG_PREFIX_TIME));
    for (QStringList::Iterator it = stlist.begin(); it != stlist.end(); it++)
    {
        QString cur = *it;
        if (cur == "") continue;
        LogComponentEnable(cur.toAscii().data(),(ns3::LogLevel)currentLogLevel);
        //std::cout<<"Enabling component"<<std::endl;
    }

}


QWidget * Logging::getCentralWidget()
{
    return centralWidget;
}
QStringList Logging::GetComponentList()
{

    QStringList stlist;
    for (uint8_t i = 0 ; i < vCompTextCombo.size(); i++)
    {
        stlist.append(vCompTextCombo[i]->GetCurrentComboSelection());

    }
    return stlist;
}

void Logging::add_log_box()
{
    hbox_log_box->addWidget(AddCombo());
}

void Logging::remove_log_box()
{
    if (vCompTextCombo.size() == 1)
    {
       return;
    }
    RemoveCombo();
}

void Logging::SetLogLevel(ns3::LogLevel level)
{
   currentLogLevel |= level;
}

void Logging::ResetLogLevel(ns3::LogLevel level)
{
    currentLogLevel &= ~level;
}
void Logging::log_prefix_func_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_PREFIX_FUNC);
    }
    else
    {
        ResetLogLevel(ns3::LOG_PREFIX_FUNC);
    }
    Notify();
}
void Logging::log_prefix_node_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_PREFIX_NODE);
    }
    else
    {
        ResetLogLevel(ns3::LOG_PREFIX_NODE);

    }
    Notify();

}
void Logging::log_prefix_time_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_PREFIX_TIME);
    }
    else
    {
        ResetLogLevel(ns3::LOG_PREFIX_TIME);
    }
    Notify();

}


void Logging::log_error_slot(int state)
{

    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_LEVEL_ERROR);
    }
    else
    {
        ResetLogLevel(ns3::LOG_LEVEL_ERROR);
    }
    Notify();

}
void Logging::log_warn_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_LEVEL_WARN);
    }
    else
    {
        ResetLogLevel(ns3::LOG_LEVEL_WARN);

    }
    Notify();

}
void Logging::log_debug_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_LEVEL_DEBUG);
    }
    else
    {

        ResetLogLevel(ns3::LOG_LEVEL_DEBUG);
    }
    Notify();

}
void Logging::log_function_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_LEVEL_FUNCTION);
    }
    else
    {
        ResetLogLevel(ns3::LOG_LEVEL_FUNCTION);
    }
    Notify();

}
void Logging::log_logic_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_LEVEL_LOGIC);
    }
    else
    {
        ResetLogLevel(ns3::LOG_LEVEL_LOGIC);

    }
    Notify();

}
void Logging::log_info_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_LEVEL_INFO);
    }
    else
    {
        ResetLogLevel(ns3::LOG_LEVEL_INFO);

    }
    Notify();

}
void Logging::log_all_slot(int state)
{
    if (state == Qt::Checked)
    {
        SetLogLevel(ns3::LOG_LEVEL_ALL);
    }
    else
    {
        ResetLogLevel(ns3::LOG_LEVEL_ALL);

    }
    Notify();

}

QWidget* Logging::RemoveCombo()
{
    CompTextCombo * p = vCompTextCombo[vCompTextCombo.size()-1];
    p->getCentralWidget()->hide();
    vCompTextCombo.pop_back();
    return 0;
}

QWidget* Logging::AddCombo()
{
    CompTextCombo *ctcmb = new CompTextCombo (this);
    vCompTextCombo.push_back(ctcmb);
    return ctcmb->getCentralWidget();
}

void Logging::AddCheckBoxes()
{

}

void CompTextCombo::SetText(QString str)
{
  te->append(str);

}

void CompTextCombo::SetlogtoFile(bool logit)
{

    logtoFile = logit;
}



void CompTextCombo::PopulateLogComponentCombo(QPointer <QComboBox> combo)
{

    QFile f("components_final");
    if (!f.exists())
    {
        std::cout<<"File does not exists"<<std::endl;
        return;
    }
    if (!f.open(QIODevice::ReadOnly))
    {
        std::cout<<"Unable to open file"<<std::endl;
        return;
    }
    QTextStream ts (&f);
    QString str = ts.readLine();
    if (!str.isNull())
        combo->addItem(str);
    while (!str.isNull())
    {
        str = ts.readLine();
        combo->addItem(str);
    }
    f.close();

}

QString Logging::GetTabName()
{
    return "Logging";
}


CompTextCombo::CompTextCombo(Logging *logging): m_logging(logging)
{
    QPointer <QVBoxLayout> vbox = new QVBoxLayout;

    combo = new QComboBox;

    connect(combo,SIGNAL( currentIndexChanged( const QString& ) ), this, SLOT( doValueChanged( const QString &) ));
    combo->addItem("");
    te = new QTextEdit;
    te->setText("Hola");
    te->setReadOnly(true);

    vbox->addWidget(combo);
    PopulateLogComponentCombo(combo);
    vbox->addWidget(te);

    centralWidget = new QWidget;
    centralWidget->setLayout(vbox);


}
void CompTextCombo::doValueChanged(const QString & s)
{
    //std::cout<<"value changed"<<std::endl;
    currentComponet = s;
    m_logging->Notify();
}

QString CompTextCombo::GetCurrentComboSelection()
{
    return currentComponet;
}

QPointer<QWidget> CompTextCombo::getCentralWidget()
{
    return centralWidget;
}
} // namespace netanim

#endif
