/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3
#ifndef LOGGING_H
#define LOGGING_H
#include "MainInfo/MainInfo.h"
#include <QWidget>
#include <QtGui>
#include <vector>
#include "ns3/log.h"
#include "AnimTab/animtab.h"


#include <iostream>
#include <streambuf>
#include <string>

namespace netanim {
class Logging;
// class QDebugStream : adapted from the version posted by
// Paul Miller
class QDebugStream : public std::basic_streambuf<char>
{
public:
    QDebugStream(std::ostream &stream, Logging * logging);
    ~QDebugStream();


protected:

    virtual int_type overflow(int_type v)
    {
               if (v == '\n')
               {
                   HandleOverflow(m_string.c_str());
                        //       qDebug(m_string.c_str());
                       m_string.clear();
               }
               else
                       m_string.push_back(v);

               return v;
    }
         virtual std::streamsize xsputn(const char *p, std::streamsize n);

private:
         void HandleOverflow (const char *);
         std::ostream &m_stream;
         std::streambuf *m_old_buf;
         std::string m_string;
         Logging * m_logging;
};



class CompTextCombo  : public QWidget
{
Q_OBJECT
public:

    CompTextCombo(Logging *);
    QPointer <QWidget> getCentralWidget();
    void PopulateLogComponentCombo(QPointer <QComboBox> combo);
    QString GetCurrentComboSelection();
    void SetText(QString str);
    void SetlogtoFile (bool logit);
private:
    QPointer <QComboBox> combo;
    QPointer <QTextEdit> te;
    QPointer <QWidget> centralWidget;
    QString currentComponet ;
    Logging * m_logging;
    bool logtoFile;
private slots:
    void doValueChanged(const QString &);

};


class Logging : public AnimTab
{

Q_OBJECT
public:
    explicit Logging(QObject *parent = 0);
    ~Logging();
    QWidget * Create (QApplication *,MainInfo *);
    QString GetTabName ();
    QWidget * AddCombo ();
    QWidget * RemoveCombo ();
    QStringList GetComponentList();
    void Notify();
    void HandleData(const char * s);
    QWidget* getCentralWidget();
    void onFocus();
    void onLostFocus();
    void closeChannels();

private slots:
    void log_error_slot(int state);
    void log_warn_slot(int state);
    void log_debug_slot(int state);
    void log_function_slot(int state);
    void log_logic_slot(int state);
    void log_info_slot(int state);
    void log_all_slot(int state);

    void log_prefix_func_slot(int state);
    void log_prefix_node_slot(int state);
    void log_prefix_time_slot(int state);

    void add_log_box();
    void remove_log_box();



private:
    MainInfo *mainInfo;
    QPointer <QHBoxLayout> hbox0;
    QPointer <QWidget> centralWidget;
    QPointer <QVBoxLayout> vbox;
    std::vector <CompTextCombo *> vCompTextCombo;
    void AddCheckBoxes();
    void SetLogLevel(ns3::LogLevel level);
    void ResetLogLevel(ns3::LogLevel level);
    int32_t currentLogLevel;
    QPointer <QHBoxLayout> hbox_log_box;
    QDebugStream * pcout;
    QDebugStream * pcerr;
    QDebugStream * pclog;

};

} // namespace netanim

#endif // LOGGING_H
#endif // WITH_NS3
