/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 * Author: John Abraham <john.abraham@gatech.edu>
 */
// Main information creating for animator

#include "MainInfo.h"
#include "MainWindow/mainwindow.h"
#include "Socket/client.h"

namespace netanim {

MainInfo::MainInfo()
    : view(0), model(0), advanceTo(0), completedThisCycle(0), mw(0)
#ifdef WITH_NS3
    ,simulationPaused (true),simulationTerminated(),animationOnline(false),xmlplatforminitialized (false), m_Args(""),
      pktfilter(0)
#endif
{
  pthread_mutex_init(&mainMutex, 0);
  pthread_cond_init(&mainCond, 0);
  pthread_mutex_init(&coutMutex, 0);
#ifdef WITH_NS3
  pthread_mutex_init(&simulationMutex,0);
  pthread_cond_init(&simulationCond,0);
#endif
  m_port = 5000;
}



void MainInfo::setArgs(int c, char ** v)
{
    argc = c;
    argv = v;
}


int MainInfo::getArgc()
{
    return argc;
}

char ** MainInfo::getArgv()
{
    return argv;
}
MainWindow * MainInfo::getMainWindow()
{
    return mw;
}

void MainInfo::setMainWindow(netanim::MainWindow * w)
{
    mw = w;
}
int MainInfo::AddNewXMLThread()
{
    ThreadInfo * ti = new ThreadInfo();
    threadInfo.push_back(*ti);
    return threadInfo.size()-1;
}

void MainInfo::SetXMLPlatformInitialized()
{

    xmlplatforminitialized = true;
}

bool MainInfo::IsXMLPlatformInitialized()
{
    return xmlplatforminitialized;
}

#ifdef WITH_NS3
void MainInfo::setPktFilter(PacketFilter *filter)
{
    pktfilter = filter;
}

void MainInfo::resetPktFilter()
{
    pktfilter = 0;

}

bool MainInfo::IsAnimationOnline()
{
    return animationOnline;

}


void MainInfo::SetAnimationOnline()
{

    animationOnline = true;
    GetSimulation()->AnimationOnline(true);
}


void MainInfo::ResetAnimationOnline()
{

    animationOnline = false;
    GetSimulation()->AnimationOnline(false);
}

void MainInfo::unlockSimulation()
{
    //std::cout<<"Before UnLock Simulation"<<std::endl;
    pthread_mutex_unlock(&simulationMutex);
    //std::cout<<"After UnLock Simulation"<<std::endl;
}

void MainInfo::signalSimulation()
{
    pthread_cond_signal(&simulationCond);

}

void MainInfo::lockSimulation()
{
    //std::cout<<"Before Lock Simulation"<<std::endl;
    pthread_mutex_lock(&simulationMutex);
    //std::cout<<"After Lock Simulation"<<std::endl;
}

bool MainInfo::IsSimulationPaused()
{
  return simulationPaused;
}

bool MainInfo::IsSimulationTerminated()
{
  return simulationTerminated;
}

void MainInfo::TerminateSimulation()
{
  std::cout<<"Terminated Simulation"<<std::endl;
  lockSimulation();
  getMainWindow()->SetStatusBarMessage("Simulation Stopped");
  //QMessageBox msg;
  //msg.setText("Simulation Stopped");
  //msg.exec();
  simulationTerminated = true;
  unlockSimulation();
  getMainWindow()->SetTerminated();

}
void MainInfo::PauseSimulation()
{

  getMainWindow()->SetStatusBarMessage("Simulation Paused");
  simulationPaused = true;
}

void MainInfo::UnPauseSimulation()
{
  lockSimulation();
  getMainWindow()->SetStatusBarMessage("Playing Simulation");
  simulationPaused = false;
  unlockSimulation();
}

void MainInfo::SetSimulation(Simulation * sim)
{
  simulation = sim;
}
Simulation * MainInfo::GetSimulation()
{
  return simulation;
}

void MainInfo::SetCommonVCR(QWidget * w)
{
  commonVCR = w;
}

QWidget * MainInfo::GetCommonVCR()
{
  return commonVCR;
}

PacketFilter * MainInfo::getPktFilter()
{
    return pktfilter;
}

#endif // WITH_NS3

bool MainInfo::Connect()
{
    return client.Connect(m_port);

}

void MainInfo::Disconnect()
{
    return client.Disconnect();
}



} //namespace netanim

