/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Shared information between threads and view for Animator

#ifndef __MAIN_INFO_H__
#define __MAIN_INFO_H__

#include <QStringList>
#include <vector>

#include "Animator/ThreadInfo.h"
#include "Socket/client.h"
#include "CmdHandlers/cmdhandlers.h"
//#include "MainWindow/mainwindow.h"

#ifdef WITH_NS3
#include "Filter/packetfilter.h"
#include "Simulation/private/simulation.h"
#endif


namespace netanim {

class NetView;
class NetModel;
class Simulation;
class MainWindow;
class MainInfo
{
public:
  MainInfo();
public:
  int AddNewXMLThread ();
  std::vector<ThreadInfo> threadInfo;
  NetView*           view;
  NetModel*          model;
  pthread_mutex_t    mainMutex;
  pthread_cond_t     mainCond;
  pthread_mutex_t    coutMutex; // debug only



  volatile double    advanceTo; // Time each thread should advance to
  unsigned           completedThisCycle;
  QStringList        XMLFileList;
  void setSimulationArgs (QString Args) { m_Args = Args;}
  QString getSimulationArgs () {return m_Args;}
  void setPort (uint16_t port) { m_port = port;}
  uint16_t getPort () { return m_port;}
  bool Connect ();
  void Disconnect ();

  netanim::MainWindow * getMainWindow ();
  void setMainWindow (netanim::MainWindow *);




#ifdef WITH_NS3
  pthread_mutex_t    simulationMutex;
  pthread_cond_t     simulationCond;
  void SetCommonVCR (QWidget *);
  QWidget * GetCommonVCR ();
  void SetSimulation (Simulation *);
  Simulation * GetSimulation ();
  void lockSimulation();
  void signalSimulation();
  void unlockSimulation();
  bool IsSimulationPaused();
  bool IsSimulationTerminated();
  void PauseSimulation();
  void UnPauseSimulation();
  void TerminateSimulation();
  void SetAnimationOnline ();
  void ResetAnimationOnline ();
  bool IsAnimationOnline ();

  void setPktFilter (PacketFilter * filter);
  PacketFilter * getPktFilter ();
  void resetPktFilter ();

#endif // WITH_NS3
  bool IsXMLPlatformInitialized ();
  void SetXMLPlatformInitialized ();
  int getArgc ();
  char ** getArgv();
  void setArgs (int ,char **);

private:
  Client client;
  uint16_t m_port;
  QString m_Args;
  CmdHandlers cmdHandlers;
  bool xmlplatforminitialized;
  netanim::MainWindow * mw;
  int argc;
  char **argv;


#ifdef WITH_NS3
  PacketFilter * pktfilter;
  bool animationOnline;
  QWidget * commonVCR;
  Simulation * simulation;
  bool simulationPaused ;
  bool simulationTerminated ;
#endif

};

//extern MainInfo * mainInfo;

} // namespace netanim
#endif
