/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include "mainwindow.h"
#include "MainInfo/MainInfo.h"
#include "Animator/NetView.h"
#include <QHBoxLayout>
#include <QToolBar>
#include <QtGui>
#include <QLabel>
#include "Dialogs/setserverportdialog.h"

#ifdef WITH_NS3
#include "Simulation/private/simulation.h"
#include "Routing/routingtab.h"
#include "Applications/applicationstab.h"
#include "NetDevice/NetDevice.h"
#include "Tcp/TcpTab.h"
#endif


extern void StartXML ();
namespace netanim {



static const char * const play_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
" ++             ",
" ++++           ",
" ++++++         ",
" ++++++++       ",
" ++++++++++     ",
" ++++++++++++   ",
" +++++++++++++  ",
" +++++++++++++  ",
" ++++++++++++   ",
" ++++++++++     ",
" ++++++++       ",
" ++++++         ",
" ++++           ",
" +++            ",
"                "};


static const char * const pause_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"                "};


static const char * const green_play_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #00FF00",
"                ",
" ++             ",
" ++++           ",
" ++++++         ",
" ++++++++       ",
" ++++++++++     ",
" ++++++++++++   ",
" +++++++++++++  ",
" +++++++++++++  ",
" ++++++++++++   ",
" ++++++++++     ",
" ++++++++       ",
" ++++++         ",
" ++++           ",
" +++            ",
"                "};

static const char * const red_play_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #FF0000",
"                ",
" ++             ",
" ++++           ",
" ++++++         ",
" ++++++++       ",
" ++++++++++     ",
" ++++++++++++   ",
" +++++++++++++  ",
" +++++++++++++  ",
" ++++++++++++   ",
" ++++++++++     ",
" ++++++++       ",
" ++++++         ",
" ++++           ",
" +++            ",
"                "};



MainWindow::MainWindow(QApplication * app, MainInfo * mi, QWidget *parent) :
    QMainWindow(parent),
    setserverPortAction(0),theApp(app),mainStatusBar(0),
    loadXMLAction(0)
{
    mainInfo = mi;
    setWindowTitle("Network Animator");
    createMenubar();
    createStatusBar();
    tabWidget = new QTabWidget();
    connect(tabWidget,SIGNAL(currentChanged(int)),this,SLOT(oncurrentabChanged(int)));

    netView = new NetView(NULL, NULL,mi);
    netView->Create(app);
    AddTab(netView->getCentralWidget(),netView->GetTabName());

#ifdef WITH_NS3
    mi->SetCommonVCR(CreateCommonVCR());
    logging  = new Logging;
    logging->Create(app,mi);
    AddTab(logging);
    routingTab = new RoutingTab;
    routingTab->Create(app,mi);
    AddTab(routingTab);
    applicationsTab = new ApplicationsTab;
    applicationsTab->Create(app,mi);
    AddTab(applicationsTab);
    netDeviceTab = new NetDeviceTab;
    netDeviceTab->Create(app,mi);
    AddTab(netDeviceTab);
    //tcpTab = new TcpTab;
    //tcpTab->Create(app,mi);
    //AddTab(tcpTab);

#endif
    RenderDefaultTab();

}

MainWindow::~MainWindow ()
{
    if (setserverPortAction)
        delete setserverPortAction;
    if (loadXMLAction)
        delete loadXMLAction;
    if (tabWidget)
        delete tabWidget;
#ifdef WITH_NS3
    mainInfo->TerminateSimulation();
    mainInfo->GetSimulation()->Stop();
#endif

}

void MainWindow::SetTerminated()
{
    //playButton->hide();
    //pauseButton->hide();

}

void MainWindow::SetStatusBarMessage(QString msg)
{
    mainStatusBar->setMessage(msg);

}

StatusBar * MainWindow::getStatusBar()
{
    return mainStatusBar;

}

void MainWindow::oncurrentabChanged(int index)
{
   #ifdef WITH_NS3
    // std::cout<<"Current Tab Index:"<<index<<std::endl;

    if (index == 0)
    {
        if (mainInfo->IsAnimationOnline())
            mainInfo->view->SetOnlineHBox();
        mainInfo->view->SetAnimatorStatusBar();
    }

   int tabIndex = tabWidget->currentIndex() -1 ;// We skip over NetView Tab
   for (uint32_t i = 0 ; i < vAnimTab.size() ; i++)
   {
       AnimTab * p = vAnimTab[i];
       if(i==tabIndex)
       {
            p->onFocus();
            mainInfo->view->ResetAnimatorStatusBar();
       }
       else
        p->onLostFocus();

   }
   #endif
}

#ifdef WITH_NS3
void MainWindow::CommonPlay()
{
  mainInfo->view->animation_onlineButton->setChecked(true);
  playButton->setIcon(QPixmap(green_play_xpm));
  //argsButton->hide();

  if (mainInfo->IsAnimationOnline())
  {
    mainInfo->view->Play();
  }
  else
  {
    mainInfo->GetSimulation()->Play();
  }


}

void MainWindow::CommonPause()
{
  playButton->setIcon(QPixmap(red_play_xpm));
  mainInfo->GetSimulation()->Pause();
}

void MainWindow::CommonSimulationArgs()
{
    setsimulationArgsDialog->exec();
}



QWidget * MainWindow::CreateCommonVCR()
{
    quitButton = new QPushButton("Quit");
    connect(quitButton, SIGNAL(clicked()), this, SLOT(Quit()));

    playButton = new QPushButton( QPixmap( play_xpm ), QString());
    connect(playButton, SIGNAL(clicked()), this, SLOT(CommonPlay()));

    pauseButton = new QPushButton( QPixmap( pause_xpm), QString());
    connect(pauseButton, SIGNAL(clicked()), this, SLOT(CommonPause()));

    lcd = new QLCDNumber;
    lcd->setToolTip("Simulation Time in seconds");
    lcd->setDigitCount(10);
    lcd->setSmallDecimalPoint(true);
    lcd->setAutoFillBackground(true);
    QPalette palette;
    palette.setColor(QPalette::Window,Qt::black);
    lcd->setPalette(palette);

    //setsimulationArgsDialog  = new SetSimulationArgs(mainInfo);
    //argsButton = new QPushButton("Args");
    //connect(argsButton, SIGNAL(clicked()), this, SLOT(CommonSimulationArgs()));


    QHBoxLayout * hbox = new QHBoxLayout;
    //hbox->addWidget(argsButton);
    hbox->addWidget(quitButton);
    hbox->addWidget(lcd);
    hbox->addWidget(playButton);
    hbox->addWidget(pauseButton);
    QWidget * w = new QWidget;
    w->setLayout(hbox);
    return w;

}

void MainWindow::SetLCD()
{
   // std::cout<<"Time:" <<ns3::Simulator::Now().GetSeconds()<<std::endl;
    lcd->display(ns3::Simulator::Now().ToDouble(ns3::Time::S));

}
#endif


void MainWindow::createStatusBar()
{
    mainStatusBar = new StatusBar (mainInfo,this);
    setStatusBar(mainStatusBar);
}
void MainWindow::createOptionsDialogs()
{
    setserverportDialog = new SetServerPortDialog (mainInfo);

}
void MainWindow::AddTab(AnimTab *tab)
{
#ifdef WITH_NS3
    QPointer <QScrollArea> sa = new QScrollArea;
    sa->setWidget(tab->getCentralWidget());
    sa->setWidgetResizable(true);
    tabWidget->addTab(sa,tab->GetTabName());
    //tabWidget->addTab(tab->getCentralWidget(),tab->GetTabName());

    vAnimTab.push_back(tab);
#endif
}

void MainWindow::AddTab(QWidget *w, QString tabName)
{
    tabWidget->addTab(w,tabName);

}

void MainWindow::RenderDefaultTab()
{
    RenderTab (0);
}

void MainWindow::setserverPort()
{
    setserverportDialog->exec();
}

void MainWindow::createMenubar()
{
    mainmenuBar = menuBar();
    setMenuBar(mainmenuBar);

    QMenu * menu1 = mainmenuBar->addMenu ("&Options");
    setserverPortAction = new QAction("Set Server Port",this);

    connect(setserverPortAction, SIGNAL(triggered()),
            this, SLOT(setserverPort()));
    //menu1->addAction(setserverPortAction);

    createOptionsDialogs();

    loadXMLAction = new QAction ("Load XML",this);
    connect(loadXMLAction, SIGNAL (triggered()),this, SLOT (loadXML()));
    menu1->addAction(loadXMLAction);

}

void MainWindow::loadXML()
{
    if (mainInfo->IsXMLPlatformInitialized())
    {
        QMessageBox msg;
        msg.setText("XML was already loaded.New XML rendering not yet supported");
        msg.exec();
        return;
    }
    QFileDialog fd;
    if (fd.exec() == QDialog::Accepted)
      {
          mainInfo->XMLFileList.clear();
          mainInfo->XMLFileList.append(fd.selectedFiles());
          StartXML ();
      }
}


void MainWindow::Quit()
{
#ifdef WITH_NS3
    this->logging->closeChannels();
    exit(0);
    mainInfo->TerminateSimulation();
#endif
    theApp->quit();

}

void MainWindow::RenderTab(uint32_t index)
{
    QHBoxLayout *hlayout = new QHBoxLayout;
    hlayout->addWidget(tabWidget);
    tabWidget->setCurrentIndex(index);

    QWidget * centralWidget = new QWidget;
    centralWidget->setLayout(hlayout);

    setCentralWidget(centralWidget);
    setUnifiedTitleAndToolBarOnMac(true);

    show();
}
} //namespace netanim
