/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

// Main window declarations

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QApplication>
#include <vector>
#include <QMenu>
#include "Dialogs/setserverportdialog.h"
#include "Dialogs/setsimulationargs.h"
#include "MainWindow/statusbar.h"
#include "MainInfo/MainInfo.h"
#include "AnimTab/animtab.h"
#include "Animator/NetView.h"

#ifdef WITH_NS3
#include "Logging/Logging.h"
#include "Routing/routingtab.h"
#include "NetDevice/NetDevice.h"
#include "Applications/applicationstab.h"
#include "Tcp/TcpTab.h"
#endif

class RoutingTab;
using namespace std;
namespace netanim {

typedef struct {
 QWidget * leftpaneWidget;
 QWidget * centralpaneWdiget;
 QWidget * rightpaneWidget;

} MultiWidget;

class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    explicit MainWindow(QApplication * app, MainInfo * mi, QWidget *parent = 0);
    ~MainWindow();
    void AddTab (AnimTab *);
    void AddTab (QWidget *,QString);
    void RenderDefaultTab ();
    void RenderTab (uint32_t index);
    vector <MultiWidget> allTabs;
    void createMenubar();
    void createOptionsDialogs ();
    void createStatusBar();
    StatusBar * getStatusBar ();
    void SetStatusBarMessage (QString msg);
    void SetTerminated ();
    void SetLCD();


signals:
#ifdef WITH_NS3
public slots:
    void CommonPlay();
    void CommonPause();
    void CommonSimulationArgs();

#endif
public slots:
    void oncurrentabChanged(int);

private slots:
    void setserverPort ();
    void loadXML ();
    void Quit();



private:
    QPointer <QLCDNumber> lcd;
    QPointer <QPushButton> quitButton;
    QPointer <QPushButton> playButton;
    QPointer <QPushButton> pauseButton;
    QPointer <QPushButton> argsButton;
    QMenuBar * mainmenuBar;
    SetServerPortDialog * setserverportDialog;
    QPointer <SetSimulationArgs> setsimulationArgsDialog;
    StatusBar * mainStatusBar;
    QAction *setserverPortAction;
    QAction *loadXMLAction;
    NetView *netView;

    QTabWidget *tabWidget;
    MainInfo * mainInfo;
    QApplication * theApp;


#ifdef WITH_NS3
    QWidget * CreateCommonVCR ();
    Logging * logging;
    RoutingTab * routingTab;
    ApplicationsTab * applicationsTab;
    NetDeviceTab * netDeviceTab;
    //TcpTab * tcpTab;
    std::vector <AnimTab *> vAnimTab;
#endif

};

} // namespace netanim



#endif // MAINWINDOW_H
