/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include "statusbar.h"
#include "MainInfo/MainInfo.h"
#include "Animator/NetView.h"

#define PLAYINGICON "images/connected.png"
#define STOPPEDICON "images/disconnected.png"


namespace netanim {

StatusBar::StatusBar(MainInfo * mi, QWidget *parent) :
    QStatusBar(parent)
{
    mainInfo = mi;
    connectButton = new QPushButton;

    shownode_cb = new QCheckBox;
    shownode_cb->setText("Show Node Id");
    shownode_cb->setChecked(true);
    //mainInfo->view->SetShowNodeId();

    connect (shownode_cb,SIGNAL(stateChanged(int)), this, SLOT(shownodestatechanged(int)));
    addWidget(shownode_cb);

    nodefontsizelabel = new QLabel ("     Node Id Font Size:");
    increaseNodefont = new QPushButton("+");
    connect (increaseNodefont,SIGNAL(clicked()), this, SLOT(increasenodefont_slot()));

    decreaseNodefont = new QPushButton("-");
    connect (decreaseNodefont,SIGNAL(clicked()), this, SLOT(decreasenodefont_slot()));

    pktCountLabel = new QLabel ("Pkts:0");
    wpktCountLabel = new QLabel ("WPkts:0");


    addWidget(nodefontsizelabel);
    addWidget(increaseNodefont);
    addWidget(decreaseNodefont);

//    addWidget(pktCountLabel);
    message = new QLabel;
    addWidget(message);

   // SetDisconnect();
   // connect (connectButton, SIGNAL (clicked ()), this, SLOT (toggleStatus()));
   // insertPermanentWidget(0,connectButton);
}

void StatusBar::setMessage(QString msg)
{

    message->setText(msg);

}


void StatusBar::SetPacketCount(uint32_t pktCount)
{
    pktCountLabel->setText("Pkts:" + QString::number(pktCount));

}


void StatusBar::SetWPacketCount(uint32_t wpktCount)
{
    wpktCountLabel->setText("WPkts:" + QString::number(wpktCount));

}
void StatusBar::SetAnimatorStatusBar()
{
    shownode_cb->show();
    nodefontsizelabel->show();
    increaseNodefont->show();
    decreaseNodefont->show();
}

void StatusBar::ResetAnimatorStatusBar()
{
    shownode_cb->hide();
    nodefontsizelabel->hide();
    increaseNodefont->hide();
    decreaseNodefont->hide();

}

void StatusBar::increasenodefont_slot()
{
    mainInfo->view->increaseNodeIdfontSize();
}

void StatusBar::decreasenodefont_slot()
{
    mainInfo->view->decreaseNodeIdfontSize();
}
void StatusBar::shownodestatechanged(int c)
{
    if (shownode_cb->isChecked())
        mainInfo->view->SetShowNodeId();
    else
        mainInfo->view->ResetShowNodeId();

}
void StatusBar::toggleStatus ()
{
    if (connectStatus)
        SetDisconnect();
    else
        SetConnect();

}

void StatusBar::SetConnect()
{
    if (mainInfo->Connect())
    {
        connectButton->setIcon(QIcon(QPixmap(CONNECTEDICON)));
        connectButton->setText("Connected");
        connectStatus = true;
    }
    else
    {
        SetDisconnect();
    }

}

void StatusBar::SetDisconnect()
{
    mainInfo->Disconnect ();
    connectButton->setIcon(QIcon(QPixmap(DISCONNECTEDICON)));
    connectButton->setText("Disconnected");
    connectStatus = false;
}
} //namespace netanim
