/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3
#include "MainInfo/MainInfo.h"
#include "MainWindow/mainwindow.h"
#include "NetDevice/NetDevice.h"
#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/point-to-point-net-device.h"
#include "ns3/wifi-module.h"
#include "ns3/wimax-module.h"
#include "ns3/csma-module.h"

using namespace ns3;

namespace netanim {

static std::vector<NetDeviceCombo3 *> gNetDeviceStats;
static std::vector<NetDeviceCombo3 *> gQueueStats;
static bool gLogToFile = false;


static int defaultGridMaxColumn = 4;


NetDeviceTab::NetDeviceTab(QObject * parent):AnimTab (parent),
    polling(false),
    simtimePolling(true),
    simtimePollingTimer(ns3::Timer::CANCEL_ON_DESTROY)

{
    wallClocktimer = new QTimer;
}

QWidget* NetDeviceTab::Create(QApplication *app, MainInfo *mi)
{

    (void)app;
    mainInfo = mi;
    numNodes = ns3::NodeList::GetNNodes();

    vbox0 = new QVBoxLayout;
    centralWidget = new QWidget;
    hbox0 = new QHBoxLayout;
    hbox0->addWidget(mi->GetCommonVCR());


    logtoFileButton = new QPushButton("Log to File");
    logtoFileButton->setToolTip("Toggle this button to Log the statistics to the logs folder");
    logtoFileButton->setCheckable(true);
    connect(logtoFileButton, SIGNAL (clicked()), this, SLOT (logtofileslot()));


    QPointer <QLabel> pollLabel = new QLabel ("Poll Interval in ms");
    pollLineEdit = new QLineEdit;
    pollLineEdit->setText("1000");
    pollLabel->setBuddy(pollLineEdit);


    applyButton = new QPushButton(("&Stopped"));
    applyButton->setCheckable(true);
    connect(applyButton, SIGNAL (clicked ()),this,SLOT(apply()));

    polling_gb = new QGroupBox;
    wallClock_rb = new QRadioButton("WallClock");
    wallClock_rb->setToolTip("Use Wall Clock Time for polling");
    connect(wallClock_rb, SIGNAL(clicked(bool)),this, SLOT(wallclockrb_slot(bool)));
    QPointer <QHBoxLayout> pollinggb_hbox = new QHBoxLayout;
    simulationTime_rb = new QRadioButton("SimTime");
    simulationTime_rb->setToolTip("Use Simulation Time for polling");
    simulationTime_rb->setChecked(true);
    connect(simulationTime_rb, SIGNAL (clicked(bool)),this,SLOT(simtimerb_slot(bool)));
    pollinggb_hbox->addWidget(simulationTime_rb);
    pollinggb_hbox->addWidget(wallClock_rb);
    polling_gb->setLayout(pollinggb_hbox);



    hbox0->addWidget(logtoFileButton);
    hbox0->addWidget(pollLabel);
    hbox0->addWidget(pollLineEdit);
    hbox0->addWidget(polling_gb);
    hbox0->addWidget(applyButton);
    vbox0->addLayout(hbox0);

    optionsHBox = new QHBoxLayout;

    showNetDeviceStats_cb = new QCheckBox("Net Device stats");
    showNetDeviceStats_cb->setChecked(true);
    connect(showNetDeviceStats_cb,SIGNAL(stateChanged(int)),this,SLOT(showNetDeviceStats_slot(int)));

    showQueueStats_cb = new QCheckBox("Queue stats");
    showQueueStats_cb->setChecked(true);
    connect(showQueueStats_cb,SIGNAL(stateChanged(int)),this,SLOT(showQueueStats_slot(int)));

    optionsHBox->addWidget(showNetDeviceStats_cb);
    optionsHBox->addWidget(showQueueStats_cb);

    vbox0->addLayout(optionsHBox);
    vbox0->setSpacing(0);


    // Node HBox
    AddNetDeviceCombos();
    vbox0->addWidget(nodesSA);

    // Gridbox
    gridbox2 = new QGridLayout;
    vbox0->addLayout(gridbox2);

    centralWidget->setLayout(vbox0);
    return centralWidget;


}


void NetDeviceTab::showNetDeviceStats_slot(int state)
{
    if(showNetDeviceStats_cb->isChecked())
        ShowNetDeviceStats(true);
    else
        ShowNetDeviceStats(false);


}

void NetDeviceTab::ShowNetDeviceStats(bool s)
{
    for (uint32_t i = 0 ; i < gNetDeviceStats.size() ; i++)
    {
        NetDeviceCombo3 * n3 = gNetDeviceStats[i];
        if (s)
        n3->getCentralWidget()->show();
        else
        n3->getCentralWidget()->hide();

    }

}

void NetDeviceTab::ShowQueueStats(bool s)
{
    for (uint32_t i = 0 ; i < gQueueStats.size() ; i++)
    {
        NetDeviceCombo3 * n3 = gQueueStats[i];
        if (s)
        n3->getCentralWidget()->show();
        else
        n3->getCentralWidget()->hide();

    }

}


void NetDeviceTab::showQueueStats_slot(int state)
{
    if(showQueueStats_cb->isChecked())
        ShowQueueStats(true);
    else
        ShowQueueStats(false);



}

void NetDeviceTab::simtimerb_slot(bool state)
{
    if(simulationTime_rb->isChecked())
        simtimePolling = true;
}

void NetDeviceTab::wallclockrb_slot(bool state)
{

    if(wallClock_rb->isChecked())
        simtimePolling = false;
}


void NetDeviceTab::onFocus()
{
    hbox0->insertWidget(0,mainInfo->GetCommonVCR());
    mainInfo->GetCommonVCR()->show();

}

void NetDeviceTab::onLostFocus()
{


}

QWidget* NetDeviceTab::getCentralWidget()
{
    return centralWidget;
}

QString NetDeviceTab::GetTabName()
{

    return "NetDevice";
}

void NetDeviceTab::logtofileslot()
{
    gLogToFile = !gLogToFile;
    if(gLogToFile)
    logtoFileButton->setText("Logging To file!!");
    else
    logtoFileButton->setText("Log to File");


}

void NetDeviceTab::DoSimTimePolling()
{
    mainInfo->getMainWindow()->SetLCD();
    static bool first = true;
    if (first)
    {
       QMessageBox msg;
       msg.setText("Please ensure you used \"Simulator::Stop (Seconds (Time_t))\" function\nOr your simulation can run endlessly");
       msg.exec();
    }

    if(simtimePollingTimer.IsRunning())
    simtimePollingTimer.Cancel();
    if(!polling || (Simulator::IsFinished() && !first))
        return;
    if(Simulator::IsFinished() && !first)
    {
       polling = false;
       QMessageBox msg;
       msg.setText("Simulation Finished");
       msg.exec();
       applyButton->setText("Stopped");

       return;
    }
    simtimePollingTimer.SetFunction (&netanim::NetDeviceTab::DoSimTimePolling,this);
    UpdateNetDevices();
    simtimePollingTimer.Schedule(ns3::MilliSeconds(pollString.toInt()));
    first = false;


}


void NetDeviceTab::DoWallClockPolling()
{
    mainInfo->getMainWindow()->SetLCD();
    if(wallClocktimer->isActive())
        wallClocktimer->stop();
    if(!polling)
        return;

    wallClocktimer->setInterval(pollString.toInt());
    connect(wallClocktimer, SIGNAL(timeout()), this, SLOT(UpdateNetDevices()));
    wallClocktimer->start();


}

void NetDeviceTab::apply()
{
    polling = !polling;
    pollString = pollLineEdit->text();

    if (simtimePolling)
        DoSimTimePolling();
    else
        DoWallClockPolling();


    if (!polling)
    {
        applyButton->setText("Stopped");
        polling_gb->setEnabled(true);
        pollLineEdit->setEnabled(true);
        return;
    }
    else
    {
        applyButton->setText("Polling");
        polling_gb->setEnabled(false);
        pollLineEdit->setEnabled(false);

    }


}


void NetDeviceTab::AddNetDeviceCombos()
{
    nodesHBox = new QHBoxLayout;
    nodesSA = new QScrollArea;
    for (uint32_t i = 0; i < numNodes ; i++)
    {
        QPointer <NetDeviceCombo> acmb = AddNetDeviceCombo(i);
        nodesHBox->addWidget(acmb->getButton());
    }
    QWidget * qw = new QWidget;
    qw->setLayout(nodesHBox);
    nodesSA->setFixedHeight(50);
    nodesSA->setWidget(qw);


}


void NetDeviceTab::UpdateNetDevices()
{

    if(mainInfo->IsSimulationPaused())
        return;
   for (mNDType::iterator i = mNetDeviceCombo.begin();
    i != mNetDeviceCombo.end();
    i++)
   {
       QPointer <NetDeviceCombo> ncmb = i->second;
       ncmb->UpdateNetDevices();
   }
}


NetDeviceCombo * NetDeviceTab::AddNetDeviceCombo(uint32_t nodeId)
{

    NetDeviceCombo * ncmb = new NetDeviceCombo(this,nodeId,mainInfo);
    mNetDeviceCombo[nodeId] = ncmb;
    return ncmb;

}


void NetDeviceTab::AdjustGrid()
{
    typedef std::map <uint32_t,QPointer <NetDeviceCombo> > mNType;
    for (mNDType::iterator i = mNetDeviceCombo.begin();
         i != mNetDeviceCombo.end();
         i++)
    {

        QPointer <NetDeviceCombo> p = i->second;
        gridbox2->removeWidget(p->getCentralWidget());
        p->getCentralWidget()->hide();
    }

    for (mNDType::iterator i = mNetDeviceCombo.begin();
         i != mNetDeviceCombo.end();
         i++)
    {
        QPointer <NetDeviceCombo> p = i->second;
        if(p->isactive())
        {
            int total = gridbox2->count();
            int r = gridbox2->rowCount();
            int maxcolumn = defaultGridMaxColumn;
            if ((total % maxcolumn) == 0)
             gridbox2->addWidget(p->getCentralWidget(),r+1,0,1,1);
            else
                gridbox2->addWidget(p->getCentralWidget(),r-1,(total % maxcolumn ),1,1);

            p->getCentralWidget()->show();
        }

    }


}


void NetDeviceTab::NotifyToggled(QPointer <NetDeviceCombo> ncmb, uint32_t NodeId,bool active,QWidget * w)
{
    int index=0;
    if ((index=gridbox2->indexOf(w)) == -1)
    {
    // lets add this widget
        if(active)
          {
             int total = gridbox2->count();
             int r = gridbox2->rowCount();
             int maxcolumn = defaultGridMaxColumn;
             if ((total % maxcolumn) == 0)
              gridbox2->addWidget(w,r+1,0,1,1);
             else
              gridbox2->addWidget(w,r-1,(total % maxcolumn ),1,1);

          }
        mNetDeviceCombo[NodeId] = ncmb;
    }
    if(!active)
    {

         gridbox2->removeWidget(w);
         w->setVisible(false);
         AdjustGrid();

    }
    else
    {
         w->setVisible(true);
         UpdateNetDevices();
         ncmb->PrintInterface();
    }

}

NetDeviceCombo::NetDeviceCombo(NetDeviceTab *ndt, uint32_t nodeId,MainInfo * mi):
    mainInfo(mi),
    logtoFile(false),
    NNetDevices(0)
{
    m_ndt = ndt;
    m_nodeId = nodeId;
    //centralWidget = new QWidget;
    nodeButton = new QPushButton(QString::number(nodeId));
    nodeButton->setCheckable(true);
    connect( nodeButton, SIGNAL(clicked()), this, SLOT(onbuttonToggled()) );

    vbox0 = new QVBoxLayout;
    centralWidget = new QWidget;

    gb = new QGroupBox("Node:" + QString::number(nodeId));
    MacIPLabel = new QLabel;
    vbox0->addWidget(MacIPLabel);
    vbox0->setSpacing(0);

    n = ns3::NodeList::GetNode(m_nodeId);
    NS_ASSERT (n);
    for (uint32_t i = 0 ; i < n->GetNDevices() ; i++)
    {
        QPointer <NetDeviceCombo2> ncmb2 = new NetDeviceCombo2 (n->GetDevice(i),mi);
        vNetDeviceCombo2s.push_back(ncmb2);
        vbox0->addWidget(ncmb2->getCentralWidget());
    }

    gb->setLayout(vbox0);

    centralWidget = gb;
    active = false;

}

NetDeviceCombo::~NetDeviceCombo()
{

}



void NetDeviceCombo::SetLogtoFile(bool logit)
{
    logtoFile = logit;
}


bool NetDeviceCombo::isactive()
{
    return active;
}

QPointer <QPushButton> NetDeviceCombo::getButton()
{
    return nodeButton;

}


void NetDeviceCombo::onbuttonToggled()
{
    active = !active;
    m_ndt->NotifyToggled(QPointer <NetDeviceCombo> (this),m_nodeId, active,centralWidget);

}


QWidget * NetDeviceCombo::getCentralWidget()
{
    return centralWidget;
}


void NetDeviceCombo::UpdateNetDevices()
{
    PrintInterface();

    for (uint32_t i = 0 ; i < vNetDeviceCombo2s.size() ; i++)
    {
        vNetDeviceCombo2s[i]->UpdateNetDevice();
    }
}


void NetDeviceCombo::PrintInterface()
{
    Ptr <ns3::Node> n = ns3::NodeList::GetNode(m_nodeId);
    NS_ASSERT (n);

    NodeStatistics & stats = mainInfo->GetSimulation()->GetNodesStatistics(n->GetId());
    if(!&stats)
        return;
    stats.ndevices = n->GetNDevices();
    if (!stats.ndevices)
            return;
    if (!stats.vMacIPv4Pairs.empty())
        return;
    if ( stats.vMacIPv4Pairs.empty())
    {

        Ptr <ns3::Ipv4> ipv4 = n->GetObject <ns3::Ipv4> ();
        NS_ASSERT (ipv4);
        for (int i = 0 ; i < stats.ndevices ; i++)
        {
            Ipv4InterfaceAddress iaddr = ipv4->GetAddress (i,0);
            Ipv4Address addri = iaddr.GetLocal ();
            Address mac;
            if(i!=0)
            mac = n->GetDevice(i-1)->GetAddress();
            else
            mac = n->GetDevice(stats.ndevices-1)->GetAddress();

            std::stringstream ssip;
            addri.Print(ssip);
            QString ipaddress = ssip.str().c_str();

            std::stringstream ssmac;
            ssmac << mac;
            QString macaddress = ssmac.str().c_str();
            netanim::macipv4pair_t pair = { macaddress, ipaddress};
            //std::cout<<"Mac:"<<macaddress.toAscii().data() <<" IP:"<<ipaddress.toAscii().data()<<endl;
            stats.vMacIPv4Pairs.push_back(pair);

        }
    }
    QString str ="";

    MacIPLabel->setToolTip("Device Id / Mac Address / Ipv4 Address");

    str += "Devices  :" + QString::number(stats.ndevices) + "\n";

    for (int i = 0 ; i < stats.vMacIPv4Pairs.size() ; i++)
    {
        netanim::macipv4pair_t pair = stats.vMacIPv4Pairs[i];
        str += (QString::number(i) + ":" + pair.Mac + "/" + pair.Ipv4address + "\n");
    }
    MacIPLabel->setText(str);


}

NetDeviceCombo2::NetDeviceCombo2(ns3::Ptr<ns3::NetDevice> nd, MainInfo *mi):
    m_nd(nd),
    mainInfo (mi)

{
    centralWidget = new QWidget;
    netDeviceStats_gb = new QGroupBox("NetDevice Stats");
    queueStats_gb = new QGroupBox("Queue Stats");

    vbox0 = new QVBoxLayout;
    vbox0->setSpacing(0);
    ndStatsvbox0 = new QVBoxLayout;
    qStatsvbox0 = new QVBoxLayout;

    MacLabel = new QLabel;

    std::stringstream ssmac;
    ssmac << nd->GetAddress();
    m_macaddress = ssmac.str().c_str();


    pcapButton = new QPushButton ("Start Pcap");
    pcapButton->setToolTip("Start a packet capture for this device");
    connect(pcapButton,SIGNAL (clicked()),this,SLOT (pcapButton_slot()));
    pcapButton->setMaximumWidth(100);
    MacLabel->setText( "Mac:" + m_macaddress);

    QFont f;
    f.setUnderline(true);
    f.setBold(true);
    MacLabel->setFont(f);
    vbox0->addWidget(MacLabel);
    vbox0->addWidget(pcapButton);


    NDType = new QLabel;
    ndStatsvbox0->addWidget(NDType);
    ndStatsvbox0->setSpacing(0);
    netDeviceStats_gb->setLayout(ndStatsvbox0);

    NQueueType = new QLabel;
    qStatsvbox0->addWidget(NQueueType);
    qStatsvbox0->setSpacing(0);
    queueStats_gb->setLayout(qStatsvbox0);

    vbox0->addWidget(netDeviceStats_gb);
    vbox0->addWidget(queueStats_gb);

    centralWidget->setLayout(vbox0);
    ConnectTraces();



}

// Pcap related methods taken from ns-3 yans-wifi-helper.cc directly
static void
WifiPcapSniffTxEvent (
  Ptr<PcapFileWrapper> file,
  Ptr<const ns3::Packet>   packet,
  uint16_t            channelFreqMhz,
  uint16_t            channelNumber,
  uint32_t            rate,
  bool                isShortPreamble)
{
  uint32_t dlt = file->GetDataLinkType ();

  switch (dlt)
    {
    case PcapHelper::DLT_IEEE802_11:
      file->Write (Simulator::Now (), packet);
      return;
    case PcapHelper::DLT_PRISM_HEADER:
      {
        NS_FATAL_ERROR ("PcapSniffTxEvent(): DLT_PRISM_HEADER not implemented");
        return;
      }
    case PcapHelper::DLT_IEEE802_11_RADIO:
      {
        ns3::Ptr<ns3::Packet> p = packet->Copy ();
        RadiotapHeader header;
        uint8_t frameFlags = RadiotapHeader::FRAME_FLAG_NONE;
        header.SetTsft (Simulator::Now ().GetMicroSeconds ());

        // Our capture includes the FCS, so we set the flag to say so.
        frameFlags |= RadiotapHeader::FRAME_FLAG_FCS_INCLUDED;

        if (isShortPreamble)
          {
            frameFlags |= RadiotapHeader::FRAME_FLAG_SHORT_PREAMBLE;
          }

        header.SetFrameFlags (frameFlags);
        header.SetRate (rate);

        uint16_t channelFlags = 0;
        switch (rate)
          {
          case 2:  // 1Mbps
          case 4:  // 2Mbps
          case 10: // 5Mbps
          case 22: // 11Mbps
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_CCK;
            break;

          default:
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_OFDM;
            break;
          }

        if (channelFreqMhz < 2500)
          {
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_SPECTRUM_2GHZ;
          }
        else
          {
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_SPECTRUM_5GHZ;
          }

        header.SetChannelFrequencyAndFlags (channelFreqMhz, channelFlags);

        p->AddHeader (header);
        file->Write (Simulator::Now (), p);
        return;
      }
    default:
      NS_ABORT_MSG ("PcapSniffTxEvent(): Unexpected data link type " << dlt);
    }
}

static void
WifiPcapSniffRxEvent (
  Ptr<PcapFileWrapper> file,
  Ptr<const ns3::Packet> packet,
  uint16_t channelFreqMhz,
  uint16_t channelNumber,
  uint32_t rate,
  bool isShortPreamble,
  double signalDbm,
  double noiseDbm)
{
  uint32_t dlt = file->GetDataLinkType ();

  switch (dlt)
    {
    case PcapHelper::DLT_IEEE802_11:
      file->Write (Simulator::Now (), packet);
      return;
    case PcapHelper::DLT_PRISM_HEADER:
      {
        NS_FATAL_ERROR ("PcapSniffRxEvent(): DLT_PRISM_HEADER not implemented");
        return;
      }
    case PcapHelper::DLT_IEEE802_11_RADIO:
      {
        Ptr<ns3::Packet> p = packet->Copy ();
        RadiotapHeader header;
        uint8_t frameFlags = RadiotapHeader::FRAME_FLAG_NONE;
        header.SetTsft (Simulator::Now ().GetMicroSeconds ());

        // Our capture includes the FCS, so we set the flag to say so.
        frameFlags |= RadiotapHeader::FRAME_FLAG_FCS_INCLUDED;

        if (isShortPreamble)
          {
            frameFlags |= RadiotapHeader::FRAME_FLAG_SHORT_PREAMBLE;
          }

        header.SetFrameFlags (frameFlags);
        header.SetRate (rate);

        uint16_t channelFlags = 0;
        switch (rate)
          {
          case 2:  // 1Mbps
          case 4:  // 2Mbps
          case 10: // 5Mbps
          case 22: // 11Mbps
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_CCK;
            break;

          default:
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_OFDM;
            break;
          }

        if (channelFreqMhz < 2500)
          {
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_SPECTRUM_2GHZ;
          }
        else
          {
            channelFlags |= RadiotapHeader::CHANNEL_FLAG_SPECTRUM_5GHZ;
          }

        header.SetChannelFrequencyAndFlags (channelFreqMhz, channelFlags);

        header.SetAntennaSignalPower (signalDbm);
        header.SetAntennaNoisePower (noiseDbm);

        p->AddHeader (header);
        file->Write (Simulator::Now (), p);
        return;
      }
    default:
      NS_ABORT_MSG ("PcapSniffRxEvent(): Unexpected data link type " << dlt);
    }
}


// taken from ns-3 wimax-helper.cc
static void WimaxPcapSniffTxRxEvent (Ptr<PcapFileWrapper> file,
                                Ptr<const PacketBurst> burst)
{
  std::list<Ptr<ns3::Packet> > packets = burst->GetPackets ();
  for (std::list<Ptr<ns3::Packet> >::iterator iter = packets.begin (); iter != packets.end (); ++iter)
    {
      Ptr<ns3::Packet> p = (*iter)->Copy ();
      WimaxMacToMacHeader m2m (p->GetSize ());
      p->AddHeader (m2m);
      file->Write (Simulator::Now (), p);
    }
}


void
NetDeviceCombo2::WimaxEnablePcapInternal (std::string prefix, Ptr<ns3::NetDevice> nd, bool explicitFilename, bool promiscuous)
{
  //
  // All of the Pcap enable functions vector through here including the ones
  // that are wandering through all of devices on perhaps all of the nodes in
  // the system.  We can only deal with devices of type WimaxNetDevice.
  //
  Ptr<WimaxNetDevice> device = nd->GetObject<WimaxNetDevice> ();


  Ptr<WimaxPhy> phy = device->GetPhy ();
  PcapHelper pcapHelper;
  std::string filename;
  if (explicitFilename)
    {
      filename = prefix;
    }
  else
    {
      filename = pcapHelper.GetFilenameFromDevice (prefix, device);
    }

  Ptr<PcapFileWrapper> file = pcapHelper.CreateFile (filename, std::ios::out, PcapHelper::DLT_EN10MB);

  phy->TraceConnectWithoutContext ("Tx", MakeBoundCallback (&WimaxPcapSniffTxRxEvent, file));
  phy->TraceConnectWithoutContext ("Rx", MakeBoundCallback (&WimaxPcapSniffTxRxEvent, file));
}

void
NetDeviceCombo2::WifiEnablePcapInternal (std::string prefix, Ptr<NetDevice> nd, bool promiscuous, bool explicitFilename)
{
  //
  // All of the Pcap enable functions vector through here including the ones
  // that are wandering through all of devices on perhaps all of the nodes in
  // the system.  We can only deal with devices of type WifiNetDevice.
  //
  Ptr<WifiNetDevice> device = nd->GetObject<WifiNetDevice> ();
  Ptr<WifiPhy> phy = device->GetPhy ();
  NS_ABORT_MSG_IF (phy == 0, "YansWifiPhyHelper::EnablePcapInternal(): Phy layer in WifiNetDevice must be set");

  PcapHelper pcapHelper;

  std::string filename;
  if (explicitFilename)
    {
      filename = prefix;
    }
  else
    {
      filename = pcapHelper.GetFilenameFromDevice (prefix, device);
    }

  Ptr<PcapFileWrapper> file = pcapHelper.CreateFile (filename, std::ios::out, PcapHelper::DLT_IEEE802_11);

  phy->TraceConnectWithoutContext ("MonitorSnifferTx", MakeBoundCallback (&WifiPcapSniffTxEvent, file));
  phy->TraceConnectWithoutContext ("MonitorSnifferRx", MakeBoundCallback (&WifiPcapSniffRxEvent, file));
}


void
NetDeviceCombo2::CsmaEnablePcapInternal (std::string prefix, Ptr<NetDevice> nd, bool promiscuous, bool explicitFilename)
{
  //
  // All of the Pcap enable functions vector through here including the ones
  // that are wandering through all of devices on perhaps all of the nodes in
  // the system.  We can only deal with devices of type CsmaNetDevice.
  //
  Ptr<CsmaNetDevice> device = nd->GetObject<CsmaNetDevice> ();

  PcapHelper pcapHelper;

  std::string filename;
  if (explicitFilename)
    {
      filename = prefix;
    }
  else
    {
      filename = pcapHelper.GetFilenameFromDevice (prefix, device);
    }

  Ptr<PcapFileWrapper> file = pcapHelper.CreateFile (filename, std::ios::out,
                                                     PcapHelper::DLT_EN10MB);
  if (promiscuous)
    {
      pcapHelper.HookDefaultSink<CsmaNetDevice> (device, "PromiscSniffer", file);
    }
  else
    {
      pcapHelper.HookDefaultSink<CsmaNetDevice> (device, "Sniffer", file);
    }
}



void
NetDeviceCombo2::P2pEnablePcapInternal (std::string prefix, Ptr<NetDevice> nd, bool promiscuous, bool explicitFilename)
{
  //
  // All of the Pcap enable functions vector through here including the ones
  // that are wandering through all of devices on perhaps all of the nodes in
  // the system.  We can only deal with devices of type PointToPointNetDevice.
  //
  Ptr<PointToPointNetDevice> device = nd->GetObject<PointToPointNetDevice> ();


  PcapHelper pcapHelper;

  std::string filename;
  if (explicitFilename)
    {
      filename = prefix;
    }
  else
    {
      filename = pcapHelper.GetFilenameFromDevice (prefix, device);
    }

  Ptr<PcapFileWrapper> file = pcapHelper.CreateFile (filename, std::ios::out,
                                                     PcapHelper::DLT_PPP);
  pcapHelper.HookDefaultSink<PointToPointNetDevice> (device, "PromiscSniffer", file);
}

void NetDeviceCombo2::startpcap(QString baseName)
{
    baseName = baseName.replace(":","_");
    std::string devname = m_nd->GetInstanceTypeId().GetName();
    if (devname == "ns3::PointToPointNetDevice")
        P2pEnablePcapInternal(baseName.toStdString(),m_nd,true,false);
    else if (devname == "ns3::WifiNetDevice")
        WifiEnablePcapInternal(baseName.toStdString(),m_nd,true,false);

    else if (devname == "ns3::WimaxNetDevice")
        WimaxEnablePcapInternal(baseName.toStdString(),m_nd,true,false);

    else if (devname == "ns3::CsmaNetDevice")
        CsmaEnablePcapInternal(baseName.toStdString(),m_nd,true,false);

    else if(devname == "ns3::LoopbackNetDevice")
    {
        QMessageBox msg;
        msg.setText("Sorry Pcaps on Loopback not available");
        msg.exec();
    }
}

void NetDeviceCombo2::pcapButton_slot()
{
    QString basename = "logs/Node_" + QString::number(m_nd->GetNode()->GetId()) + "_Mac_" + m_macaddress + "_";
    startpcap(basename);
    pcapButton->setText("Capturing!!!");
    pcapButton->setEnabled(false);
    ns3::Ptr <ns3::PointToPointNetDevice> pnd;

}

void NetDeviceCombo2::p2p_macTxTrace( ns3::Ptr < const ns3::Packet> p)
{
    mp2pcounters["MacTx"]->incStat();
}
void NetDeviceCombo2::p2p_m_macTxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["MacTxDrop"]->incStat();

}
void NetDeviceCombo2::p2p_m_macRxTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["MacRx"]->incStat();

}
void NetDeviceCombo2::p2p_m_macRxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["MacRxDrop"]->incStat();

}
void NetDeviceCombo2::p2p_m_phyTxBeginTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["PhyTxBegin"]->incStat();

}
void NetDeviceCombo2::p2p_m_phyTxEndTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["PhyTxEnd"]->incStat();

}
void NetDeviceCombo2::p2p_m_phyTxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["PhyTxDrop"]->incStat();

}
void NetDeviceCombo2::p2p_m_phyRxEndTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["PhyRxEnd"]->incStat();

}

void NetDeviceCombo2::p2p_m_phyRxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mp2pcounters["PhyRxDrop"]->incStat();

}


void NetDeviceCombo2::wifi_m_macTxTrace(ns3::Ptr<const ns3::Packet> p)
{
    mwificounters["MacTx"]->incStat();


}
void NetDeviceCombo2::wifi_m_macTxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mwificounters["MacTxDrop"]->incStat();

}
void NetDeviceCombo2::wifi_m_macRxTrace(ns3::Ptr<const ns3::Packet> p)
{
    mwificounters["MacRx"]->incStat();

}
void NetDeviceCombo2::wifi_m_macRxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mwificounters["MacRxDrop"]->incStat();


}
void NetDeviceCombo2::wifi_m_macPromiscRxTrace(ns3::Ptr<const ns3::Packet> p)
{
    mwificounters["MacPromiscRx"]->incStat();

}


void NetDeviceCombo2::wimax_m_traceTx(ns3::Ptr<const ns3::Packet> p, const ns3::Mac48Address &mac)
{
    mwimaxcounters["Tx"]->incStat();
}

void NetDeviceCombo2::wimax_m_traceRx(ns3::Ptr<const ns3::Packet> p, const ns3::Mac48Address &mac)
{

    mwimaxcounters["Tx"]->incStat();

}


void NetDeviceCombo2::csma_m_macTxTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["MacTx"]->incStat();


}


void NetDeviceCombo2::csma_m_macTxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["MacTxDrop"]->incStat();


}


void NetDeviceCombo2::csma_m_macTxBackoffTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["MacTxBackoff"]->incStat();


}


void NetDeviceCombo2::csma_m_macRxTrace(ns3::Ptr<const ns3::Packet> p)
{

    mcsmacounters["MacRx"]->incStat();

}

void NetDeviceCombo2::csma_m_macRxDropTrace(ns3::Ptr<const ns3::Packet> p)
{

    mcsmacounters["MacRx"]->incStat();

}


void NetDeviceCombo2::csma_m_macPromiscRxTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["MacPromiscRx"]->incStat();


}

void NetDeviceCombo2::csma_m_phyRxBeginTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["PhyRxBegin"]->incStat();


}

void NetDeviceCombo2::csma_m_phyRxDropTrace(ns3::Ptr<const ns3::Packet> p)
{

    mcsmacounters["PhyRxDrop"]->incStat();

}

void NetDeviceCombo2::csma_m_phyRxEndTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["PhyRxEnd"]->incStat();


}

void NetDeviceCombo2::csma_m_phyTxBeginTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["PhyTxBegin"]->incStat();


}

void NetDeviceCombo2::csma_m_phyTxDropTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["PhyTxDrop"]->incStat();


}

void NetDeviceCombo2::csma_m_phyTxEndTrace(ns3::Ptr<const ns3::Packet> p)
{
    mcsmacounters["PhyTxEnd"]->incStat();


}



void NetDeviceCombo2::AddP2PNetDeviceCombo3(QString traceName)
{
    mp2pcounters[traceName] = new NetDeviceCombo3("P2P NetDevice",traceName,m_macaddress);
    gNetDeviceStats.push_back(mp2pcounters[traceName]);
    ndStatsvbox0->addWidget(mp2pcounters[traceName]->getCentralWidget());

}


void NetDeviceCombo2::AddWifiNetDeviceCombo3(QString traceName)
{
    mwificounters[traceName] = new NetDeviceCombo3("Wifi NetDevice",traceName,m_macaddress);
    gNetDeviceStats.push_back(mwificounters[traceName]);
    ndStatsvbox0->addWidget(mwificounters[traceName]->getCentralWidget());


}

void NetDeviceCombo2::AddWimaxNetDeviceCombo3(QString traceName)
{

    mwimaxcounters[traceName] = new NetDeviceCombo3("Wimax NetDevice",traceName,m_macaddress);
    gNetDeviceStats.push_back(mwimaxcounters[traceName]);
    ndStatsvbox0->addWidget(mwimaxcounters[traceName]->getCentralWidget());

}

void NetDeviceCombo2::AddCsmaNetDeviceCombo3(QString traceName)
{
    mcsmacounters[traceName] = new NetDeviceCombo3("Csma NetDevice",traceName,m_macaddress);
    gNetDeviceStats.push_back(mcsmacounters[traceName]);
    ndStatsvbox0->addWidget(mcsmacounters[traceName]->getCentralWidget());

}


void NetDeviceCombo2::Connectp2pTraces()
{
    NDType->setText("Type:PointToPointNetDevice");
    ns3::Ptr <ns3::PointToPointNetDevice> pnd = DynamicCast<ns3::PointToPointNetDevice> (m_nd);
    NS_ASSERT(pnd);


    AddP2PNetDeviceCombo3("MacTx");
    AddP2PNetDeviceCombo3("MacTxDrop");
    AddP2PNetDeviceCombo3("MacRx");
    AddP2PNetDeviceCombo3("MacRxDrop");
    AddP2PNetDeviceCombo3("PhyTxBegin");
    AddP2PNetDeviceCombo3("PhyTxEnd");
    AddP2PNetDeviceCombo3("PhyTxDrop");
    AddP2PNetDeviceCombo3("PhyRxEnd");
    AddP2PNetDeviceCombo3("PhyRxDrop");


    pnd->TraceConnectWithoutContext("MacTx",MakeCallback(&NetDeviceCombo2::p2p_macTxTrace,this));
    pnd->TraceConnectWithoutContext("MacTxDrop",MakeCallback(&NetDeviceCombo2::p2p_m_macTxDropTrace,this));
    pnd->TraceConnectWithoutContext("MacRx",MakeCallback(&NetDeviceCombo2::p2p_m_macRxTrace,this));
    pnd->TraceConnectWithoutContext("MacRxDrop",MakeCallback(&NetDeviceCombo2::p2p_m_macRxDropTrace,this));
    pnd->TraceConnectWithoutContext("PhyTxBegin",MakeCallback(&NetDeviceCombo2::p2p_m_phyTxBeginTrace,this));
    pnd->TraceConnectWithoutContext("PhyTxEnd",MakeCallback(&NetDeviceCombo2::p2p_m_phyTxEndTrace,this));
    pnd->TraceConnectWithoutContext("PhyTxDrop",MakeCallback(&NetDeviceCombo2::p2p_m_phyTxDropTrace,this));
    pnd->TraceConnectWithoutContext("PhyRxEnd",MakeCallback(&NetDeviceCombo2::p2p_m_phyRxEndTrace,this));
    pnd->TraceConnectWithoutContext("PhyRxDrop",MakeCallback(&NetDeviceCombo2::p2p_m_phyRxDropTrace,this));


}

void NetDeviceCombo2::ConnectwifiTraces()
{
    NDType->setText("Type:WifiNetDevice");
    ns3::Ptr <ns3::WifiNetDevice> pnd = DynamicCast<ns3::WifiNetDevice> (m_nd);
    NS_ASSERT(pnd);


    AddWifiNetDeviceCombo3("MacTx");
    AddWifiNetDeviceCombo3("MacTxDrop");
    AddWifiNetDeviceCombo3("MacPromiscRx");
    AddWifiNetDeviceCombo3("MacRx");
    AddWifiNetDeviceCombo3("MacRxDrop");


    ns3::Ptr <ns3::WifiMac > wifimac = pnd->GetMac();
    NS_ASSERT(wifimac);
    wifimac->TraceConnectWithoutContext("MacTx",MakeCallback(&NetDeviceCombo2::wifi_m_macTxTrace,this));
    wifimac->TraceConnectWithoutContext("MacTxDrop",MakeCallback(&NetDeviceCombo2::wifi_m_macTxDropTrace,this));
    wifimac->TraceConnectWithoutContext("MacPromiscRx",MakeCallback(&NetDeviceCombo2::wifi_m_macPromiscRxTrace,this));
    wifimac->TraceConnectWithoutContext("MacRx",MakeCallback(&NetDeviceCombo2::wifi_m_macRxTrace,this));
    wifimac->TraceConnectWithoutContext("MacRxDrop",MakeCallback(&NetDeviceCombo2::wifi_m_macTxDropTrace,this));



}


void NetDeviceCombo2::ConnectwimaxTraces()
{

    NDType->setText("Type:WimaxNetDevice");
    ns3::Ptr <ns3::WimaxNetDevice> pnd = DynamicCast<ns3::WimaxNetDevice> (m_nd);
    NS_ASSERT(pnd);

    AddWimaxNetDeviceCombo3("Tx");
    AddWimaxNetDeviceCombo3("Rx");

    pnd->TraceConnectWithoutContext("Tx",MakeCallback(&NetDeviceCombo2::wimax_m_traceTx,this));
    pnd->TraceConnectWithoutContext("Rx",MakeCallback(&NetDeviceCombo2::wimax_m_traceRx,this));


}

void NetDeviceCombo2::ConnectCsmaTraces()
{

    NDType->setText("Type:CsmaNetDevice");
    ns3::Ptr <ns3::CsmaNetDevice> pnd = DynamicCast<ns3::CsmaNetDevice> (m_nd);
    NS_ASSERT(pnd);


    AddCsmaNetDeviceCombo3("MacTx");
    AddCsmaNetDeviceCombo3("MacTxDrop");
    AddCsmaNetDeviceCombo3("MacPromiscRx");
    AddCsmaNetDeviceCombo3("MacRx");
    AddCsmaNetDeviceCombo3("MacRxDrop");
    AddCsmaNetDeviceCombo3("MacTxBackoff");
    AddCsmaNetDeviceCombo3("PhyTxBegin");
    AddCsmaNetDeviceCombo3("PhyTxEnd");
    AddCsmaNetDeviceCombo3("PhyTxDrop");
    AddCsmaNetDeviceCombo3("PhyRxBegin");
    AddCsmaNetDeviceCombo3("PhyRxEnd");
    AddCsmaNetDeviceCombo3("PhyRxDrop");


    pnd->TraceConnectWithoutContext("MacTx", MakeCallback(&NetDeviceCombo2::csma_m_macTxTrace,this));
    pnd->TraceConnectWithoutContext("MacTxDrop", MakeCallback(&NetDeviceCombo2::csma_m_macTxDropTrace,this));
    pnd->TraceConnectWithoutContext("MacPromiscRx", MakeCallback(&NetDeviceCombo2::csma_m_macPromiscRxTrace,this));
    pnd->TraceConnectWithoutContext("MacRx", MakeCallback(&NetDeviceCombo2::csma_m_macRxTrace,this));
    pnd->TraceConnectWithoutContext("MacRxDrop", MakeCallback(&NetDeviceCombo2::csma_m_macRxDropTrace,this));
    pnd->TraceConnectWithoutContext("MacTxBackoff", MakeCallback(&NetDeviceCombo2::csma_m_macTxBackoffTrace,this));
    pnd->TraceConnectWithoutContext("PhyTxBegin", MakeCallback(&NetDeviceCombo2::csma_m_phyTxBeginTrace,this));
    pnd->TraceConnectWithoutContext("PhyTxEnd", MakeCallback(&NetDeviceCombo2::csma_m_phyTxEndTrace,this));
    pnd->TraceConnectWithoutContext("PhyTxDrop", MakeCallback(&NetDeviceCombo2::csma_m_phyTxDropTrace,this));
    pnd->TraceConnectWithoutContext("PhyRxBegin", MakeCallback(&NetDeviceCombo2::csma_m_phyRxBeginTrace,this));
    pnd->TraceConnectWithoutContext("PhyRxEnd", MakeCallback(&NetDeviceCombo2::csma_m_phyRxEndTrace,this));
    pnd->TraceConnectWithoutContext("PhyRxDrop", MakeCallback(&NetDeviceCombo2::csma_m_phyRxDropTrace,this));




}

void NetDeviceCombo2::AddDropTailQueueNetDeviceCombo3(QString traceName)
{

    mdqcounters[traceName] = new NetDeviceCombo3("Drop-Tail queue",traceName,m_macaddress);
    gQueueStats.push_back(mdqcounters[traceName]);
    qStatsvbox0->addWidget(mdqcounters[traceName]->getCentralWidget());

}

void NetDeviceCombo2::UpdateQueueStats()
{
    if (!pdq)
        return;

    mdqcounters["NBytes"]->setStat(pdq->GetNBytes());
    mdqcounters["NPackets"]->setStat(pdq->GetNPackets());
    mdqcounters["TotalRecvdBytes"]->setStat(pdq->GetTotalReceivedBytes());
    mdqcounters["TotalRecvdPackets"]->setStat(pdq->GetTotalReceivedPackets());
    mdqcounters["TotalDroppedBytes"]->setStat(pdq->GetTotalDroppedBytes());
    mdqcounters["TotalDroppedPackets"]->setStat(pdq->GetTotalDroppedPackets());

    mdqcounters["NBytes"]->timeToPlot();
    mdqcounters["NPackets"]->timeToPlot();
    mdqcounters["TotalRecvdBytes"]->timeToPlot();
    mdqcounters["TotalRecvdPackets"]->timeToPlot();
    mdqcounters["TotalDroppedBytes"]->timeToPlot();
    mdqcounters["TotalDroppedPackets"]->timeToPlot();


}

void NetDeviceCombo2::AddDropTailTraces()
{

    AddDropTailQueueNetDeviceCombo3("NBytes");
    AddDropTailQueueNetDeviceCombo3("NPackets");

    AddDropTailQueueNetDeviceCombo3("TotalRecvdBytes");
    AddDropTailQueueNetDeviceCombo3("TotalRecvdPackets");

    AddDropTailQueueNetDeviceCombo3("TotalDroppedBytes");
    AddDropTailQueueNetDeviceCombo3("TotalDroppedPackets");



}

void NetDeviceCombo2::ConnectCsmaQueueTraces()
{

    ns3::Ptr <ns3::CsmaNetDevice> pnd = DynamicCast<ns3::CsmaNetDevice> (m_nd);
    NS_ASSERT(pnd);


    Ptr <ns3::Queue> pq = pnd->GetQueue();
    NS_ASSERT (pq);
    if (pq->GetInstanceTypeId().GetName()  == "ns3::DropTailQueue")
       {
          pdq =  DynamicCast<ns3::DropTailQueue> (pq);
          NQueueType->setText("Queue Type:Drop-tail");
          AddDropTailTraces();
       }

}

void NetDeviceCombo2::ConnectP2pQueueTraces()
{

    NDType->setText("Type:PointToPointNetDevice");
    ns3::Ptr <ns3::PointToPointNetDevice> pnd = DynamicCast<ns3::PointToPointNetDevice> (m_nd);
    NS_ASSERT(pnd);


    Ptr <ns3::Queue> pq = pnd->GetQueue();
    NS_ASSERT (pq);
    if (pq->GetInstanceTypeId().GetName()  == "ns3::DropTailQueue")
       {
          pdq =  DynamicCast<ns3::DropTailQueue> (pq);
          NQueueType->setText("Queue Type:Drop-tail");
          AddDropTailTraces();

       }



}

void NetDeviceCombo2::ConnectTraces()
{
    devname = m_nd->GetInstanceTypeId().GetName();
    if (devname == "ns3::PointToPointNetDevice")
       {
           Connectp2pTraces();
           ConnectP2pQueueTraces();
       }
    else if (devname == "ns3::WifiNetDevice")
        ConnectwifiTraces();
    else if (devname == "ns3::WimaxNetDevice")
        ConnectwimaxTraces();
    else if (devname == "ns3::CsmaNetDevice")
        {
            ConnectCsmaTraces();
            ConnectCsmaQueueTraces();
        }
    else if(devname == "ns3::LoopbackNetDevice")
        NDType->setText("Type:LoopbackNetDevice");



}
QWidget * NetDeviceCombo2::getCentralWidget()
{

    return centralWidget;
}

void NetDeviceCombo2::UpdateCsmaCounters()
{

    mcsmacounters["MacTx"]->timeToPlot();
    mcsmacounters["MacTxDrop"]->timeToPlot();
    mcsmacounters["MacPromiscRx"]->timeToPlot();
    mcsmacounters["MacRx"]->timeToPlot();
    mcsmacounters["MacRxDrop"]->timeToPlot();
    mcsmacounters["MacTxBackoff"]->timeToPlot();
    mcsmacounters["PhyTxBegin"]->timeToPlot();
    mcsmacounters["PhyTxEnd"]->timeToPlot();
    mcsmacounters["PhyTxDrop"]->timeToPlot();
    mcsmacounters["PhyRxBegin"]->timeToPlot();
    mcsmacounters["PhyRxEnd"]->timeToPlot();
    mcsmacounters["PhyRxDrop"]->timeToPlot();


}

void NetDeviceCombo2::UpdateWifiCounters()
{
    mwificounters["MacTx"]->timeToPlot();
    mwificounters["MacTxDrop"]->timeToPlot();
    mwificounters["MacPromiscRx"]->timeToPlot();
    mwificounters["MacRx"]->timeToPlot();
    mwificounters["MacRxDrop"]->timeToPlot();

}

void NetDeviceCombo2::UpdateWimaxCounters()
{

    mwimaxcounters["Tx"]->timeToPlot();
    mwimaxcounters["Rx"]->timeToPlot();


}

void NetDeviceCombo2::UpdateP2PCounters()
{


    mp2pcounters["MacTx"]->timeToPlot();
    mp2pcounters["MacTxDrop"]->timeToPlot();
    mp2pcounters["MacRx"]->timeToPlot();
    mp2pcounters["MacRxDrop"]->timeToPlot();
    mp2pcounters["PhyTxBegin"]->timeToPlot();
    mp2pcounters["PhyTxEnd"]->timeToPlot();
    mp2pcounters["PhyTxDrop"]->timeToPlot();
    mp2pcounters["PhyRxEnd"]->timeToPlot();
    mp2pcounters["PhyRxDrop"]->timeToPlot();


}


void NetDeviceCombo2::UpdateNetDevice()
{
    if (devname == "ns3::PointToPointNetDevice")
    {
        UpdateP2PCounters();
        UpdateQueueStats();
    }
    else if (devname == "ns3::CsmaNetDevice")
    {
        UpdateCsmaCounters();
        UpdateQueueStats();

    }
    else if (devname == "ns3::WifiNetDevice")
    {
        UpdateWifiCounters();


    }
    else if (devname == "ns3::WimaxNetDevice")
    {
        UpdateWimaxCounters();

    }
}


NetDeviceCombo3::NetDeviceCombo3(QString appName, QString traceName, QString auxString):
    pgrapher(0),
    theCounter(0),
    plotOn(false),
    m_appName(appName),
    m_traceName(traceName),
    plotCounter(0),
    m_auxString(auxString)

{
    file = new QFile("logs/" + appName.replace(" ","_") + "_" + auxString.replace(":","_") + "_" + traceName + ".txt");
    pCounterLabel = new QLabel;
    centralWidget = new QWidget;
    hbox0 = new QHBoxLayout;
    hbox0->addWidget(pCounterLabel);
    pCounterLabel->setText(m_traceName + ":" + QString::number(theCounter) );
    plotButton = new QPushButton ("Plot");
    connect(plotButton, SIGNAL (clicked()),this,SLOT(plotSlot()));
    hbox0->addWidget(plotButton);
    centralWidget->setLayout(hbox0);

}


void NetDeviceCombo3::OpenFile(QPointer <QFile> f)
{
    if (f->isOpen())
        return;
    if (!f->open(QIODevice::WriteOnly))
    {
        std::cout<<"Unable to open:"<<f->fileName().toAscii().data()<<endl;
        exit(1);
    }


}
void NetDeviceCombo3::AppendToFile(QPointer <QFile> f,QString str)
{
    OpenFile (f);

    if(f->write(str.toAscii().data()) == -1)
    {

        std::cout<<"Write Failed:"<<endl;
        exit(1);
    }
    f->flush();
}


QWidget * NetDeviceCombo3::getCentralWidget()
{
    return centralWidget;
}


TwoDGrapherView* NetDeviceCombo3::getTwodGrapher()
{

    return pgrapher;
}

void NetDeviceCombo3::incStat()
{
    theCounter++;
    plotCounter++;
}


void NetDeviceCombo3::setStat(double t)
{
    theCounter = t;
    plotCounter = t;
}

void NetDeviceCombo3::timeToPlot()
{
    if(pgrapher)
    pgrapher->AppendDataPoint(Simulator::Now().GetSeconds(),plotCounter);
    plotCounter = 0;
    pCounterLabel->setText(m_traceName + ":" + QString::number(theCounter) );
    if(gLogToFile)
        AppendToFile(file,QString::number(Simulator::Now().GetSeconds()) +"\t" + QString::number(plotCounter) + "\n");
}

 void NetDeviceCombo3::plotSlot()
{
    plotOn = !plotOn;
    if(plotOn)
    {
        pgrapher = new TwoDGrapherView(m_appName+":" + m_auxString,"Simulation Time",m_traceName,m_appName+":"+m_auxString);//,this);
        hbox0->addWidget(pgrapher);
        plotButton->setText("Plot Off");
        pgrapher->setMinimumSize(500,500);
    }
    else
    {
        plotButton->setText("Plot");
        delete pgrapher;
        pgrapher = 0;
    }



}


} // namespace netanim

#endif //WITH_NS3
