/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3
#ifndef NETDEVICETAB_H
#define NETDEVICETAB_H

#include "MainInfo/MainInfo.h"
#include <AnimTab/animtab.h>
#include <QtGui>
#include "ns3/core-module.h"
#include "ns3/applications-module.h"
#include "ns3/sgi-hashmap.h"
#include <map>
#include "Graphing/TwodGrapher.h"

using namespace std;

namespace netanim {
class NetDeviceTab;


class NetDeviceCombo3 : public QWidget
{
    Q_OBJECT
public:
    QWidget * getCentralWidget();
    NetDeviceCombo3 (QString appName, QString traceName, QString auxString);
    void OpenFile(QPointer <QFile> f);
    void AppendToFile(QPointer <QFile> f,QString str);
    TwoDGrapherView * getTwodGrapher();
    void incStat();
    void setStat(double t);
    QPointer <QLabel> pCounterLabel;
    void timeToPlot();
private:
    uint32_t theCounter;
    uint32_t plotCounter;
    QWidget * centralWidget;
    TwoDGrapherView * pgrapher;
    bool plotOn;
    QPointer <QPushButton> plotButton;

    QString m_appName;
    QString m_traceName;
    QString m_auxString;
    QPointer <QHBoxLayout> hbox0 ;
    QPointer <QFile> file;
private slots:
     void plotSlot();

};

class NetDeviceCombo2 : public QWidget
{
    Q_OBJECT
public:
    NetDeviceCombo2(ns3::Ptr <ns3::NetDevice> nd, MainInfo *mi);



    void UpdateNetDevice();
    void UpdateP2PCounters();
    void UpdateCsmaCounters();
    void UpdateWifiCounters();
    void UpdateWimaxCounters();

    QWidget * getCentralWidget();
    void ConnectTraces();
    void Connectp2pTraces();
    void ConnectwifiTraces();
    void ConnectwimaxTraces();
    void ConnectCsmaTraces();
    void ConnectCsmaQueueTraces();
    void ConnectP2pQueueTraces();


    // p2p callbacks
    void p2p_macTxTrace( ns3::Ptr < const ns3::Packet> p);
    void p2p_m_macTxDropTrace(ns3::Ptr<const ns3::Packet> p) ;
    void p2p_m_macRxTrace(ns3::Ptr<const ns3::Packet> p) ;
    void p2p_m_macRxDropTrace(ns3::Ptr<const ns3::Packet> p) ;
    void p2p_m_phyTxBeginTrace(ns3::Ptr<const ns3::Packet> p) ;
    void p2p_m_phyTxEndTrace(ns3::Ptr<const ns3::Packet> p) ;
    void p2p_m_phyTxDropTrace(ns3::Ptr<const ns3::Packet> p) ;
    void p2p_m_phyRxEndTrace(ns3::Ptr<const ns3::Packet> p) ;
    void p2p_m_phyRxDropTrace(ns3::Ptr<const ns3::Packet> p) ;
    typedef std::map <QString,NetDeviceCombo3 *> mp2pcounters_t;
    mp2pcounters_t mp2pcounters;
    void AddP2PNetDeviceCombo3 (QString traceName);

    // wifi callbacks
    void wifi_m_macTxTrace (ns3::Ptr<const ns3::Packet> p) ;
    void wifi_m_macTxDropTrace (ns3::Ptr<const ns3::Packet> p);
    void wifi_m_macPromiscRxTrace (ns3::Ptr<const ns3::Packet> p) ;
    void wifi_m_macRxTrace (ns3::Ptr<const ns3::Packet> p) ;
    void wifi_m_macRxDropTrace (ns3::Ptr<const ns3::Packet> p) ;

    typedef std::map <QString,NetDeviceCombo3 *> mwificounters_t;
    mwificounters_t mwificounters;
    void AddWifiNetDeviceCombo3 (QString traceName);


    // wimax callbacks
    void wimax_m_traceRx(ns3::Ptr<const ns3::Packet> p, const ns3::Mac48Address& mac) ;
    void wimax_m_traceTx(ns3::Ptr<const ns3::Packet> p , const ns3::Mac48Address& mac) ;

    typedef std::map <QString, NetDeviceCombo3 *> mwimaxcounters_t;
    mwimaxcounters_t mwimaxcounters;
    void AddWimaxNetDeviceCombo3 (QString traceName);

    // csma callbacks

    void csma_m_macTxTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_macTxDropTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_macPromiscRxTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_macRxTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_macRxDropTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_macTxBackoffTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_phyTxBeginTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_phyTxEndTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_phyTxDropTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_phyRxBeginTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_phyRxEndTrace(ns3::Ptr<const ns3::Packet> p) ;
    void csma_m_phyRxDropTrace(ns3::Ptr<const ns3::Packet> p) ;

    typedef std::map <QString, NetDeviceCombo3 *> mcsmacounters_t;
    mcsmacounters_t mcsmacounters;
    void AddCsmaNetDeviceCombo3 (QString traceName);


    void AddDropTailQueueNetDeviceCombo3 (QString traceName);
    typedef std::map <QString, NetDeviceCombo3 *> mdqcounters_t;
    mdqcounters_t mdqcounters;
    void AddDropTailTraces();
    void UpdateQueueStats();



private:
    QPointer <QGroupBox> netDeviceStats_gb;
    QPointer <QGroupBox> queueStats_gb;
    ns3::Ptr <ns3::DropTailQueue> pdq;
    std::string devname;
    QString m_macaddress;
    QPointer <QVBoxLayout> vbox0;
    QPointer <QVBoxLayout> ndStatsvbox0;
    QPointer <QVBoxLayout> qStatsvbox0;

    QPointer <QLabel> MacLabel;
    QPointer <QPushButton> pcapButton;
    QPointer <QLabel> NDType;
    QPointer <QLabel> NQueueType;
    MainInfo * mainInfo;
    ns3::Ptr <ns3::NetDevice > m_nd;
    QWidget * centralWidget;

    void startpcap (QString baseName);
    void P2pEnablePcapInternal (std::string prefix, ns3::Ptr<ns3::NetDevice> nd, bool promiscuous, bool explicitFilename);
    void CsmaEnablePcapInternal (std::string prefix, ns3::Ptr<ns3::NetDevice> nd, bool promiscuous, bool explicitFilename);
    void WifiEnablePcapInternal (std::string prefix, ns3::Ptr<ns3::NetDevice> nd, bool promiscuous, bool explicitFilename);
    void WimaxEnablePcapInternal (std::string prefix, ns3::Ptr<ns3::NetDevice> nd, bool promiscuous, bool explicitFilename);

 private slots:
    void pcapButton_slot ();
};






class NetDeviceCombo : public QWidget
{
    Q_OBJECT
    public:

        NetDeviceCombo(NetDeviceTab * mndt, uint32_t nodeId,MainInfo *);
        ~NetDeviceCombo();
        QPointer <QPushButton> getButton();
        QWidget * getCentralWidget();
        bool isactive();
        void SetLogtoFile(bool logit);
        void UpdateNetDevices();
        void PrintInterface();
    private:

        QPointer <QLabel> MacIPLabel;
        QPointer <QGroupBox> gb;
        std::vector < QPointer <NetDeviceCombo2> > vNetDeviceCombo2s;

        bool logtoFile;

        QPointer <QPushButton> nodeButton;
        QPointer <QVBoxLayout> vbox0;
        QPointer <QLabel> nodeLabel;
     //   QPointer <QLabel> NApplicationsLabel;

        ns3::Ptr <ns3::Node> n;
        uint32_t NNetDevices;


        NetDeviceTab * m_ndt;
        uint32_t m_nodeId;
        bool active;

        QWidget * centralWidget;
        MainInfo * mainInfo;


    public:


    public slots:
        void onbuttonToggled ();

};



class NetDeviceTab : public AnimTab
{
    Q_OBJECT
public:
    NetDeviceTab( QObject *parent = 0);
    QWidget * Create (QApplication * app, MainInfo *mi);

    QString GetTabName();
    QWidget * getCentralWidget();
    void onFocus();
    void onLostFocus();
    void NotifyToggled(QPointer <NetDeviceCombo>, uint32_t,bool active,QWidget *);
    void UpdateNetDevices();
    typedef std::map <uint32_t,QPointer <NetDeviceCombo> > mNDType;
    void AdjustGrid();


    QPointer <QHBoxLayout> optionsHBox;
    QPointer <QCheckBox> showNetDeviceStats_cb;
    QPointer <QCheckBox> showQueueStats_cb;


    void ShowNetDeviceStats(bool s);
    void ShowQueueStats(bool s);

public slots:
    void showNetDeviceStats_slot(int state);
    void showQueueStats_slot(int state);

private:
    MainInfo * mainInfo;
    uint32_t numNodes;
    QPointer <QVBoxLayout> vbox0;

    // Hbox 0
    QPointer <QHBoxLayout> hbox0;
    QPointer <QPushButton> logtoFileButton;
    QPointer <QLineEdit> pollLineEdit;
    QPointer <QPushButton> applyButton;
    QPointer <QGroupBox> polling_gb;
    QPointer <QRadioButton> wallClock_rb;
    QPointer <QRadioButton> simulationTime_rb;

    //Polling
    bool polling;
    bool simtimePolling;
    QString pollString;
    void DoSimTimePolling();
    void DoWallClockPolling();
    QPointer <QTimer> wallClocktimer;
    ns3::Timer simtimePollingTimer;

    // Gridbox 2

    QPointer <QGridLayout> gridbox2;
    void AddNetDeviceCombos();
    NetDeviceCombo * AddNetDeviceCombo(uint32_t nodeId);
    std::map <uint32_t,QPointer <NetDeviceCombo> > mNetDeviceCombo;


    // Nodes Hbox
    QPointer <QHBoxLayout> nodesHBox;
    QPointer <QScrollArea> nodesSA;



    QWidget * centralWidget;





public slots:

    void apply();
    void logtofileslot();
    void wallclockrb_slot(bool state);
    void simtimerb_slot(bool state);



};

} // namespace netanim
#endif // NETDEVICETAB_H
#endif  //WITH_NS3
