#ifdef WITH_NS3
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include <QtGui>
#include "ns3/network-module.h"
#include "ns3/core-module.h"
#include "ns3/internet-module.h"
#include "routingtab.h"
#include "Animator/NetView.h"
#include "staticroute.h"

using namespace ns3;


namespace netanim {

RoutingTab::RoutingTab(QObject *parent) :
    AnimTab(parent),
    showroutingtable (true),
    showipv4(true),
    showinterfaces(true),
    polling(false),
    logtoFile(false),
    simtimepolling(true),
    simtimepolling_timer(ns3::Timer::CANCEL_ON_DESTROY),
    filtering(false),
    modifying(false)
{
    timer = new QTimer();
}

void RoutingTab::onFocus()
{
    hbox0->insertWidget(0,mainInfo->GetCommonVCR());
    mainInfo->GetCommonVCR()->show();
}

void RoutingTab::onLostFocus()
{

}

QWidget * RoutingTab::getCentralWidget()
{
    return centralWidget;
}


QWidget * RoutingTab::Create(QApplication *app, MainInfo *mi)
{
    (void)app;
    mainInfo = mi;
    numNodes = ns3::NodeList::GetNNodes();

    vbox = new QVBoxLayout;
    vbox->setSpacing(0);
    centralWidget = new QWidget;

    hbox0 = new QHBoxLayout;
    hbox0->addWidget(mi->GetCommonVCR());


    logtoFileButton = new QPushButton("Log to File");
    logtoFileButton->setToolTip("Toggle this button to Log the statistics to the logs folder");
    logtoFileButton->setCheckable(true);
    connect(logtoFileButton, SIGNAL (clicked()), this, SLOT (logtofileslot()));


    QPointer <QLabel> label = new QLabel ("Poll Interval in ms");
    lineEdit = new QLineEdit;
    lineEdit->setText("1000");
    lineEdit->setMinimumWidth(50);
    label->setBuddy(lineEdit);


    applyButton = new QPushButton(("&Stopped"));
    applyButton->setCheckable(true);
    connect(applyButton, SIGNAL (clicked ()),this,SLOT(apply()));

    QPointer <QPushButton> force_update_button = new QPushButton("Force Update");
    connect (force_update_button,SIGNAL (clicked()),this, SLOT(force_update_slot()));

    QPointer <QPushButton> stopButton = new QPushButton("&Stop");
    connect(stopButton, SIGNAL (clicked ()),this,SLOT(stop()));

    hbox0->addWidget(logtoFileButton);
    hbox0->addWidget(label);
    hbox0->addWidget(lineEdit);


    polling_gb = new QGroupBox;
    WallClock_rb = new QRadioButton("WallClock");
    WallClock_rb->setToolTip("Use Wall Clock Time for polling");
    connect(WallClock_rb, SIGNAL(clicked(bool)),this, SLOT(wallclockrb_slot(bool)));
    QPointer <QHBoxLayout> pollinggb_hbox = new QHBoxLayout;
    SimulationTime_rb = new QRadioButton("SimTime");
    SimulationTime_rb->setToolTip("Use Simulation Time for polling");
    SimulationTime_rb->setChecked(true);
    connect(SimulationTime_rb, SIGNAL (clicked(bool)),this,SLOT(simtimerb_slot(bool)));
    pollinggb_hbox->addWidget(SimulationTime_rb);
    pollinggb_hbox->addWidget(WallClock_rb);
    polling_gb->setLayout(pollinggb_hbox);


    hbox0->addWidget(polling_gb);
    hbox0->addWidget(applyButton);
    hbox0->addWidget(force_update_button);

    hbox1 = new QHBoxLayout;


    optionsHbox = new QHBoxLayout;
    routingtable_cb = new QCheckBox ("Routing Table");
    routingtable_cb->setChecked(true);
    connect(routingtable_cb, SIGNAL (stateChanged (int)), this, SLOT(showroutingtable_slot(int)));
    ipv4l3protocol_cb = new QCheckBox ("Ipv4 l3");
    ipv4l3protocol_cb->setChecked(true);
    connect(ipv4l3protocol_cb, SIGNAL (stateChanged (int)), this, SLOT (showipv4l3_slot(int)));

    interfaces_cb = new QCheckBox ("Interfaces");
    interfaces_cb->setChecked(true);
    connect(interfaces_cb, SIGNAL (stateChanged (int)), this, SLOT (showinterfaces_slot(int)));

    filter_cb = new QCheckBox("Filters");
    connect(filter_cb, SIGNAL (stateChanged (int)), this, SLOT (filter_cb_slot(int)));


    modifyRoute_cb = new QCheckBox("Modify Route");
    connect(modifyRoute_cb, SIGNAL (stateChanged (int)), this, SLOT (modifyRoute_cb_slot(int)));

    optionsHbox->addWidget(routingtable_cb);
    optionsHbox->addWidget(ipv4l3protocol_cb);
    optionsHbox->addWidget(interfaces_cb);
    optionsHbox->addWidget(filter_cb);
    optionsHbox->addWidget(modifyRoute_cb);

    gridbox2 = new QGridLayout;
    AddRoutingCombos();




    vbox->addLayout(hbox0);
    vbox->addLayout(optionsHbox);
    pktfilter  = new PacketFilter (false,true,false);
    vbox->addWidget(pktfilter->getCentralWidget());


    vbox->addWidget(sa);
    vbox->addLayout(gridbox2);

    centralWidget->setLayout(vbox);
    filter_cb_slot(0);
    modifyRoute_cb_slot(0);


    return centralWidget;

}

void RoutingTab::force_update_slot()
{
    forceUpdateRT();
}

void RoutingTab::filter_cb_slot(int state)
{
    if (filter_cb->isChecked())
        filtering = true;
    else
        filtering = false;

    if (filtering)
    {
        pktfilter->getCentralWidget()->setVisible(true);
        mainInfo->setPktFilter(pktfilter);
    }
    else
    {
        pktfilter->getCentralWidget()->setVisible(false);
        mainInfo->resetPktFilter();
    }


}
void RoutingTab::modifyRoute_cb_slot(int state)
{
    if (modifyRoute_cb->isChecked())
    {
        modifying = true;
        for (int i = 0 ; i< mRoutingCombo.size(); i++)
        {
           QPointer <RoutingCombo> p = mRoutingCombo[i];
           p->ShowModifyRoute(true);

        }

    }
    else
    {
        modifying = false;
        for (int i = 0 ; i< mRoutingCombo.size(); i++)
        {
           QPointer <RoutingCombo> p = mRoutingCombo[i];
           p->ShowModifyRoute(false);

        }
    }



}


void RoutingTab::wallclockrb_slot(bool state)
{
    if(WallClock_rb->isChecked())
        simtimepolling = false;
}

void RoutingTab::simtimerb_slot(bool state)
{
    if(SimulationTime_rb->isChecked())
        simtimepolling = true;
}

void RoutingTab::logtofileslot()
{
    logtoFile = !logtoFile;
    for (int i = 0 ; i< mRoutingCombo.size(); i++)
    {
       QPointer <RoutingCombo> p = mRoutingCombo[i];
       p->SetLogtoFile(logtoFile);

    }

}

void RoutingTab::showinterfaces_slot(int state)
{
    if(interfaces_cb->isChecked())
        showinterfaces = true;
    else
        showinterfaces = false;
    for (int i = 0 ; i< mRoutingCombo.size(); i++)
    {
       QPointer <RoutingCombo> p = mRoutingCombo[i];
       p->ShowInterfaces(showinterfaces);

    }

}

void RoutingTab::showroutingtable_slot(int state)
{
    if (routingtable_cb->isChecked())
        showroutingtable = true;
    else
        showroutingtable = false;

    for (int i = 0 ; i< mRoutingCombo.size(); i++)
    {
       QPointer <RoutingCombo> p = mRoutingCombo[i];
       p->ShowRoutingTable(showroutingtable);

    }
}

void RoutingTab::showipv4l3_slot(int state)
{
    if (ipv4l3protocol_cb->isChecked())
        showipv4 = true;
    else
        showipv4 = false;
    for (int i = 0 ; i < mRoutingCombo.size(); i++)
    {
       QPointer <RoutingCombo> p = mRoutingCombo[i];
       if(p)
       p->ShowIpv4l3(showipv4);

    }

}
void RoutingTab::forceUpdateRT()
{
    for (mRType::iterator i = mRoutingCombo.begin();
     i != mRoutingCombo.end();
     i++)
    {
        QPointer <RoutingCombo> rcmb = i->second;
        rcmb->PrintRoutingTable();
        rcmb->PringIpv4L3Stats();
        rcmb->PrintInterface();


    }

}
void RoutingTab::updateRT()
{
    if(mainInfo->IsSimulationPaused())
        return;
   for (mRType::iterator i = mRoutingCombo.begin();
    i != mRoutingCombo.end();
    i++)
   {
       QPointer <RoutingCombo> rcmb = i->second;
       rcmb->PrintRoutingTable();
       rcmb->PringIpv4L3Stats();
       rcmb->PrintInterface();


   }
}

void RoutingTab::DoSimTimePolling()
{
     mainInfo->getMainWindow()->SetLCD();
     static bool first = true;
     if (first)
     {
        QMessageBox msg;
        msg.setText("Please ensure you used \"Simulator::Stop (Seconds (Time_t))\" function\nOr your simulation can run endlessly");
        msg.exec();
     }

     if(simtimepolling_timer.IsRunning())
     simtimepolling_timer.Cancel();
     if(!polling || (Simulator::IsFinished() && !first))
         return;
     if(Simulator::IsFinished() && !first)
     {
        polling = false;
        QMessageBox msg;
        msg.setText("Simulation Finished");
        msg.exec();
        applyButton->setText("Stopped");

        return;
     }
     simtimepolling_timer.SetFunction (&netanim::RoutingTab::DoSimTimePolling,this);
     simtimepolling_timer.Schedule(ns3::MilliSeconds(pollString.toInt()));
     updateRT();
     first = false;
}


void RoutingTab::DoWallClockPolling()
{
    mainInfo->getMainWindow()->SetLCD();
    if(timer->isActive())
        timer->stop();
    if(!polling)
        return;

    timer->setInterval(pollString.toInt());
    connect(timer, SIGNAL(timeout()), this, SLOT(updateRT()));
    timer->start();

}


void RoutingTab::apply()
{
    polling = !polling;
    pollString = lineEdit->text();

    if (simtimepolling)
        DoSimTimePolling();
    else
        DoWallClockPolling();


    if (!polling)
    {
        applyButton->setText("Stopped");
        polling_gb->setEnabled(true);
        lineEdit->setEnabled(true);
        return;
    }
    else
    {
        applyButton->setText("Polling");
        polling_gb->setEnabled(false);
        lineEdit->setEnabled(false);

    }

}

void RoutingTab::stop()
{
    if(timer->isActive())
        timer->stop();
}

void RoutingTab::AddRoutingCombos()
{
    sa = new QScrollArea;
    for (uint32_t i = 0; i < numNodes ; i++)
    {
        QPointer <RoutingCombo> rcmb = AddRoutingCombo(i);
        hbox1->addWidget(rcmb->getButton());
    }
    QWidget * qw = new QWidget;
    qw->setLayout(hbox1);
    sa->setFixedHeight(50);
    sa->setWidget(qw);

}

RoutingCombo * RoutingTab::AddRoutingCombo(uint32_t nodeId)
{
    RoutingCombo * rcmb = new RoutingCombo(this,nodeId,mainInfo);
    vRoutingCombo.push_back(rcmb);
    mRoutingCombo[nodeId] = rcmb;
    return rcmb;

}

QString RoutingTab::GetTabName ()
{
    return "Routing";
}

void RoutingTab::AdjustGrid()
{
    typedef std::map <uint32_t,QPointer <RoutingCombo> > RCMap_t;
    for (RCMap_t::iterator i = mRoutingCombo.begin();
         i != mRoutingCombo.end();
         i++)
    {

        QPointer <RoutingCombo> p = i->second;
        gridbox2->removeWidget(p->getCentralWidget());
        p->getCentralWidget()->hide();
    }

    for (RCMap_t::iterator i = mRoutingCombo.begin();
         i != mRoutingCombo.end();
         i++)
    {
        QPointer <RoutingCombo> p = i->second;
        if(p->isactive())
        {
            int total = gridbox2->count();
            int r = gridbox2->rowCount();
            int maxcolumn = 4;
            if ((total % maxcolumn) == 0)
             gridbox2->addWidget(p->getCentralWidget(),r+1,0,1,1);
            else
                gridbox2->addWidget(p->getCentralWidget(),r-1,(total % maxcolumn ),1,1);

            p->getCentralWidget()->show();
        }

    }


}

void RoutingTab::NotifyToggled(QPointer <RoutingCombo> rcmb, uint32_t NodeId,bool active, QWidget * w)
{


   int index=0;
   if ((index=gridbox2->indexOf(w)) == -1)
   {
   // lets add this widget
       if(active)
         {
            int total = gridbox2->count();
            int r = gridbox2->rowCount();
            int maxcolumn = 4;
            if ((total % maxcolumn) == 0)
             gridbox2->addWidget(w,r+1,0,1,1);
            else
                gridbox2->addWidget(w,r-1,(total % maxcolumn ),1,1);

         }
       mRoutingCombo[NodeId] = rcmb;
   }
   if(!active)
   {

        gridbox2->removeWidget(w);
        w->setVisible(false);
        AdjustGrid();

   }
   else
   {
        w->setVisible(true);
   }

   rcmb->ShowRoutingTable(showroutingtable);
   rcmb->ShowIpv4l3(showipv4);
   rcmb->ShowInterfaces(showinterfaces);

}

RoutingCombo::RoutingCombo(RoutingTab *rt, uint32_t nodeId,MainInfo * mi):
    mainInfo(mi),
    logtoFile(false)
{
    m_rt = rt;
    m_nodeId = nodeId;
    nodeButton = new QPushButton(QString::number(nodeId));
    nodeButton->setCheckable(true);
    connect( nodeButton, SIGNAL(clicked()), this, SLOT(onbuttonToggled()) );

    vbox = new QVBoxLayout;
    vbox->setSpacing(0);
    centralWidget = new QWidget;
    label = new QLabel(QString::number(nodeId));
    te = new QTextEdit();
    te->setMinimumSize(350,150);
    te->setLineWrapMode(QTextEdit::NoWrap);

    ipv4l3protocol_gb = new QGroupBox;
    ipv4l3protocol_gb->setTitle("Ipv4 L3");

    ipv4l3gb_vbox = new QVBoxLayout;
    ipv4l3protocol_gb->setLayout(ipv4l3gb_vbox);
    InitIpv4L3Stats();


    interfaces_vbox = new QVBoxLayout;
    interfaces_gb = new QGroupBox;
    interfaces_gb->setTitle("Interfaces");
    interfaces_gb->setLayout(interfaces_vbox);
    InitInterfaceStats();


    modifyRoute_vbox = new QVBoxLayout;
    modifyRoute_gb = new QGroupBox;
    modifyRoute_gb->setTitle("Modify Routes");
    modifyRoute_gb->setLayout(modifyRoute_vbox);
    randomEcmp_cb = new QCheckBox;
    randomEcmp_cb->setText("RandomECMP");
    connect(randomEcmp_cb, SIGNAL (stateChanged (int)), this, SLOT (randomEcmp_cb_slot(int)));


    addHostRoute_button = new QPushButton("Add Static Host Route");
    connect(addHostRoute_button, SIGNAL (clicked ()), this , SLOT (addHostRoute_button_slot()));
    addHostRoute_button->setMaximumWidth(250);


    addNetworkRoute_button = new QPushButton("Add Static Network Route");
    connect(addNetworkRoute_button, SIGNAL (clicked ()), this , SLOT (addNetworkRoute_button_slot()));
    addNetworkRoute_button->setMaximumWidth(250);


    addDefaultRoute_button = new QPushButton("Set Default Route");
    connect(addDefaultRoute_button, SIGNAL (clicked ()), this , SLOT (addDefaultRoute_button_slot()));
    addDefaultRoute_button->setMaximumWidth(250);


    removeRoute_le = new QLineEdit("");
    removeRoute_le->setMaximumWidth(50);
    removeRoute_label = new QLabel ("Route Index");
    removeRoute_button = new QPushButton("Remove Route");
    connect(removeRoute_button, SIGNAL (clicked ()), this, SLOT (removeRoute_button_slot()));


    bool grExists = InitModifyRoute();
    if (grExists)
    {
      modifyRoute_vbox->addWidget(randomEcmp_cb);
    }
      modifyRoute_vbox->addWidget(addHostRoute_button);
      modifyRoute_vbox->addWidget(addNetworkRoute_button);
      modifyRoute_vbox->addWidget(addDefaultRoute_button);


    QHBoxLayout * removeRouteHbox = new QHBoxLayout;
    removeRouteHbox->addWidget(removeRoute_label);
    removeRouteHbox->addWidget(removeRoute_le);
    removeRouteHbox->addWidget(removeRoute_button);
    modifyRoute_vbox->addLayout(removeRouteHbox);


    vbox->addWidget(label);
    vbox->addWidget(modifyRoute_gb);
    vbox->addWidget(te);
    vbox->addWidget(ipv4l3protocol_gb);
    vbox->addWidget(interfaces_gb);


    centralWidget->setLayout(vbox);
    active = false;
    InitRoutingTable();
}

RoutingCombo::~RoutingCombo()
{

    if (ipv4l3_file->isOpen())
        ipv4l3_file->close();
    if (routingtbl_file->isOpen())
        routingtbl_file->close();

}

void RoutingCombo::randomEcmp_cb_slot(int state)
{
    if (randomEcmp_cb->isChecked())
        GetGlobalRouting()->SetAttributeFailSafe("RandomEcmpRouting", BooleanValue(true));
    else
        GetGlobalRouting()->SetAttributeFailSafe("RandomEcmpRouting", BooleanValue(false));


}


void RoutingCombo::addHostRoute_button_slot()
{
    StaticRoute * sr = new StaticRoute (0,NodeList::GetNode(m_nodeId),"Add Host Route");
   sr->getCentralWidget()->show();


}

void RoutingCombo::addNetworkRoute_button_slot()
{
    StaticRoute * sr = new StaticRoute (1,NodeList::GetNode(m_nodeId), "Add Network Route");
   sr->getCentralWidget()->show();


}

void RoutingCombo::addDefaultRoute_button_slot()
{
    StaticRoute * sr = new StaticRoute (2,NodeList::GetNode(m_nodeId), "Set Default Route");
   sr->getCentralWidget()->show();


}
void RoutingCombo::removeRoute_button_slot()
{
    if (removeRoute_le->text() == "")
        return;
    ns3::Ptr <ns3::Ipv4> ipv4 = NodeList::GetNode(m_nodeId)->GetObject <ns3::Ipv4> ();
    ns3::Ipv4StaticRoutingHelper srhelper;
    srhelper.GetStaticRouting(ipv4)->RemoveRoute(removeRoute_le->text().toUInt());

}


void RoutingCombo::SetLogtoFile(bool logit)
{
    logtoFile = logit;
}

bool RoutingCombo::isactive()
{
    return active;
}
void RoutingCombo::ShowRoutingTable(bool show)
{
    if(show)
        te->setVisible(true);
    else
        te->setVisible(false);
}

void RoutingCombo::ShowModifyRoute(bool show)
{
    if(show)
        modifyRoute_gb->setVisible(true);
    else
        modifyRoute_gb->setVisible(false);
}


QWidget* RoutingCombo::getCentralWidget()
{
    return centralWidget;
}

void RoutingCombo::ShowIpv4l3(bool show)
{
    if(show)
        ipv4l3protocol_gb->setVisible(true);
    else
        ipv4l3protocol_gb->setVisible(false);

}

void RoutingCombo::ShowInterfaces(bool show)
{
    if(show)
        interfaces_gb->setVisible(true);
    else
        interfaces_gb->setVisible(false);

}

void RoutingCombo::InitRoutingTable()
{
    routingtbl_file = new QFile("logs/" + QString::number(m_nodeId) + "-rttbl.txt");

}

Ptr <ns3::Ipv4GlobalRouting> RoutingCombo::GetGlobalRouting ()
{
    Ptr <ns3::Node> n = NodeList::GetNode(m_nodeId);
    Ptr <ns3::GlobalRouter> gr = n->GetObject<ns3::GlobalRouter> ();
    if (!gr) return 0;
    return gr->GetRoutingProtocol();


}
bool RoutingCombo::InitModifyRoute()
{
    BooleanValue b;
    ns3::Ptr <ns3::Ipv4GlobalRouting> gr = GetGlobalRouting();
    if (!gr) return false;
    gr->GetAttributeFailSafe ("RandomEcmpRouting", b);
    if(b.Get())
        randomEcmp_cb->setChecked(true);
    else
        randomEcmp_cb->setChecked(false);
    return true;

}


void RoutingCombo::InitInterfaceStats()
{
    this->interfaces_ndevices_stat = new QLabel;
    interfaces_ndevices_stat->setToolTip("Number of Devices");

    this->interfaces_vbox->addWidget(interfaces_ndevices_stat);
}

void RoutingCombo::InitIpv4L3Stats()
{

    ipv4l3_file = new QFile("logs/" + QString::number(m_nodeId) + "-ipv4l3.txt");


    this->ipv4l3_drop_stat = new QLabel;
    this->ipv4l3_rx_stat = new QLabel;
    this->ipv4l3_tx_stat = new QLabel;
    this->ipv4l3_sendoutgoing_stat = new QLabel;
    this->ipv4l3_sendoutgoing_stat->setToolTip("Send Outgoing");
    this->ipv4l3_localdeliver_stat = new QLabel;
    this->ipv4l3_localdeliver_stat->setToolTip("Local Deliver");
    this->ipv4l3_unicastforward_stat = new QLabel;
    this->ipv4l3_unicastforward_stat->setToolTip("Unicast Forward");

    this->ipv4l3gb_vbox->addWidget(ipv4l3_tx_stat);
    this->ipv4l3gb_vbox->addWidget(ipv4l3_rx_stat);
    this->ipv4l3gb_vbox->addWidget(ipv4l3_drop_stat);
    this->ipv4l3gb_vbox->addWidget(ipv4l3_sendoutgoing_stat);
    this->ipv4l3gb_vbox->addWidget(ipv4l3_unicastforward_stat);
    this->ipv4l3gb_vbox->addWidget(ipv4l3_localdeliver_stat);

    QString str = "Time\tTx\tRx\tDrop\tSndOut\tUnifwd\tlclDel\n";
    AppendToFile(ipv4l3_file,str);

}

void RoutingCombo::PringIpv4L3Stats()
{
    NodeStatistics stats = mainInfo->GetSimulation()->GetNodesStatistics(m_nodeId);
    this->ipv4l3_drop_stat->setText("Drop  :" + QString::number(stats.ipv4l3protocol_drop));
    this->ipv4l3_rx_stat->setText("Rx    :" + QString::number(stats.ipv4l3protocol_rx));
    this->ipv4l3_tx_stat->setText("Tx    :" + QString::number(stats.ipv4l3protocol_tx));
    this->ipv4l3_sendoutgoing_stat->setText("SndOut:" + QString::number(stats.ipv4l3protocol_sendoutgoing));
    this->ipv4l3_localdeliver_stat->setText("LclDeliver:" + QString::number(stats.ipv4l3protocol_localdeliver));
    this->ipv4l3_unicastforward_stat->setText("UniFwd:" + QString::number(stats.ipv4l3protocol_unicastforward));

    QString str = QString::number(Simulator::Now().GetSeconds()) + "\t"
            + QString::number(stats.ipv4l3protocol_drop) + "\t"
            + QString::number(stats.ipv4l3protocol_rx) + "\t"
            + QString::number(stats.ipv4l3protocol_tx) + "\t"
            + QString::number(stats.ipv4l3protocol_sendoutgoing) + "\t"
            + QString::number(stats.ipv4l3protocol_localdeliver) + "\t"
            + QString::number(stats.ipv4l3protocol_unicastforward) + "\n";

    //std::cout<<"*****"<<str.toAscii().data()<<endl;
    if(logtoFile)
        AppendToFile(ipv4l3_file,str);

}

void RoutingCombo::onbuttonToggled()
{
    active = !active;
    m_rt->NotifyToggled(QPointer <RoutingCombo> (this),m_nodeId, active,centralWidget);
}

QPointer <QPushButton> RoutingCombo::getButton()
{

    return nodeButton;
}

QPointer <QTextEdit> RoutingCombo::getTextEdit()
{
    return te;
}

void RoutingCombo::PrintInterface()
{
    Ptr <ns3::Node> n = ns3::NodeList::GetNode(m_nodeId);
    NS_ASSERT (n);

    NodeStatistics & stats = mainInfo->GetSimulation()->GetNodesStatistics(n->GetId());
    stats.ndevices = n->GetNDevices();
    this->interfaces_ndevices_stat->setText("Devices  :" + QString::number(stats.ndevices));
    if (!stats.ndevices)
            return;
    if (!stats.vMacIPv4Pairs.empty())
        return;
    if ( stats.vMacIPv4Pairs.empty())
    {

        Ptr <ns3::Ipv4> ipv4 = n->GetObject <ns3::Ipv4> ();
        NS_ASSERT (ipv4);
        for (int i = 0 ; i < stats.ndevices ; i++)
        {
            Ipv4InterfaceAddress iaddr = ipv4->GetAddress (i,0);
            Ipv4Address addri = iaddr.GetLocal ();
            Address mac;
            if(i!=0)
            mac = n->GetDevice(i-1)->GetAddress();
            else
            mac = n->GetDevice(stats.ndevices-1)->GetAddress();
            std::stringstream ssip;
            addri.Print(ssip);
            QString ipaddress = ssip.str().c_str();

            std::stringstream ssmac;
            ssmac << mac;
            QString macaddress = ssmac.str().c_str();
            netanim::macipv4pair_t pair = { macaddress, ipaddress};
            //std::cout<<"Mac:"<<macaddress.toAscii().data() <<" IP:"<<ipaddress.toAscii().data()<<endl;
            stats.vMacIPv4Pairs.push_back(pair);

        }
    }

    for (int i = 0 ; i < stats.vMacIPv4Pairs.size() ; i++)
    {
        netanim::macipv4pair_t pair = stats.vMacIPv4Pairs[i];
        vMacIPv4.push_back(new QLabel);
        QPointer <QLabel> p = vMacIPv4[i];
        p->setText(QString::number(i) + ":" + pair.Mac + "/" + pair.Ipv4address);
        p->setToolTip("Device Id / Mac Address / Ipv4 Address");
        interfaces_vbox->addWidget (p);

    }



}

void RoutingCombo::OpenFile(QPointer <QFile> f)
{
    if (f->isOpen())
        return;
    if (!f->open(QIODevice::WriteOnly))
    {
        std::cout<<"Unable to open:"<<f->fileName().toAscii().data()<<endl;
        exit(1);
    }


}
void RoutingCombo::AppendToFile(QPointer <QFile> f,QString str)
{
    OpenFile (f);

    if(f->write(str.toAscii().data()) == -1)
    {

        std::cout<<"Write Failed:"<<endl;
        exit(1);
    }
    f->flush();
}

void RoutingCombo::PrintRoutingTable()
{
    QString pretty = "-------";
    te->append(pretty);
    QString simTime = QString::number(ns3::Simulator::Now().GetSeconds());
    te->append("Time:" + simTime + " s");
    te->append(pretty);

    Ptr <ns3::Node> n = ns3::NodeList::GetNode(m_nodeId);
    NS_ASSERT (n);
    Ptr <ns3::Ipv4> ipv4 = n->GetObject <ns3::Ipv4> ();
    std::stringstream stream;
    Ptr<OutputStreamWrapper> routingstream = Create<OutputStreamWrapper> (&stream);
    ipv4->GetRoutingProtocol ()->PrintRoutingTable (routingstream);
    QString table = stream.str().c_str();
    te->append(table);
    if(logtoFile)
      AppendToFile(routingtbl_file,table);

}
} //namespace

#endif
