#ifdef WITH_NS3
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifndef ROUTINGTAB_H
#define ROUTINGTAB_H

#include <QtGui>
#include <QApplication>
#include <QObject>
#include "MainInfo/MainInfo.h"
#include "stdint.h"
#include "AnimTab/animtab.h"
#include "Filter/packetfilter.h"


namespace netanim {

class RoutingTab;

class RoutingCombo : public QWidget
{
    Q_OBJECT
    public:

        RoutingCombo(RoutingTab * rt, uint32_t nodeId,MainInfo *);
        ~RoutingCombo();
        QPointer <QPushButton> getButton();
        QPointer <QTextEdit> getTextEdit();
        void PrintRoutingTable();
        void PringIpv4L3Stats();
        void PrintInterface();

        void ShowRoutingTable (bool show);
        void ShowModifyRoute (bool show);
        void ShowIpv4l3 (bool show);
        void ShowInterfaces (bool show);
        QWidget * getCentralWidget();
        bool isactive();
        void SetLogtoFile(bool logit);
    private:

        void AppendToFile(QPointer <QFile> f, QString str);
        void OpenFile (QPointer <QFile> f);
        bool logtoFile;


        void InitRoutingTable();
        QPointer <QFile> routingtbl_file;


        void InitIpv4L3Stats();
        QPointer <QFile> ipv4l3_file;
        QPointer <QLabel> ipv4l3_drop_stat;
        QPointer <QLabel> ipv4l3_rx_stat;
        QPointer <QLabel> ipv4l3_tx_stat;
        QPointer <QLabel> ipv4l3_sendoutgoing_stat;
        QPointer <QLabel> ipv4l3_unicastforward_stat;
        QPointer <QLabel> ipv4l3_localdeliver_stat;
        QPointer <QVBoxLayout> ipv4l3gb_vbox;
        QPointer <QGroupBox> ipv4l3protocol_gb;





        QPointer <QPushButton> nodeButton;
        QPointer <QTextEdit> te;


        QPointer <QVBoxLayout> interfaces_vbox;
        void InitInterfaceStats();
        QPointer <QLabel> interfaces_ndevices_stat;
        std::vector <QPointer <QLabel> > vMacIPv4;
        QPointer <QGroupBox> interfaces_gb;



        QPointer <QVBoxLayout> modifyRoute_vbox;
        bool InitModifyRoute ();

        ns3::Ptr <ns3::Ipv4GlobalRouting> GetGlobalRouting ();

        QPointer <QGroupBox> modifyRoute_gb;
        QPointer <QCheckBox> randomEcmp_cb;
        QPointer <QPushButton> addHostRoute_button;
        QPointer <QPushButton> addNetworkRoute_button;
        QPointer <QPushButton> addDefaultRoute_button;

        QPointer <QLabel> removeRoute_label;
        QPointer <QLineEdit> removeRoute_le;
        QPointer <QPushButton> removeRoute_button;


        RoutingTab * m_rt;
        uint32_t m_nodeId;
        bool active;
        QPointer <QVBoxLayout> vbox;
        QPointer <QLabel> label;
        QWidget * centralWidget;

        MainInfo * mainInfo;

    public slots:
        void onbuttonToggled ();
        void randomEcmp_cb_slot(int);
        void addHostRoute_button_slot ();
        void addNetworkRoute_button_slot ();
        void addDefaultRoute_button_slot ();
        void removeRoute_button_slot ();

};

class RoutingTab : public AnimTab
{
    Q_OBJECT
public:
    explicit RoutingTab(QObject *parent = 0);

    QWidget * Create(QApplication * app,MainInfo *mi);
    QString GetTabName();
    void AddRoutingCombos();
    RoutingCombo * AddRoutingCombo(uint32_t nodeId);
    void NotifyToggled(QPointer <RoutingCombo>, uint32_t,bool active,QWidget *);
    typedef std::map <uint32_t,QPointer <RoutingCombo> > mRType;
    QWidget * getCentralWidget();
    void AdjustGrid();

    void onFocus();
    void onLostFocus();
signals:

public slots:
    void DoSimTimePolling();

private:

    QPointer <QGroupBox> polling_gb;
    QPointer <QRadioButton> WallClock_rb;
    QPointer <QRadioButton> SimulationTime_rb;
    QPointer <QPushButton> applyButton;
    QPointer <QPushButton> logtoFileButton;

    QPointer <QVBoxLayout> vbox;
    QPointer <QHBoxLayout> hbox0;
    QPointer <QHBoxLayout> hbox1;
    QPointer <QHBoxLayout> hbox2;
    QPointer <QGridLayout> gridbox2;

    QPointer <QHBoxLayout> optionsHbox;

    QPointer <QScrollArea>  sa;
    QPointer <QLineEdit>   lineEdit;
    MainInfo * mainInfo;
    QWidget * centralWidget;
    uint32_t numNodes;
    std::vector <RoutingCombo *> vRoutingCombo;
    QString pollString;
    QPointer <QTimer> timer;

    std::map <uint32_t,QPointer <RoutingCombo> > mRoutingCombo;

    QPointer <QCheckBox> routingtable_cb;
    QPointer <QCheckBox> ipv4l3protocol_cb;
    QPointer <QCheckBox> interfaces_cb;


    bool showroutingtable;
    bool showipv4;
    bool showinterfaces;


    bool polling;
    bool logtoFile;

    bool simtimepolling;
    void DoWallClockPolling();


    ns3::Timer simtimepolling_timer;


    // Filter ;
    bool filtering;
    bool modifying;
    QPointer <PacketFilter> pktfilter;
    QPointer <QCheckBox> filter_cb;
    QPointer <QCheckBox> modifyRoute_cb;



private slots:
    void force_update_slot();
    void filter_cb_slot(int state);
    void modifyRoute_cb_slot(int state);
    void wallclockrb_slot(bool state);
    void simtimerb_slot(bool state);
    void apply();
    void stop();
    void forceUpdateRT();
    void updateRT();
    void showroutingtable_slot (int state);
    void showipv4l3_slot (int state);
    void showinterfaces_slot (int state);
    void logtofileslot();


};

} // namespace netanim

#endif // ROUTINGTAB_H
#endif // WITH_NS3
