#ifdef WITH_NS3
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include "staticroute.h"
#include "QtNetwork/QHostAddress"
#include "ns3/internet-module.h"

StaticRoute::StaticRoute(int mode, ns3::Ptr <ns3::Node> n,QString title, QWidget *parent) :
    QWidget(parent), m_mode(mode), valid (false), m_n (n)
{

   // mode 0 == Host route
   // mode 1 == Network route
   // mode 2 == Default route
    hbox = new QHBoxLayout;
    dst_le = new QLineEdit ("");
    mask_le = new QLineEdit ("");
    nextHop_le = new QLineEdit("");
    interface_le = new QLineEdit("");
    metric_le = new QLineEdit("");
    applyButton = new QPushButton("Apply");
    connect(applyButton,SIGNAL (clicked()),this,SLOT(applyButton_slot()));

    dst_label = new QLabel ("Destination");
    mask_label = new QLabel ("Mask");
    nextHop_label = new QLabel ("Next Hop");
    interface_label = new QLabel ("Interface Index");
    metric_label = new QLabel ("Metric");


    if (mode ==0 || mode ==1)
    {
      hbox->addWidget(dst_label);
      hbox->addWidget(dst_le);
    }
    if (mode ==1)
    {
        hbox->addWidget(mask_label);
        hbox->addWidget(mask_le);
    }
    hbox->addWidget(nextHop_label);
    hbox->addWidget(nextHop_le);
    hbox->addWidget(interface_label);
    hbox->addWidget(interface_le);
    hbox->addWidget(metric_label);
    hbox->addWidget(metric_le);



    hbox->addWidget(applyButton);
    centralWidget = new QWidget;
    centralWidget->setLayout(hbox);
    centralWidget->setWindowTitle(title);

}


QWidget* StaticRoute::getCentralWidget()
{
    return centralWidget;

}

bool IsIpValid(QString ip)
{
    if (ip == "ANY")
        return true;
    QHostAddress addr ;
    if (addr.setAddress(ip))
        return true;
    else
    {
        QMessageBox msg;
        msg.setText("Invalid IP:"+ip);
        msg.exec();
        return false;
    }
}


void StaticRoute::applyButton_slot()
{

    if (!IsIpValid(dst_le->text()))
        goto end;
    dst_string = dst_le->text();


    if (m_mode == 1)
    {
      if (!IsIpValid(mask_le->text()))
        goto end;
      mask_string = mask_le->text();
    }

    if (nextHop_le->text() != "")
    {
      if (!IsIpValid(nextHop_le->text()))
          goto end;
      nextHop_string = nextHop_le->text();
    }
    else
    {
        nextHop_string = "";

    }
    if (m_mode ==3)
    {
        if (nextHop_string == "")
        {
            QMessageBox msg;
            msg.setText("Invalid nextHop:"+nextHop_string);
            msg.exec();
            goto end;
        }

    }
    interface_string = interface_le->text();
    if (interface_string == "")
    {
        QMessageBox msg;
        msg.setText("Invalid Interface Index:"+interface_string);
        msg.exec();
        goto end;
    }
    metric_string = metric_le->text();

    if (metric_string == "")
    {
        metric_string = "0";
    }

    valid = true;
    if(valid)
    {
        ns3::Ptr <ns3::Ipv4> ipv4 = m_n->GetObject <ns3::Ipv4> ();
        ns3::Ipv4StaticRoutingHelper srhelper;
        if (m_mode == 0)
        {
            if (nextHop_string != "")

                srhelper.GetStaticRouting(ipv4)->AddHostRouteTo
                (
                    ns3::Ipv4Address(dst_string.toAscii().data()),
                    ns3::Ipv4Address(nextHop_string.toAscii().data()),
                    interface_string.toUInt(),
                    metric_string.toUInt()

                 );
            else
                srhelper.GetStaticRouting(ipv4)->AddHostRouteTo
                (
                    ns3::Ipv4Address(dst_string.toAscii().data()),
                    interface_string.toUInt(),
                    metric_string.toUInt()

                 );
        }
        else if (m_mode == 1)
        {
            if (nextHop_string != "")

                srhelper.GetStaticRouting(ipv4)->AddNetworkRouteTo
                (
                    ns3::Ipv4Address(dst_string.toAscii().data()),
                    ns3::Ipv4Mask(mask_string.toAscii().data()),
                    ns3::Ipv4Address(nextHop_string.toAscii().data()),
                    interface_string.toUInt(),
                    metric_string.toUInt()

                 );
            else
                srhelper.GetStaticRouting(ipv4)->AddNetworkRouteTo
                (
                    ns3::Ipv4Address(dst_string.toAscii().data()),
                    ns3::Ipv4Mask(mask_string.toAscii().data()),
                    interface_string.toUInt(),
                    metric_string.toUInt()

                 );
        }
        else if (m_mode ==2)
        {

            srhelper.GetStaticRouting(ipv4)->SetDefaultRoute
            (
                ns3::Ipv4Address(nextHop_string.toAscii().data()),
                interface_string.toUInt(),
                metric_string.toUInt()

             );



        }


    }
    end:
    centralWidget->hide();


}

#endif
