#ifdef WITH_NS3
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Simulation Script Adapted from "first.cc" in ns-3
 */


#include "simulation.h"
#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/internet-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/applications-module.h"
#include "ns3/wifi-module.h"
#include "ns3/flow-monitor-module.h"
#include "ns3/simulator-impl.h"
#include "ns3/mobility-module.h"
#include "animsimulator-impl.h"
#include "ns3/point-to-point-layout-module.h"
#include "Animator/NetView.h"

using namespace ns3;

namespace netanim {

bool Simulation::Paused = false;
bool Simulation::configureSimulationComplete = false;

static MainInfo * gMainInfo = 0;


ArgsParser::ArgsParser(QString args):argc(0)
{
    QStringList arglist = args.split(" ");
    argc = arglist.size();
    int j = 0;
    largv = new (char *);
    for (QStringList::Iterator i = arglist.begin(),j=0;
         i != arglist.end();
         i++,j++ )
    {
        QString str = *i;
        *largv = new char [str.length()+1];
        strcpy(*largv,str.toAscii().data());

    }
}

int ArgsParser::getArgc()
{
    return argc;
}

char ** ArgsParser::getArgv()
{

    return largv;
}

Simulation::Simulation(MainInfo *mi):RunOnce(false),mainInfo(mi)
{
    gMainInfo = mi;
    ns3::Ptr <ns3::AnimSimulatorImpl> pImpl = CreateObject <AnimSimulatorImpl> (mi);
    Simulator::SetImplementation(pImpl);
    ConfigureSimulation(mainInfo->getArgc(),mainInfo->getArgv());
    if (AnimationInterface::IsInitialized())
    {
        QString t = "AnimationInterface object already initialized\n \
                     Please remove any AnimationInterface statements\n \
                     from Simulation::ConfigureSimulation";
        std::cout << t.toAscii().data() <<std::endl;

        exit(-1);

    }
    configureSimulationComplete = true;
    InitializeNodeStatistics();
    pAnim = new ns3::AnimationInterface();
    pAnim->SetXMLOutput();
    pAnim->SetOutputFile("logs/test.xml");

}


bool Simulation::IsConfigureSimulationComplete()
{
    return configureSimulationComplete;
}
void Simulation::AnimationInterfaceCallBack(const char *s)
{
    gMainInfo->view->NotifyAnimationOnline(s);

}

void Simulation::AnimationOnline(bool online)
{
    if (online)
    {
        pAnim->SetAnimWriteCallback(AnimationInterfaceCallBack);
        pAnim->StartAnimation();
    }
    else
        pAnim->ResetAnimWriteCallback();

}


Simulation::~Simulation()
{
   Simulator::Stop();
   Simulator::Destroy();
}

void Simulation::Stop()
{
   Simulator::Stop();
   Simulator::Destroy();
}

void Simulation::InitializeNodeStatistics()
{
   for (NodeList::Iterator i = NodeList::Begin();
        i != NodeList::End();
        i++)
   {

       vNodeStatistics.push_back(NodeStatistics());
   }

}

NodeStatistics & Simulation::GetNodesStatistics(uint32_t index)
{
    return vNodeStatistics[index];

}

std::vector<std::string> Simulation::GetElementsFromContext (std::string context)
{
  std::vector <std::string> elements;
  size_t pos1=0, pos2;
  while (pos1 != context.npos)
  {
    pos1 = context.find ("/",pos1);
    pos2 = context.find ("/",pos1+1);
    elements.push_back (context.substr (pos1+1,pos2-(pos1+1)));
    pos1 = pos2;
    pos2 = context.npos;
  }
  return elements;
}



Ptr <NetDevice>
Simulation::GetNetDeviceFromContext (std::string context)
{
  // Use "NodeList/*/DeviceList/*/ as reference
  // where element [1] is the Node Id
  // element [2] is the NetDevice Id

  std::vector <std::string> elements = GetElementsFromContext (context);
  Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
  NS_ASSERT (n);
  return n->GetDevice (atoi (elements[3].c_str ()));
}



Ptr <ns3::Node>
Simulation::GetNodeFromContext (std::string context)
{
  // Use "NodeList/*/DeviceList/*/ as reference
  // where element [1] is the Node Id
  // element [2] is the NetDevice Id

  std::vector <std::string> elements = GetElementsFromContext (context);
  Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
  return n;
}



void Simulation::ipv4l3protocolsendoutgoing_tracecb(std::string context, const ns3::Ipv4Header & hdr, ns3::Ptr<const ns3::Packet> p, uint32_t iface)
{

    std::vector<std::string> elements = GetElementsFromContext(context);
    Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
    uint32_t NodeId = n->GetId();
    NodeStatistics & stat = vNodeStatistics[NodeId];
    stat.ipv4l3protocol_sendoutgoing++;


}

void Simulation::ipv4l3protocolunicastforward_tracecb(std::string context, const ns3::Ipv4Header & hdr, Ptr<const ns3::Packet> p, uint32_t iface)
{

    std::vector<std::string> elements = GetElementsFromContext(context);
    ns3::Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
    uint32_t NodeId = n->GetId();
    NodeStatistics & stat = vNodeStatistics[NodeId];
    stat.ipv4l3protocol_unicastforward++;

}

void Simulation::ipv4l3protocollocaldeliver_tracecb(std::string context, const Ipv4Header & hdr, Ptr<const ns3::Packet> p, uint32_t iface)
{

    std::vector<std::string> elements = GetElementsFromContext(context);
    Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
    uint32_t NodeId = n->GetId();
    NodeStatistics & stat = vNodeStatistics[NodeId];
    stat.ipv4l3protocol_localdeliver++;

}

void Simulation::ipv4l3protocoldrop_tracecb(std::string context, const Ipv4Header & hdr,
                                            Ptr<const ns3::Packet> p, ns3::Ipv4L3Protocol::DropReason dr, Ptr<Ipv4> ipv4, uint32_t iface)
{
   std::vector<std::string> elements = GetElementsFromContext(context);
   Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
   uint32_t NodeId = n->GetId();
   NodeStatistics & stat = vNodeStatistics[NodeId];
   stat.ipv4l3protocol_drop++;
}

void Simulation::ipv4l3protocoltx_tracecb(std::string context, Ptr<const ns3::Packet> p , Ptr<Ipv4> ipv4, uint32_t iface)
{
    PacketFilter * pktfilter =  mainInfo->getPktFilter();
    if (pktfilter)
    {
        if (!pktfilter->pass (p))
            return;
    }

    std::vector<std::string> elements = GetElementsFromContext(context);
    Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
    uint32_t NodeId = n->GetId();
    NodeStatistics & stat = vNodeStatistics[NodeId];
    stat.ipv4l3protocol_tx++;

}

void Simulation::ipv4l3protocolrx_tracecb(std::string context, Ptr<const ns3::Packet> p , Ptr<Ipv4> ipv4, uint32_t iface)
{
    PacketFilter * pktfilter =  mainInfo->getPktFilter();
    if (pktfilter)
    {
        if (!pktfilter->pass (p))
            return;
    }
    std::vector<std::string> elements = GetElementsFromContext(context);
    Ptr <ns3::Node> n = NodeList::GetNode (atoi (elements[1].c_str ()));
    uint32_t NodeId = n->GetId();
    NodeStatistics & stat = vNodeStatistics[NodeId];
    stat.ipv4l3protocol_rx++;

}

void Simulation::ConnectTraces()
{

    Config::Connect ("/NodeList/*/$ns3::Ipv4L3Protocol/Drop",
                     MakeCallback (&Simulation::ipv4l3protocoldrop_tracecb, this));
    Config::Connect ("/NodeList/*/$ns3::Ipv4L3Protocol/Rx",
                     MakeCallback (&Simulation::ipv4l3protocolrx_tracecb, this));
    Config::Connect ("/NodeList/*/$ns3::Ipv4L3Protocol/Tx",
                     MakeCallback (&Simulation::ipv4l3protocoltx_tracecb, this));

    Config::Connect ("/NodeList/*/$ns3::Ipv4L3Protocol/SendOutgoing",
                     MakeCallback (&Simulation::ipv4l3protocolsendoutgoing_tracecb, this));
    Config::Connect ("/NodeList/*/$ns3::Ipv4L3Protocol/UnicastForward",
                     MakeCallback (&Simulation::ipv4l3protocolunicastforward_tracecb, this));
    Config::Connect ("/NodeList/*/$ns3::Ipv4L3Protocol/LocalDeliver",
                     MakeCallback (&Simulation::ipv4l3protocollocaldeliver_tracecb, this));



}

void Simulation::Play()
{
    std::cout <<"Playing"<<std::endl;
    if(!RunOnce)
     {
         RunOnce = true;
         ConnectTraces();
         mainInfo->UnPauseSimulation();
         Simulator::Run();
         PostSimulation();
         mainInfo->TerminateSimulation();
         //Simulator::Stop();
         //Simulator::Destroy();

     }
    else
    {
       mainInfo->UnPauseSimulation();
       Simulator::Run();
       mainInfo->TerminateSimulation();
       //Simulator::Stop();
       //Simulator::Destroy();
    }

}



void Simulation::RestartAnimationInterface()
{

    if (pAnim)
    {
        pAnim->StartAnimation();
        pAnim->SetAnimWriteCallback(AnimationInterfaceCallBack);
    }
}

void Simulation::StopAnimationInterface()
{

    if (pAnim)
        pAnim->StopAnimation();
}



void Simulation::Pause()
{
    std::cout <<"Pause"<<endl;
    if(!RunOnce) //Simulation was never started
        return;
    mainInfo->PauseSimulation();
    std::cout<<"Simulation Paused"<<std::endl;
}
} //namespace netanim


#endif // WITH_NS3
