/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3

#ifndef SIMULATION_H
#define SIMULATION_H
#include "MainInfo/MainInfo.h"
#include "Statistics/nodestatistics.h"
#include "ns3/core-module.h"
#include "ns3/internet-module.h"
#include "ns3/network-module.h"
#include "ns3/netanim-module.h"

namespace netanim {

class MainInfo;

class ArgsParser
{
public:
    ArgsParser(QString args);
    int getArgc ();
    char ** getArgv ();
private:
    int argc;
    char ** largv;
};

class Simulation
{

public:
    Simulation (MainInfo *mi);
    ~Simulation();
public:
    void Play();
    void Pause();
    void Stop();
    static bool Paused;
    void AnimationOnline (bool online);
    static void AnimationInterfaceCallBack (const char * s);

    NodeStatistics & GetNodesStatistics (uint32_t);

    void StopAnimationInterface();
    void RestartAnimationInterface();

    void PostSimulation ();

    static bool IsConfigureSimulationComplete ();
private:
    int ConfigureSimulation (int argc,char **argv);
    void ConnectTraces();
    void InitializeNodeStatistics();
    std::vector <NodeStatistics> vNodeStatistics;

    bool RunOnce;
    MainInfo *mainInfo;

    // Traces
    // Ipv4L3Protocol
    void ipv4l3protocoltx_tracecb (std::string context,ns3::Ptr<const ns3::Packet>, ns3::Ptr<ns3::Ipv4>,  uint32_t);
    void ipv4l3protocolrx_tracecb (std::string context,ns3::Ptr<const ns3::Packet>, ns3::Ptr<ns3::Ipv4>, uint32_t);
    // <ip-header, payload, reason, ifindex> (ifindex not valid if reason is DROP_NO_ROUTE)
    void ipv4l3protocoldrop_tracecb (std::string context,const ns3::Ipv4Header &, ns3::Ptr<const ns3::Packet>, ns3::Ipv4L3Protocol::DropReason, ns3::Ptr<ns3::Ipv4>, uint32_t);
    void ipv4l3protocolsendoutgoing_tracecb
    (std::string context,const ns3::Ipv4Header &, ns3::Ptr<const ns3::Packet>, uint32_t);
    void ipv4l3protocolunicastforward_tracecb
    (std::string context,const ns3::Ipv4Header &, ns3::Ptr<const ns3::Packet>, uint32_t);
    void ipv4l3protocollocaldeliver_tracecb
    (std::string context,const ns3::Ipv4Header &, ns3::Ptr<const ns3::Packet>, uint32_t);


    std::vector<std::string> GetElementsFromContext (std::string context);
    ns3::Ptr <ns3::NetDevice> GetNetDeviceFromContext (std::string context);
    ns3::Ptr <ns3::Node> GetNodeFromContext (std::string context);


    ns3::AnimationInterface * pAnim;

    static bool configureSimulationComplete;





};


} // namespace netanim
#endif // SIMULATION_H


#endif
