/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include "client.h"
#include "sys/socket.h"
#include "arpa/inet.h"
#include "fcntl.h"
#include <pthread.h>
#include <iostream>
#include "CmdHandlers/CmdDemux.h"

namespace netanim {

#define NS_ASSERT(x)
#define NS_LOG_ERROR(x) std::cout<<x<<std::endl
#define NS_LOG_INFO(x)
#define NS_FATAL_ERROR(x) std::cout<<x<<std::endl
#define NS_LOG_DEBUG(x) std::cout<<x<<std::endl

Client::Client():thr_created (false)
{
    m_port = 5000;
}

Client::Client(uint16_t port):m_port (port), thr_created (false)
{

}

void * thread_func (void * arg)
{
  Client * client  = static_cast <Client *> (arg);
  client->Run ();
  return NULL;
}

void Client::ProcessConnectFailed()
{

}

void Client::ProcessConnectionClosed()
{

}

void Client::ProcessConnectSuccess()
{

}

void Client::ProcessRead()
{
    if (m_connectedSocket < 0)
      {
        return;
      }
    if (FD_ISSET (m_connectedSocket, &read_flags))
      {
        AnimPacket p;
        DeSerialize (&p);
        //p.Print ();
        FD_CLR (m_connectedSocket, &read_flags);
        CmdDemux::Demux(static_cast <ANIM_CMD> (*p.GetCommandId()), *p.GetPayload());
      }

}

void Client::ProcessWrite()
{

}

void Client::Run ()
{
    int flags;
    flags=fcntl(m_connectedSocket,F_GETFL,0);
    fcntl(m_connectedSocket,F_SETFL,flags | O_NONBLOCK);
    fd_set readfdSet;
    fd_set allfdSet;
    FD_ZERO (&allfdSet);
    FD_SET (m_connectedSocket, &allfdSet);

    struct timeval timeout;
    timeout.tv_sec = 1;
    timeout.tv_usec = 0;



    while (1)
      {
        readfdSet = allfdSet;
        int ret = select (m_connectedSocket + 1, &readfdSet, NULL, NULL, &timeout);
        if (ret == -1)
          {
            std::string ErrorString = "Connected Socket Select Failed";
            perror (ErrorString.c_str ());
          }
        if (m_connectedSocket < 0)
          {
              ProcessConnectionClosed();
          }
        else
          {
            SetReadWriteFlags ();
          }
        ProcessRead ();
        ProcessWrite ();
      }

}

void Client::SetReadWriteFlags ()
{

  int ret;
  if (m_connectedSocket < 0)
    {
      return;
    }
  FD_ZERO (&read_flags);
  FD_ZERO (&write_flags);
  FD_SET (m_connectedSocket, &read_flags);
  FD_SET (m_connectedSocket, &write_flags);
  struct timeval timeout;
  timeout.tv_sec = 1;
  timeout.tv_usec = 0;
  ret = select (m_connectedSocket + 1, &read_flags,&write_flags, NULL, &timeout);
  if (ret == -1)
    {
      std::string ErrorString = "Connected Socket Select Failed";
      perror (ErrorString.c_str ());
    }
}
bool Client::Connect(uint16_t port)
{

    bool success = false;
    m_port = port;
    int ret;
    struct sockaddr_in sin;

    m_connectedSocket=socket(AF_INET, SOCK_STREAM, 0);

    memset(&sin, 0, sizeof(struct sockaddr_in));
    sin.sin_family=AF_INET;
    sin.sin_port=htons(m_port);
    sin.sin_addr.s_addr = inet_addr("127.0.0.1");

    ret=connect(m_connectedSocket, (struct sockaddr *)&sin, sizeof(sin));
    if (ret == 0)
    {
       ProcessConnectSuccess();
       success = true;
    }
    else if (ret == -1)
        ProcessConnectFailed();

    if (success)

    {
        int32_t err = pthread_create (&m_tid, NULL, thread_func, this);
	if (!err)
          thr_created = true;
    }
    return success;
}

void Client::Disconnect()
{
    close (m_connectedSocket);
    if (thr_created)
    pthread_cancel(m_tid);
    ProcessConnectionClosed();
}







//////////////////
const uint32_t Client::magicStart = 0xDEADFEE1;
const uint32_t Client::magicEnd = 0xDEADF00D;


void Client::ReadMagicStart ()
{
  int ret = 0;
  NS_ASSERT (readp);
  uint8_t fieldLength = 4;
  uint8_t rdbuf [4];
  if ((ret = read (m_connectedSocket, rdbuf, fieldLength)) <= 0)
    {
      NS_LOG_ERROR ("Read Magic Start Failed. Closing socket");
      Close (m_connectedSocket);
      return;
    }
  VALIDATELENGTH;
  if (memcmp (rdbuf, &Client::magicStart, fieldLength))
    {
      NS_LOG_ERROR ("AnimPacketMagicStart Not available.Closing socket");
      Close (m_connectedSocket);
      return;
    }
  NS_LOG_INFO ("Got Magic Start");
}

void Client::ReadMagicEnd ()
{
  int ret = 0;
  NS_ASSERT (readp);
  uint8_t fieldLength = 4;
  uint8_t rdbuf[4];
  if ((ret = read (m_connectedSocket, rdbuf, fieldLength)) <= 0)
    {
      NS_LOG_ERROR ("Read Magic End Failed. Closing socket");
      Close (m_connectedSocket);
      return;
    }
  VALIDATELENGTH;
  if (memcmp (rdbuf, &Client::magicEnd, fieldLength))
    {
      NS_LOG_ERROR ("AnimPacketMagicEnd Not available.Closing socket");
      Close (m_connectedSocket);
      return;
    }
  NS_LOG_INFO ("Got Magic End");
}

void Client::ReadRequestResponseType ()
{
  int ret = 0;
  NS_ASSERT (readp);
  uint8_t fieldLength = 1;
  uint8_t rdbuf[1];
  if ((ret = read (m_connectedSocket, rdbuf, fieldLength)) <= 0)
    {
      NS_LOG_ERROR ("Read Request Reponsei Type Failed. Closing socket");
      Close (m_connectedSocket);
      return;
    }
  VALIDATELENGTH;
  readp->SetRequestType (rdbuf[0]);
}


void Client::ReadRequestId ()
{
  int ret = 0;
  NS_ASSERT (readp);
  uint8_t fieldLength = 4;
  uint8_t rdbuf[4];
  if ((ret = read (m_connectedSocket, rdbuf, fieldLength)) <= 0)
    {
      NS_LOG_ERROR ("Read Request Id Failed. Closing socket");
      Close (m_connectedSocket);
      return;
    }
  VALIDATELENGTH;
  readp->SetRequestId (rdbuf);
}

void Client::ReadCommandId ()
{
  int ret = 0;
  NS_ASSERT (readp);
  uint8_t fieldLength = 4;
  uint8_t rdbuf[4];
  if ((ret = read (m_connectedSocket, rdbuf, fieldLength)) <= 0)
    {
      NS_LOG_ERROR ("Read Command Id Failed. Closing socket");
      Close (m_connectedSocket);
      return;
    }
  VALIDATELENGTH;
  readp->SetCommandId (rdbuf);
}

void Client::ReadPayloadLength ()
{

  int ret = 0;
  NS_ASSERT (readp);
  uint8_t fieldLength = 4;
  uint8_t rdbuf[4];
  if ((ret = read (m_connectedSocket, rdbuf, fieldLength)) <= 0)
    {
      NS_LOG_ERROR ("Read Payload Length Failed. Closing socket");
      Close (m_connectedSocket);
      return;
    }
  VALIDATELENGTH;
  readp->SetPayloadLength (rdbuf);
}


void Client::ReadPayload ()
{

  int ret = 0;
  NS_ASSERT (readp);
  uint32_t payloadLength = (*readp->GetPayloadLength ());
  if (!payloadLength)
    {
      NS_LOG_DEBUG ("No payload to send");
      return;
    }
  uint32_t fieldLength = payloadLength * sizeof (uint32_t);
  uint8_t rdbuf[1500];
  uint32_t nbytesRead = 0;

  while (fieldLength)
  {
    if ((ret = read (m_connectedSocket, rdbuf + nbytesRead, fieldLength)) <= 0)
      {
        if (errno == EWOULDBLOCK)
          {
            continue;
          }
        NS_LOG_ERROR ("Read Payload Failed. Closing socket");
        Close (m_connectedSocket);
        return;
      }
    else
      {
        fieldLength -= ret;
        nbytesRead  += ret;
      }
  }
  readp->SetPayload (rdbuf);
}



void Client::DeSerialize (AnimPacket *p)
{
  NS_ASSERT (m_connectedSocket >= 0);
  NS_ASSERT (p);
  readp = p;
  ReadMagicStart ();
  ReadRequestResponseType ();
  ReadRequestId ();
  ReadCommandId ();
  ReadPayloadLength ();
  ReadPayload ();
  ReadMagicEnd ();
}

void Client::ValidateLength (std::string fname, int ret,
                                       uint32_t validLength)
{
  if (static_cast <uint32_t> (ret) != validLength)
    {
        Close (m_connectedSocket);
      NS_FATAL_ERROR (fname << ": Bytes expected:" << validLength
                      << " Bytes got:" << ret << " Socket close");
      return;
    }
  else
    {
      return;
    }
}

void Client::Close(int & s)
{
    close (s);
    s = -1;
}

} //namespace netanim
