/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */
#ifndef CLIENT_H
#define CLIENT_H

#include <QApplication>
#include "sys/socket.h"
#include "errno.h"
#include <pthread.h>
#include <stdio.h>

#include "Animator/AnimPacket.h"

namespace netanim {

#define CONNECTEDICON "images/connected.png"
#define DISCONNECTEDICON "images/disconnected.png"
class Client
{
public:
    Client();
    Client(uint16_t port);
    bool Connect(uint16_t port);
    void Disconnect();
    void setPort (uint16_t port) { m_port = port;}
    uint16_t getPort () { return m_port;}
    int getConnectedSocket () { return m_connectedSocket;}
    void Run ();

private:
    uint16_t m_port;
    int m_connectedSocket;
    pthread_t m_tid;
    bool thr_created;
    void ProcessConnectionClosed ();
    void ProcessConnectFailed ();
    void ProcessConnectSuccess ();
    void ProcessRead ();
    void ProcessWrite ();
    void SetReadWriteFlags ();
    fd_set read_flags;
    fd_set write_flags;
    AnimPacket * readp;
    AnimPacket * writep;

    void ValidateLength (std::string fname,
                         int ret, uint32_t validLength);

    static const uint32_t magicStart;
    static const uint32_t magicEnd;
    void ReadMagicStart ();
    void ReadMagicEnd ();
    void ReadRequestResponseType ();
    void ReadRequestId ();
    void ReadCommandId ();
    void ReadPayloadLength ();
    void ReadPayload ();
    void DeSerialize (AnimPacket *p);

    void Close (int & s);





};

} // namespace netanim
#endif // CLIENT_H
