#if 0
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3
#include "MainInfo/MainInfo.h"
#include "MainWindow/mainwindow.h"
#include "Tcp/TcpTab.h"
#include "ns3/internet-module.h"
#include "ns3/tcp-socket-base.h"
#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/point-to-point-net-device.h"
#include "ns3/wifi-module.h"
#include "ns3/wimax-module.h"
#include "ns3/csma-module.h"

using namespace ns3;

namespace netanim {

static std::vector<TcpCombo3 *> gNetDeviceStats;
static std::vector<TcpCombo3 *> gQueueStats;
static bool gLogToFile = false;


static int defaultGridMaxColumn = 4;


TcpTab::TcpTab(QObject * parent):AnimTab (parent),
    polling(false),
    simtimePolling(true),
    simtimePollingTimer(ns3::Timer::CANCEL_ON_DESTROY)

{
    wallClocktimer = new QTimer;
}

QWidget* TcpTab::Create(QApplication *app, MainInfo *mi)
{

    (void)app;
    mainInfo = mi;
    numNodes = ns3::NodeList::GetNNodes();

    vbox0 = new QVBoxLayout;
    centralWidget = new QWidget;
    hbox0 = new QHBoxLayout;
    hbox0->addWidget(mi->GetCommonVCR());


    logtoFileButton = new QPushButton("Log to File");
    logtoFileButton->setToolTip("Toggle this button to Log the statistics to the logs folder");
    logtoFileButton->setCheckable(true);
    connect(logtoFileButton, SIGNAL (clicked()), this, SLOT (logtofileslot()));


    QPointer <QLabel> pollLabel = new QLabel ("Poll Interval in ms");
    pollLineEdit = new QLineEdit;
    pollLineEdit->setText("1000");
    pollLabel->setBuddy(pollLineEdit);


    applyButton = new QPushButton(("&Stopped"));
    applyButton->setCheckable(true);
    connect(applyButton, SIGNAL (clicked ()),this,SLOT(apply()));

    polling_gb = new QGroupBox;
    wallClock_rb = new QRadioButton("WallClock");
    wallClock_rb->setToolTip("Use Wall Clock Time for polling");
    connect(wallClock_rb, SIGNAL(clicked(bool)),this, SLOT(wallclockrb_slot(bool)));
    QPointer <QHBoxLayout> pollinggb_hbox = new QHBoxLayout;
    simulationTime_rb = new QRadioButton("SimTime");
    simulationTime_rb->setToolTip("Use Simulation Time for polling");
    simulationTime_rb->setChecked(true);
    connect(simulationTime_rb, SIGNAL (clicked(bool)),this,SLOT(simtimerb_slot(bool)));
    pollinggb_hbox->addWidget(simulationTime_rb);
    pollinggb_hbox->addWidget(wallClock_rb);
    polling_gb->setLayout(pollinggb_hbox);



    hbox0->addWidget(logtoFileButton);
    hbox0->addWidget(pollLabel);
    hbox0->addWidget(pollLineEdit);
    hbox0->addWidget(polling_gb);
    hbox0->addWidget(applyButton);
    vbox0->addLayout(hbox0);

    optionsHBox = new QHBoxLayout;

    showNetDeviceStats_cb = new QCheckBox("Net Device stats");
    showNetDeviceStats_cb->setChecked(true);
    connect(showNetDeviceStats_cb,SIGNAL(stateChanged(int)),this,SLOT(showNetDeviceStats_slot(int)));

    showQueueStats_cb = new QCheckBox("Queue stats");
    showQueueStats_cb->setChecked(true);
    connect(showQueueStats_cb,SIGNAL(stateChanged(int)),this,SLOT(showQueueStats_slot(int)));

    optionsHBox->addWidget(showNetDeviceStats_cb);
    optionsHBox->addWidget(showQueueStats_cb);

    vbox0->addLayout(optionsHBox);


    // Node HBox
    AddTcpCombos();
    vbox0->addWidget(nodesSA);

    // Gridbox
    gridbox2 = new QGridLayout;
    vbox0->addLayout(gridbox2);

    centralWidget->setLayout(vbox0);
    return centralWidget;


}


void TcpTab::showNetDeviceStats_slot(int state)
{
    if(showNetDeviceStats_cb->isChecked())
        ShowNetDeviceStats(true);
    else
        ShowNetDeviceStats(false);


}

void TcpTab::ShowNetDeviceStats(bool s)
{
    for (uint32_t i = 0 ; i < gNetDeviceStats.size() ; i++)
    {
        TcpCombo3 * n3 = gNetDeviceStats[i];
        if (s)
        n3->getCentralWidget()->show();
        else
        n3->getCentralWidget()->hide();

    }

}

void TcpTab::ShowQueueStats(bool s)
{
    for (uint32_t i = 0 ; i < gQueueStats.size() ; i++)
    {
        TcpCombo3 * n3 = gQueueStats[i];
        if (s)
        n3->getCentralWidget()->show();
        else
        n3->getCentralWidget()->hide();

    }

}


void TcpTab::showQueueStats_slot(int state)
{
    if(showQueueStats_cb->isChecked())
        ShowQueueStats(true);
    else
        ShowQueueStats(false);



}

void TcpTab::simtimerb_slot(bool state)
{
    if(simulationTime_rb->isChecked())
        simtimePolling = true;
}

void TcpTab::wallclockrb_slot(bool state)
{

    if(wallClock_rb->isChecked())
        simtimePolling = false;
}


void TcpTab::onFocus()
{
    hbox0->insertWidget(0,mainInfo->GetCommonVCR());
    mainInfo->GetCommonVCR()->show();

}

void TcpTab::onLostFocus()
{


}

QWidget* TcpTab::getCentralWidget()
{
    return centralWidget;
}

QString TcpTab::GetTabName()
{

    return "Tcp";
}

void TcpTab::logtofileslot()
{
    gLogToFile = !gLogToFile;
    if(gLogToFile)
    logtoFileButton->setText("Logging To file!!");
    else
    logtoFileButton->setText("Log to File");


}

void TcpTab::DoSimTimePolling()
{
    mainInfo->getMainWindow()->SetLCD();
    static bool first = true;
    if (first)
    {
       QMessageBox msg;
       msg.setText("Please ensure you used \"Simulator::Stop (Seconds (Time_t))\" function\nOr your simulation can run endlessly");
       msg.exec();
    }

    if(simtimePollingTimer.IsRunning())
    simtimePollingTimer.Cancel();
    if(!polling || (Simulator::IsFinished() && !first))
        return;
    if(Simulator::IsFinished() && !first)
    {
       polling = false;
       QMessageBox msg;
       msg.setText("Simulation Finished");
       msg.exec();
       applyButton->setText("Stopped");

       return;
    }
    simtimePollingTimer.SetFunction (&netanim::TcpTab::DoSimTimePolling,this);
    UpdateTcps();
    simtimePollingTimer.Schedule(ns3::MilliSeconds(pollString.toInt()));
    first = false;


}


void TcpTab::DoWallClockPolling()
{
    mainInfo->getMainWindow()->SetLCD();
    if(wallClocktimer->isActive())
        wallClocktimer->stop();
    if(!polling)
        return;

    wallClocktimer->setInterval(pollString.toInt());
    connect(wallClocktimer, SIGNAL(timeout()), this, SLOT(UpdateTcps()));
    wallClocktimer->start();


}

void TcpTab::apply()
{
    polling = !polling;
    pollString = pollLineEdit->text();

    if (simtimePolling)
        DoSimTimePolling();
    else
        DoWallClockPolling();


    if (!polling)
    {
        applyButton->setText("Stopped");
        polling_gb->setEnabled(true);
        pollLineEdit->setEnabled(true);
        return;
    }
    else
    {
        applyButton->setText("Polling");
        polling_gb->setEnabled(false);
        pollLineEdit->setEnabled(false);

    }


}


void TcpTab::AddTcpCombos()
{
    nodesHBox = new QHBoxLayout;
    nodesSA = new QScrollArea;
    for (uint32_t i = 0; i < numNodes ; i++)
    {
        QPointer <TcpCombo> tcmb = AddTcpCombo(i);
        nodesHBox->addWidget(tcmb->getButton());
    }
    QWidget * qw = new QWidget;
    qw->setLayout(nodesHBox);
    nodesSA->setFixedHeight(50);
    nodesSA->setWidget(qw);


}


void TcpTab::UpdateTcps()
{

    if(mainInfo->IsSimulationPaused())
        return;
   for (mTcpType::iterator i = mTcpCombo.begin();
    i != mTcpCombo.end();
    i++)
   {
       QPointer <TcpCombo> tcmb = i->second;
       tcmb->UpdateTcps();
   }
}


TcpCombo * TcpTab::AddTcpCombo(uint32_t nodeId)
{

    TcpCombo * ncmb = new TcpCombo(this,nodeId,mainInfo);
    mTcpCombo[nodeId] = ncmb;
    return ncmb;

}


void TcpTab::AdjustGrid()
{

    for (mTcpType::iterator i = mTcpCombo.begin();
         i != mTcpCombo.end();
         i++)
    {

        QPointer <TcpCombo> p = i->second;
        gridbox2->removeWidget(p->getCentralWidget());
        p->getCentralWidget()->hide();
    }

    for (mTcpType::iterator i = mTcpCombo.begin();
         i != mTcpCombo.end();
         i++)
    {
        QPointer <TcpCombo> p = i->second;
        if(p->isactive())
        {
            int total = gridbox2->count();
            int r = gridbox2->rowCount();
            int maxcolumn = defaultGridMaxColumn;
            if ((total % maxcolumn) == 0)
             gridbox2->addWidget(p->getCentralWidget(),r+1,0,1,1);
            else
                gridbox2->addWidget(p->getCentralWidget(),r-1,(total % maxcolumn ),1,1);

            p->getCentralWidget()->show();
        }

    }


}


void TcpTab::NotifyToggled(QPointer <TcpCombo> ncmb, uint32_t NodeId,bool active,QWidget * w)
{
    int index=0;
    if ((index=gridbox2->indexOf(w)) == -1)
    {
    // lets add this widget
        if(active)
          {
             int total = gridbox2->count();
             int r = gridbox2->rowCount();
             int maxcolumn = defaultGridMaxColumn;
             if ((total % maxcolumn) == 0)
              gridbox2->addWidget(w,r+1,0,1,1);
             else
              gridbox2->addWidget(w,r-1,(total % maxcolumn ),1,1);

          }
        mTcpCombo[NodeId] = ncmb;
    }
    if(!active)
    {

         gridbox2->removeWidget(w);
         w->setVisible(false);
         AdjustGrid();

    }
    else
    {
         w->setVisible(true);
    }

}

TcpCombo::TcpCombo(TcpTab *ndt, uint32_t nodeId,MainInfo * mi):
    mainInfo(mi),
    logtoFile(false),
    currentSocketCount(0)
{
    m_ndt = ndt;
    m_nodeId = nodeId;
    nodeButton = new QPushButton(QString::number(nodeId));
    nodeButton->setCheckable(true);
    connect( nodeButton, SIGNAL(clicked()), this, SLOT(onbuttonToggled()) );

    vbox0 = new QVBoxLayout;
    centralWidget = new QWidget;

    gb = new QGroupBox("Node:" + QString::number(nodeId));
    socketType = new QLabel;
    vbox0->addWidget(socketType);




    n = ns3::NodeList::GetNode(m_nodeId);
    NS_ASSERT (n);

    tcpl4protocol = n->GetObject <ns3::TcpL4Protocol> ();
    NS_ASSERT (tcpl4protocol);
    QString str = tcpl4protocol->GetInstanceTypeId().GetAttribute(1).name.c_str();
    str += ":";

    tcpl4protocol->GetAttributeFailSafe ("SocketType", sockettypeIdValue);
    str += QString(sockettypeIdValue.Get().GetName().c_str()).split("::")[1];
//    for (uint32_t i = 0 ; i < n->GetNDevices() ; i++)
//    {
//        QPointer <NetDeviceCombo2> ncmb2 = new NetDeviceCombo2 (n->GetDevice(i),mi);
//        vNetDeviceCombo2s.push_back(ncmb2);
//        vbox0->addWidget(ncmb2->getCentralWidget());
//    }
    socketType->setText(str);

    tcpl4protocol->GetAttributeFailSafe("RttEstimatorType",rttEstimatorTypeIdValue);
    str = "RttEstimatorType:" + QString(rttEstimatorTypeIdValue.Get().GetName().c_str()).split("::")[1];
    rttEstimatorTypeLabel = new QLabel(str);
    vbox0->addWidget(rttEstimatorTypeLabel);

    tcpl4protocol->GetAttributeFailSafe("SocketList", socketListVectorValue);
    str = "Socket Count:" + QString::number(socketListVectorValue.GetN());
    socketCountLabel = new QLabel(str);
    vbox0->addWidget(socketCountLabel);

    gb->setLayout(vbox0);

    centralWidget = gb;
    active = false;

}

TcpCombo::~TcpCombo()
{

}



void TcpCombo::SetLogtoFile(bool logit)
{
    logtoFile = logit;
}


bool TcpCombo::isactive()
{
    return active;
}

QPointer <QPushButton> TcpCombo::getButton()
{
    return nodeButton;

}


void TcpCombo::onbuttonToggled()
{
    active = !active;
    m_ndt->NotifyToggled(QPointer <TcpCombo> (this),m_nodeId, active,centralWidget);
    //std::cout<<tcpl4protocol->GetInstanceTypeId().GetAttribute(0).name.c_str() <<std::endl;

}


QWidget * TcpCombo::getCentralWidget()
{
    return centralWidget;
}


void TcpCombo::UpdateTcps()
{
    QString str;
    tcpl4protocol->GetAttributeFailSafe("SocketList",socketListVectorValue);
    uint32_t newSocketCount = socketListVectorValue.GetN();
    str = "Socket Count:" + QString::number(newSocketCount);
    socketCountLabel->setText(str);
    for (uint32_t i = 0 ; i <socketListVectorValue.GetN(); i++ )
    {

    }


    for (uint32_t i = 0 ; i < vTcpCombo2s.size() ; i++)
    {
        //vTcpCombo2s[i]->UpdateTcp();
    }
}



TcpCombo3::TcpCombo3(QString appName, QString traceName, QString auxString):
    pgrapher(0),
    theCounter(0),
    plotOn(false),
    m_appName(appName),
    m_traceName(traceName),
    plotCounter(0),
    m_auxString(auxString)

{
    file = new QFile("logs/" + appName.replace(" ","_") + "_" + auxString.replace(":","_") + "_" + traceName + ".txt");
    pCounterLabel = new QLabel;
    centralWidget = new QWidget;
    hbox0 = new QHBoxLayout;
    hbox0->addWidget(pCounterLabel);
    pCounterLabel->setText(m_auxString +":" + m_traceName + ":" + QString::number(theCounter) );
    plotButton = new QPushButton ("Plot");
    connect(plotButton, SIGNAL (clicked()),this,SLOT(plotSlot()));
    hbox0->addWidget(plotButton);
    centralWidget->setLayout(hbox0);

}


void TcpCombo3::OpenFile(QPointer <QFile> f)
{
    if (f->isOpen())
        return;
    if (!f->open(QIODevice::WriteOnly))
    {
        std::cout<<"Unable to open:"<<f->fileName().toAscii().data()<<endl;
        exit(1);
    }


}
void TcpCombo3::AppendToFile(QPointer <QFile> f,QString str)
{
    OpenFile (f);

    if(f->write(str.toAscii().data()) == -1)
    {

        std::cout<<"Write Failed:"<<endl;
        exit(1);
    }
    f->flush();
}


QWidget * TcpCombo3::getCentralWidget()
{
    return centralWidget;
}


TwoDGrapherView* TcpCombo3::getTwodGrapher()
{

    return pgrapher;
}

void TcpCombo3::incStat()
{
    theCounter++;
    plotCounter++;
}


void TcpCombo3::setStat(double t)
{
    theCounter = t;
    plotCounter = t;
}

void TcpCombo3::timeToPlot()
{
    if(pgrapher)
    pgrapher->AppendDataPoint(Simulator::Now().GetSeconds(),plotCounter);
    plotCounter = 0;
    pCounterLabel->setText(m_auxString +":" + m_traceName + ":" + QString::number(theCounter) );
    if(gLogToFile)
        AppendToFile(file,QString::number(Simulator::Now().GetSeconds()) +"\t" + QString::number(plotCounter) + "\n");
}

 void TcpCombo3::plotSlot()
{
    plotOn = !plotOn;
    if(plotOn)
    {
        pgrapher = new TwoDGrapherView(m_appName+":" + m_auxString,"Simulation Time",m_traceName,m_appName+":"+m_auxString);//,this);
        hbox0->addWidget(pgrapher);
        plotButton->setText("Plot Off");
        pgrapher->setMinimumSize(500,500);
    }
    else
    {
        plotButton->setText("Plot");
        delete pgrapher;
        pgrapher = 0;
    }



}



} // namespace netanim

#endif //WITH_NS3

#endif
