#if 0
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3
#ifndef TCPTAB_H
#define TCPTAB_H

#include "MainInfo/MainInfo.h"
#include <AnimTab/animtab.h>
#include <QtGui>
#include "ns3/core-module.h"
#include "ns3/applications-module.h"
#include "ns3/sgi-hashmap.h"
#include <map>
#include "Graphing/TwodGrapher.h"

using namespace std;

namespace netanim {
class TcpTab;


//class Ipv4AddressHashNetAnim : public std::unary_function<ns3::Ipv4Address, size_t> {
//public:
//  size_t operator() (ns3::Ipv4Address const &x) const
//  {
//    return x.Get();
//  }
//  size_t operator() (ns3::Ipv4Address const &x, ns3::Ipv4Address const &y) const
//  {
//    return x < y;
//  }
//};

class TcpCombo3 : public QWidget
{
    Q_OBJECT
public:
    QWidget * getCentralWidget();
    TcpCombo3 (QString appName, QString traceName, QString auxString);
    void OpenFile(QPointer <QFile> f);
    void AppendToFile(QPointer <QFile> f,QString str);
    TwoDGrapherView * getTwodGrapher();
    void incStat();
    void setStat(double t);
    QPointer <QLabel> pCounterLabel;
    void timeToPlot();
private:
    uint32_t theCounter;
    uint32_t plotCounter;
    QWidget * centralWidget;
    TwoDGrapherView * pgrapher;
    bool plotOn;
    QPointer <QPushButton> plotButton;

    QString m_appName;
    QString m_traceName;
    QString m_auxString;
    QPointer <QHBoxLayout> hbox0 ;
    QPointer <QFile> file;
private slots:
     void plotSlot();

};

class TcpCombo2 : public QWidget
{
    Q_OBJECT
public:
    TcpCombo2(ns3::Ptr <ns3::NetDevice> nd, MainInfo *mi);




    QWidget * getCentralWidget();




private:
    ns3::Ptr <ns3::DropTailQueue> pdq;
    std::string devname;
    QString m_macaddress;
    QPointer <QVBoxLayout> vbox0;
    QPointer <QLabel> MacLabel;
    QPointer <QLabel> NDType;
    QPointer <QLabel> NQueueType;
    MainInfo * mainInfo;
    ns3::Ptr <ns3::NetDevice > m_nd;
    QWidget * centralWidget;
};






class TcpCombo : public QWidget
{
    Q_OBJECT
    public:

        TcpCombo(TcpTab * mndt, uint32_t nodeId,MainInfo *);
        ~TcpCombo();
        QPointer <QPushButton> getButton();
        QWidget * getCentralWidget();
        bool isactive();
        void SetLogtoFile(bool logit);
        void UpdateTcps();
    private:
        ns3::Ptr <ns3::TcpL4Protocol> tcpl4protocol;
        QPointer <QLabel> socketType;
        QPointer <QLabel> rttEstimatorTypeLabel;
        QPointer <QLabel> socketCountLabel;
        uint32_t currentSocketCount ;


        ns3::TypeIdValue sockettypeIdValue;
        ns3::TypeIdValue rttEstimatorTypeIdValue;
        ns3::ObjectVectorValue socketListVectorValue;


        QPointer <QGroupBox> gb;
        std::vector < QPointer <TcpCombo2> > vTcpCombo2s;

        bool logtoFile;

        QPointer <QPushButton> nodeButton;
        QPointer <QVBoxLayout> vbox0;
        QPointer <QLabel> nodeLabel;

        ns3::Ptr <ns3::Node> n;
        uint32_t NNetDevices;


        TcpTab * m_ndt;
        uint32_t m_nodeId;
        bool active;

        QWidget * centralWidget;
        MainInfo * mainInfo;


    public:


    public slots:
        void onbuttonToggled ();

};



class TcpTab : public AnimTab
{
    Q_OBJECT
public:
    TcpTab( QObject *parent = 0);
    QWidget * Create (QApplication * app, MainInfo *mi);

    QString GetTabName();
    QWidget * getCentralWidget();
    void onFocus();
    void onLostFocus();
    void NotifyToggled(QPointer <TcpCombo>, uint32_t,bool active,QWidget *);
    void UpdateTcps();
    typedef std::map <uint32_t,QPointer <TcpCombo> > mTcpType;
    void AdjustGrid();


    QPointer <QHBoxLayout> optionsHBox;
    QPointer <QCheckBox> showNetDeviceStats_cb;
    QPointer <QCheckBox> showQueueStats_cb;


    void ShowNetDeviceStats(bool s);
    void ShowQueueStats(bool s);

public slots:
    void showNetDeviceStats_slot(int state);
    void showQueueStats_slot(int state);

private:
    MainInfo * mainInfo;
    uint32_t numNodes;
    QPointer <QVBoxLayout> vbox0;

    // Hbox 0
    QPointer <QHBoxLayout> hbox0;
    QPointer <QPushButton> logtoFileButton;
    QPointer <QLineEdit> pollLineEdit;
    QPointer <QPushButton> applyButton;
    QPointer <QGroupBox> polling_gb;
    QPointer <QRadioButton> wallClock_rb;
    QPointer <QRadioButton> simulationTime_rb;

    //Polling
    bool polling;
    bool simtimePolling;
    QString pollString;
    void DoSimTimePolling();
    void DoWallClockPolling();
    QPointer <QTimer> wallClocktimer;
    ns3::Timer simtimePollingTimer;

    // Gridbox 2

    QPointer <QGridLayout> gridbox2;
    void AddTcpCombos();
    TcpCombo * AddTcpCombo(uint32_t nodeId);
    std::map <uint32_t,QPointer <TcpCombo> > mTcpCombo;


    // Nodes Hbox
    QPointer <QHBoxLayout> nodesHBox;
    QPointer <QScrollArea> nodesSA;



    QWidget * centralWidget;





public slots:

    void apply();
    void logtofileslot();
    void wallclockrb_slot(bool state);
    void simtimerb_slot(bool state);



};

} // namespace netanim
#endif // TCPTAB_H
#endif  //WITH_NS3


#endif
