/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 * Modified by: John Abraham <john.abraham@gatech.edu>
 */

#include "XML/PartialStream.h"
#include "Animator/ThreadInfo.h"
#include "MainInfo/MainInfo.h"

namespace netanim {

PartialFileInputSource::PartialFileInputSource(XMLCh* const filePath,
                                               unsigned int id0, MainInfo * mi)
    : LocalFileInputSource(filePath), fileName(filePath), threadId(id0), mainInfo (mi)
{
}



BinInputStream* PartialFileInputSource::makeStream() const
{
  PartialBinFileInputStream* r = new (getMemoryManager())
    PartialBinFileInputStream(fileName, threadId,mainInfo);
  if (!r->getIsOpen())
    {
      delete r;
      return 0;
    }
  return r;
}

// PartialBinFileInputStream methods
PartialBinFileInputStream::PartialBinFileInputStream(const XMLCh* const fn,
                                                     unsigned int id0, MainInfo * mi)
    : currentPos(0), threadId(id0), mainInfo (mi)
{
  char* fnch = XMLString::transcode(fn);
  f = fopen(fnch, "r");
  printf("Opened %s\n", fnch);
}

PartialBinFileInputStream::~PartialBinFileInputStream()
{
}

bool PartialBinFileInputStream::getIsOpen() const
{
  return f;
}

XMLFilePos PartialBinFileInputStream::curPos() const
{
  return currentPos;
}

XMLSize_t PartialBinFileInputStream::readBytes(
                                               XMLByte* const toFill,
                                               const XMLSize_t count)
{
  ThreadInfo& myInfo = mainInfo->threadInfo[threadId];

  while(!myInfo.done)
    {
      fseek(f, currentPos, SEEK_SET);
      unsigned int actual = fread(toFill, 1, count, f);
      currentPos += actual;
      //if (actual) cout << "Read actual " << actual << endl;
      //pthread_mutex_lock(&mainInfo->coutMutex);
      //pthread_mutex_unlock(&mainInfo->coutMutex);
      if (actual) return actual; // Got some data
      //cout << "Read eof" << endl;
      // Signal the "mainCond", but don't update "completedThisCycle
      // This allows the display to continue to be responsive to Qt
      // events while we are spinning here
      //pthread_mutex_lock(&mainInfo->coutMutex);
      //cout << "Thr " << threadId
      //     << " signaling mainCond" << endl;
      //pthread_mutex_unlock(&mainInfo->coutMutex);
      myInfo.advancedTo = myInfo.lastXMLTime; // set actual advanced time
      SignalAdvanced(threadId);
      WaitForAdvance(threadId);
      //pthread_mutex_lock(&mainInfo->mainMutex);
      //pthread_cond_signal(&mainInfo->mainCond);
      //pthread_mutex_unlock(&mainInfo->mainMutex);
      //usleep(10000); // wait a little while (10ms)
    }
  return 0;  // End of file
}

const XMLCh* PartialBinFileInputStream::getContentType() const
{
  return 0;
}

} //namespace netanim
