/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 * Modified by: John Abraham <john.abraham@gatech.edu>
 */
#ifndef ANIMDOCHANDLERS_H
#define ANIMDOCHANDLERS_H

#include "animparser.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/BinFileInputStream.hpp>
#include <xercesc/framework/StdInInputSource.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/parsers/SAXParser.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>

#include "MainInfo/MainInfo.h"

namespace netanim {


// Define my Anim Doc handler
class AnimDocHandlers : public DocumentHandler, public ErrorHandler
{
public:
  AnimDocHandlers(MainInfo *);
  virtual ~AnimDocHandlers();
  void characters(const XMLCh* const chars, const XMLSize_t);
  void docComment(const XMLCh* const);
  void docPI(const XMLCh* const, const XMLCh* const);
  void endDocument();
  void endElement(const XMLCh* const name);
  void ignorableWhitespace(const XMLCh* const chars, const XMLSize_t length);
  void processingInstruction(const XMLCh* const, const XMLCh* const){}
  void resetDocument();
  void setDocumentLocator(const Locator* const) {}
  void startDocument();
  void startElement (const   XMLCh* const, AttributeList&  attrs);
  // Error handlers
   void warning(const SAXParseException& exc);
   void error(const SAXParseException& exc);
   void fatalError(const SAXParseException& exc);
   void resetErrors();
public:
  AnimParser* parser;
private:
  MainInfo * mainInfo;
};

} // namespace netanim
#endif // ANIMDOCHANDLERS_H
