/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 * Modified by: John Abraham <john.abraham@gatech.edu>
 */
#ifndef ANIMPARSER_H
#define ANIMPARSER_H

#include "Common/CommonDefs.h"
#include "Animator/NetModel.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/BinFileInputStream.hpp>
#include <xercesc/framework/StdInInputSource.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/parsers/SAXParser.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>

#include "MainInfo/MainInfo.h"

using namespace xercesc;
using namespace std;

namespace netanim {


class AnimParser : public SAXParser
{
public:
  AnimParser(unsigned tid, MainInfo * );
  // Add specific element handlers as member functions
  void ProcessAnim(AttributeList& attrs);
  void ProcessTopology(AttributeList& attrs);
  void ProcessNode(AttributeList& attrs);
  void ProcessLink(AttributeList& attrs);
  void ProcessMove(AttributeList& attrs);
  void ProcessQueue(AttributeList& attrs);
  void ProcessPacket(AttributeList& attrs, bool);
  void ProcessRx(AttributeList& attrs);
  string GetNamedAttribute(AttributeList& attributes, const string& which);

public:
  LpId_t lpId;
  Packet pkt;
  string xmlFileName;  // File name to parse
  NetModel* model;
  unsigned  threadIndex;  // Index into the global thread info structure
  bool first;
private:
  MainInfo * mainInfo;
  };



} // namespace netanim
#endif // ANIMPARSER_H
