#include "Simulation/private/simulation.h"
#ifdef WITH_NS3
using namespace netanim;
/// DO NOT EDIT ABOVE THIS LINE

/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
// Adapted from star-animation.cc (ns-3)

#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/internet-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/applications-module.h"
#include "ns3/wifi-module.h"
#include "ns3/flow-monitor-module.h"
#include "ns3/simulator-impl.h"
#include "ns3/mobility-module.h"
#include "ns3/point-to-point-layout-module.h"

using namespace ns3;
using namespace std;



NS_LOG_COMPONENT_DEFINE ("FirstScriptExample");



ApplicationContainer hubApp;  // Make this global so that
// Simulation::PostSimulation can have access to it.

int Simulation::ConfigureSimulation(int argc,char **argv)
{
    //
    // Set up some default values for the simulation.
    //
    Config::SetDefault ("ns3::OnOffApplication::PacketSize", UintegerValue (137));

    // ??? try and stick 15kb/s into the data rate
    Config::SetDefault ("ns3::OnOffApplication::DataRate", StringValue ("14Mb/s"));

    //
    // Default number of nodes in the star.  Overridable by command line argument.
    //
    uint32_t nSpokes = 8;
    uint32_t animPort = 0;
    std::string animFile;

    CommandLine cmd;
    cmd.AddValue ("nSpokes", "Number of spoke nodes to place in the star", nSpokes);
    cmd.AddValue ("animPort",      "Port Number for Remote Animation", animPort);
    cmd.AddValue ("animFile",  "File Name for Animation Output", animFile);

    cmd.Parse (argc, argv);

    NS_LOG_INFO ("Build star topology.");
    PointToPointHelper pointToPoint;
    pointToPoint.SetDeviceAttribute ("DataRate", StringValue ("5Mbps"));
    pointToPoint.SetChannelAttribute ("Delay", StringValue ("2ms"));
    PointToPointStarHelper star (nSpokes, pointToPoint);

    NS_LOG_INFO ("Install internet stack on all nodes.");
    InternetStackHelper internet;
    star.InstallStack (internet);

    NS_LOG_INFO ("Assign IP Addresses.");
    star.AssignIpv4Addresses (Ipv4AddressHelper ("10.1.1.0", "255.255.255.0"));

    NS_LOG_INFO ("Create applications.");
    //
    // Create a packet sink on the star "hub" to receive packets.
    //
    uint16_t port = 50000;
    Address hubLocalAddress (InetSocketAddress (Ipv4Address::GetAny (), port));
    PacketSinkHelper packetSinkHelper ("ns3::TcpSocketFactory", hubLocalAddress);
    hubApp.Add (packetSinkHelper.Install (star.GetHub ()));
    hubApp.Start (Seconds (1.0));
    hubApp.Stop (Seconds (10.0));

    //
    // Create OnOff applications to send TCP to the hub, one on each spoke node.
    //
    OnOffHelper onOffHelper ("ns3::TcpSocketFactory", Address ());
    onOffHelper.SetAttribute ("OnTime", RandomVariableValue (ConstantVariable (1)));
    onOffHelper.SetAttribute ("OffTime", RandomVariableValue (ConstantVariable (0)));

    ApplicationContainer spokeApps;
    for (uint32_t i = 0; i < star.SpokeCount (); ++i)
      {
        AddressValue remoteAddress (InetSocketAddress (star.GetHubIpv4Address (i), port));
        onOffHelper.SetAttribute ("Remote", remoteAddress);
        spokeApps.Add (onOffHelper.Install (star.GetSpokeNode (i)));
      }
    spokeApps.Start (Seconds (1.0));
    spokeApps.Stop (Seconds (10.0));

    NS_LOG_INFO ("Enable static global routing.");
    //
    // Turn on global static routing so we can actually be routed across the star.
    //
    Ipv4GlobalRoutingHelper::PopulateRoutingTables ();

    // Set the bounding box for animation
    star.BoundingBox (1, 1, 100, 100);
    Simulator::Stop(Seconds(20)); // This is important or the animator will run endlessly.

}

void Simulation::PostSimulation()
{
    Ptr <Application> app = hubApp.Get (0);
    Ptr <PacketSink> pktSink = DynamicCast <PacketSink> (app);
    std::cout << "HubApp Total bytes Rx:" << pktSink->GetTotalRx() << std::endl;
}







/// DO NOT EDIT BELOW THIS LINE
#endif // WITH_NS3

