/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */

// Convert ns3 trace file to xml file
// George F. Riley, Georgia Tech, Spring 2010

// Assume lpId = 0 for all.  Fix later

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#include "Node.h"
#include "WiredLink.h"
#include "Packet.h"

using namespace std;

NodeMap_t nodes;
PktMap_t  packets;
LinkVec_t links;

void ConvertEvent(ifstream& ifs, ofstream& ofs)
{
  Time_t time;
  char   action;
  int    id;
  double x;
  double y;
  int    n1;
  int    n2;
  Time_t t1;
  Time_t t2;
  Time_t t3;
  ifs >> time >> action;
  switch (action) 
    {
    case 'N' : // Node
      ifs >> id >> x >> y;
      //Node::Insert(Node(0, id, x*1000, y*1000, 0));
      nodes.insert(NodeMap_t::value_type(KeyPair_t(0, id),
                                         Node(0, id, x * 1000, y * 1000, 0)));
      break;
    case 'L' : // Link
      ifs >> n1 >> n2;
      links.push_back(WiredLink(0, n1, 0, n2));
      break;
    case 'P' : // Packet
      ifs >> n1 >> n2 >> t1 >> t2 >> t3;
      packets.insert(PktMap_t::value_type(time, 
                                          Packet(0, n1, 0, n2, false,
                                                 time, t1, t2, t3)));
      //Packet::Insert(Packet(0, n1, 0, n2, false,
      //time, t1, t2, t3));
      break;
    default:
      break;
    }
}

int main(int argc, char** argv)
{
  if (argc < 3)
    {
      cout << "Usage: cvns3 ns3file xmlfile" << endl;
      exit(1);
    }
  ifstream ifs(argv[1]);
  if (!ifs)
    {
      cout << "Can't open input file " << argv[1] << endl;
      exit(2);
    }
  ofstream ofs(argv[2]);
  if (!ofs)
    {
      cout << "Can't open output file " << argv[2] << endl;
      exit(2);
    }
  while(ifs)
    {
      ConvertEvent(ifs, ofs);
    }
  cout << "Found " << nodes.size() << " nodes" << endl;
  cout << "Found " << links.size() << " links" << endl;
  cout << "Found " << packets.size() << " packets" << endl;
  // Now find the min/max x/y for the anim xml object
  double minx = nodes.begin()->second.x;
  double miny = nodes.begin()->second.y;
  double maxx = nodes.begin()->second.x;
  double maxy = nodes.begin()->second.y;
  for (NodeMap_t::iterator i = nodes.begin(); i != nodes.end(); ++i)
    {
      if (i->second.x < minx) minx = i->second.x;
      if (i->second.y < miny) miny = i->second.y;
      if (i->second.x > maxx) maxx = i->second.x;
      if (i->second.y > maxy) maxy = i->second.y;
    }
  // Add a 10% margin to min/max
  double w = maxx - minx;
  double h = maxy - miny;
  double mx = w * 0.1;
  minx -= mx;
  maxx += mx;
  double my = h * 0.1;
  miny -= my;
  maxy += my;
  
  ofs << "<anim lp = \"0\">" << endl;
  ofs << "<topology "
      << " minX = \"" << minx << "\""
      << " minY = \"" << miny << "\"" 
      << " maxX = \"" << maxx << "\""
      << " maxY = \"" << maxy << "\"" 
      << ">" << endl;
  // Ouptut the nodes
  for (NodeMap_t::iterator i = nodes.begin(); 
       i != nodes.end();
         ++i)
    {
      ofs << "<node lp = \"0\" id = \"" << i->second.id << "\""
           << " locX = \"" << i->second.x << "\""
           << " locY = \"" << i->second.y << "\""
           << " shape = \"square\"/>" << endl;
    }
  // Ouput links
  for (unsigned i = 0; i < links.size(); ++i)
    {
      ofs << "<link fromLp = \"" << links[i].lp1 << "\""
          << " fromId = \"" << links[i].n1 << "\""
          << " toLp = \"" << links[i].lp2 << "\""
          << " toId = \"" << links[i].n2 << "\"/>" << endl;
    }
  // Close the topology tag
  ofs << "</topology>" << endl;
  // Output the packets
  for (PktMap_t::iterator i = packets.begin();
       i != packets.end(); ++i)
    {
      ofs << "<packet fromLp = \"0\" fromId = \"" << i->second.txId << "\""
          << " fbTx = \"" << i->second.fbTx << "\""
          << " lbTx = \"" << i->second.lbTx << "\">" << endl;
      // And output the rx (just one for wired
      ofs << "<rx toLp = \"0\" toId = \"" << i->second.rxId << "\""
          << " fbRx = \"" << i->second.fbRx << "\""
          << " lbRx = \"" << i->second.lbRx << "\"/>" << endl;
      ofs << "</packet>" << endl;
    }
  ofs << "</anim>" << endl;
  ofs.close();
}

