/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */

// Test program to parse the netanim xml files
// George F. Riley, Georgia Tech, Spring 2010

#include <iostream>
#include <fstream>
#include <string>
#include <map>

#include <stdio.h>

#include "html/ParserDom.h"
#include "html/ParserSax.h"
#include "html/utils.h"
#include "html/wincstring.h"

using namespace std;
using namespace htmlcxx;

void ToLower(string& st)
{
  for (string::size_type i = 0; i < st.length(); ++i)
    {
      st[i] = tolower(st[i]);
    }
}

// typedef for attributes lists
typedef map<string, string>    AttributeList_t;
typedef AttributeList_t::const_iterator AttributeIterator_t;

void PrintAttributes(const AttributeList_t& at)
{
  for (AttributeIterator_t i = at.begin(); i != at.end(); ++i)
    {
      cout << "attr " << i->first << " is " << i->second << endl;
    }
}

class MyParser : public HTML::ParserSax
{
public:
  MyParser() {}
  virtual ~MyParser() {}

protected:
  virtual void beginParsing();
  virtual void endParsing();
  virtual void foundTag(HTML::Node& node, bool isEnd);
  virtual void foundText(HTML::Node& node);
};

  
void MyParser::beginParsing()
{
  cout << "begin parsing" << endl;
}

void MyParser::endParsing()
{
  cout << "end parsing" << endl;
}

void MyParser::foundTag(HTML::Node& node, bool isEnd)
  {
    string element = node.tagName();
    ToLower(element);
    if (isEnd)
      {
        cout << "End Element " << element << endl;
      }
    else
      { // Start of element
        cout << "Start Element " << element << endl;
        node.parseAttributes();
        PrintAttributes(node.attributes());
      }
  }

void MyParser::foundText(HTML::Node& node)
{
  //cout << "Found text " << node.text() << endl;
}


int main(int argc, char** argv)
{
  if (argc < 2) exit(1);
  ifstream file(argv[1]);
  if (!file) exit(1);
  string html;
  static char buf[BUFSIZ];
  while (1)
    {
      file.read(buf, BUFSIZ);
      if(file.gcount() == 0) {
        break;
      }
      html.append(buf, file.gcount());
    }
  file.close();

  MyParser p;
  try {
    p.parse(html);
  } catch (exception &e) {
    cerr << "Exception " << e.what() << " caught" << endl;
    exit(1);
  } catch (...) {
    cerr << "Unknow exception caught " << endl;
  }
}

